import QtQuick 2.12
import Components 1.0

Item {
    id: root

    property real minValue: 0
    property real maxValue: 100
    property real startValue: 0
    property string startDisplayValue: startValue
    property real finishValue: maxValue
    property string finishDisplayValue: finishValue
    property variant valueProxy // declarative binding for value
    property int precision: 0
    property bool clickableTrack: true
    property real stepValue: 1
    property int _minpos: 0
    property int _maxpos: (orientation == Qt.Horizontal) ? trackWidth: track.height
    property int trackWidth: 1000
    property bool showValue: true
    property string startLabel: qsTr("START") + application.emptyString
    property string finishLabel: qsTr("FINISH") + application.emptyString
    property int valueOffsetX: 0
    property int valueOffsetY: 0
    property int buttonOffsetX: 0
    property int buttonOffsetY: 0
    property int buttonMargin: 20
    property int orientation: Qt.Horizontal
    property real thumbMouseAreaScale: 1.0
    property real buttonAreaScale: 2.0
    property int dragAreaMargin: 0
    property bool rightToLeft: false //main.rightToLeft
    property ListModel indexPoints
    property color fillLineColor: main.colorFontLight
    property color trackLineColor: main.colorFontLight
    property color textColor: main.colorFontLight
    property bool showLabels: true
    property int labelOverlapDistance: (leftSliderKnob.x + -(labelKnobLeft.width - leftSliderKnob.width)/2 + labelKnobLeft.width) - (rightSliderKnob.x + -(labelKnobRight.width - rightSliderKnob.width)/2)

    signal released()

    width: trackWidth
    height: 150 * main.vScaleFactor
    enabled: visible && opacity > 0

    onStartValueChanged: startValue = setValue(startValue)
    onFinishValueChanged: finishValue = setValue(finishValue)

    onValueProxyChanged: {
        if (leftSliderKnob.selected)
            startValue = setValue(valueProxy.x);
        else
            finishValue = setValue(valueProxy.y);
    }

    Component.onCompleted: {
        root.startValue = validateRangeStartValue(root.startValue)
        root.finishValue = validateRangeFinishValue(root.finishValue)
    }

    function setValue(newValue) {
        newValue = (newValue / stepValue).toFixed(0) * stepValue
        newValue = validateRange(newValue)
        return newValue
    }

    function validateRangeStartValue(newValue) {
        if (newValue > root.finishValue) {
            newValue = root.finishValue - 1
        }
        newValue = validateRange(newValue)
        return newValue
    }

    function validateRangeFinishValue(newValue) {
        if (newValue < root.startValue) {
            newValue = root.startValue + 1
        }
        newValue = validateRange(newValue)
        return newValue
    }


    function validateRange(newValue) {
        if( newValue < root.minValue )
            newValue = root.minValue
        if( newValue > root.maxValue )
            newValue = root.maxValue
        return newValue
    }

    function positionFromValue(value) {
        var pos = (value-minValue) * (_maxpos / (maxValue - minValue))
        if (isNaN(pos)) {
            pos = 0
        }
        return rightToLeft ? _maxpos-pos : pos
    }

    function valueFromPosition(position) {
        if (rightToLeft)
            position = _maxpos - position
        var newValue = minValue + position *(maxValue-minValue)/(_maxpos-_minpos)
        newValue = validateRange(newValue)
        return newValue.toFixed(precision)
    }

    Item {
        id: track
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        width: root.width
        height: root.height

        Rectangle {
            id: fillLine
            z: 2 // Place below thumb grips, but on top of track
            color: root.fillLineColor
            height: 10 * main.vScaleFactor
            anchors.verticalCenter: parent.verticalCenter
            x: leftSliderKnob.x + leftSliderKnob.width/2
            width: rightSliderKnob.x - x + rightSliderKnob.width/2
        }

        Repeater {
            id: indexLines
            anchors {
                left: track.left
                bottom: track_line.bottom
            }

            model: root.indexPoints
            delegate: Component {
                    Item {
                        id: indexItem
                        TextLabel {
                            id: labelDisplay
                            font.pixelSize: 14 * main.vScaleFactor
                            text: model.label
                            horizontalAlignment: Text.AlignRight
                            anchors {
                                right: index === 0 ? undefined : line.right
                                rightMargin: index === 0 ? 0 : (model.offset ? -model.offset : 0)
                                // First item is a special case, being anchored on left
                                left: index === 0 ? line.left : undefined
                                leftMargin: index === 0 ? (model.offset ? model.offset : 0) : 0
                                bottom: valueDisplay.top
                                bottomMargin: 2 * main.vScaleFactor
                            }
                        }
                        NumberLabel {
                            id: valueDisplay
                            color: root.textColor
                            font.pixelSize: 14 * main.vScaleFactor
                            text: options.metricUnits ? Globals.formatWithSeparator(Globals.feetToMeters(model.value).toFixed(0)) : Globals.formatWithSeparator(model.value)
                            anchors {
                                right: index === 0 ? undefined : labelDisplay.right
                                // First item is a special case, being anchored on left
                                left: index === 0 ? labelDisplay.left : undefined
                                bottom: line.top
                                bottomMargin: 5 * main.vScaleFactor
                            }
                        }

                        Rectangle {
                            id: line
                            color: root.textColor
                            height: 0.35*root.height
                            width: 2
                            y: (root.height-height)/2
                            anchors.left: parent.left
                            anchors.leftMargin: root.positionFromValue(model.value) - 1 // Subtract half of width for centering
                        }
                    }
            }
        }

        Rectangle {
            id: track_line
            height: 2 * main.vScaleFactor
            width: parent.width
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            color: root.trackLineColor
        }

        // mouse area for clicking the track to move the slider
        MouseArea {
            id: track_mouse

            enabled: clickableTrack
            anchors.fill: parent
            onPressed: {
                // TODO If position is after finish, move forward
                // If position is before start, move backward
                // If position is inbetween, ???
//                var position = (root.orientation == Qt.Horizontal ? (_minpos + mouseX ) : (_maxpos -_minpos - mouseY))
//                value = valueFromPosition( position )
//                buzzer.buttonPress()
            }
            Rectangle{
                opacity: 0.3
                color: main.colorTrackMouseStimulusArea
                anchors.fill: parent
                visible: main.showStimulusArea
            }
        }

        Rectangle {
            id: leftSliderKnob

            property bool selected: true

            width: 50 * main.hScaleFactor
            height: 40 * main.vScaleFactor
            radius: main.defaultRadius
            border.color: main.colorUIBorder
            border.width: main.widthBorder
            color: selected ? main.colorSelected : main.colorDialogBackground

            anchors.left: parent.left
            anchors.leftMargin: positionFromValue(startValue) - width/2
            anchors.bottomMargin: positionFromValue(startValue) - height/2
            z: 99

            Text {
                id: labelKnobLeft
                font.bold: false
                font.pixelSize: 16 * main.vScaleFactor
                horizontalAlignment: Text.AlignHCenter
                lineHeight: 0.7
                color: root.textColor
                text: root.startLabel + "\n" + root.startDisplayValue
                anchors {
                    top: parent.bottom
                    topMargin: 10 * main.vScaleFactor
                }
                x: -(labelKnobLeft.width - leftSliderKnob.width)/2 - (labelOverlapDistance > 0 ? labelOverlapDistance/2 + 5 : 0)
            }

            MouseArea {
                id: knobMouseArea
                property int knobMargin: (root.orientation == Qt.Horizontal ? (leftSliderKnob.width/2) : (leftSliderKnob.height/2))

                anchors.fill: parent
                anchors.margins: root.dragAreaMargin
                scale: root.thumbMouseAreaScale

                drag {
                    target: leftSliderKnob ;
                    axis: (root.orientation === Qt.Horizontal) ? Drag.XAxis : Drag.YAxis;
                    minimumX: _minpos - knobMargin ; minimumY: _minpos - knobMargin
                    //maximumX: _maxpos - knobMargin + 5  ; maximumY: _maxpos - knobMargin + 5    // +5 fudge factor need for some reason to allow full sliding to max value
                    maximumX: rightSliderKnob.x - rightSliderKnob.width/2 - 15; maximumY: rightSliderKnob.y
                }

                onPressed: {
                    leftSliderKnob.anchors.left = undefined
                    leftSliderKnob.anchors.bottom = undefined
                    leftSliderKnob.selected = true
                    rightSliderKnob.selected = false
                    buzzer.buttonPress()
                } // un-anchor to allow drag
                onReleased: {
                    if(root.orientation == Qt.Horizontal)
                        leftSliderKnob.anchors.left = leftSliderKnob.parent.left
                    else
                        leftSliderKnob.anchors.bottom = leftSliderKnob.parent.bottom
                    root.released()
                } // re-anchor at end of drag

                onMouseXChanged: {
                    if( root.orientation == Qt.Horizontal )
                        root.startValue = setValue(valueFromPosition(leftSliderKnob.x + leftSliderKnob.width/2))
                }
                onMouseYChanged: {
                    if( root.orientation != Qt.Horizontal )
                        setValue(valueFromPosition(_maxpos - _minpos - leftSliderKnob.y - leftSliderKnob.height/2))
                }

                Rectangle{
                    opacity: 0.3
                    color: main.colorKnobStimulusArea
                    anchors.fill: parent
                    visible: main.showStimulusArea
                }
            }
        }

        Rectangle {
            id: rightSliderKnob

            property bool selected: false

            width: 50 * main.hScaleFactor
            height: 40 * main.vScaleFactor
            radius: main.defaultRadius
            border.color: main.colorUIBorder
            border.width: main.widthBorder
            color: selected ? main.colorSelected : main.colorDialogBackground

            anchors.left: parent.left
            anchors.leftMargin: positionFromValue(finishValue) - width/2
            anchors.bottomMargin: positionFromValue(finishValue) - height/2
            z: leftSliderKnob.z + 1

            Text {
                id: labelKnobRight
                font.bold: false
                font.pixelSize: 16 * main.vScaleFactor
                text: root.finishLabel + "\n" + root.finishDisplayValue
                horizontalAlignment: Text.AlignHCenter
                lineHeight: 0.7
                color: root.textColor
                anchors {
                    top: parent.bottom
                    topMargin: 10 * main.vScaleFactor

                }
                x: -(labelKnobRight.width - rightSliderKnob.width)/2 + (labelOverlapDistance > 0 ? labelOverlapDistance/2 + 5 : 0)
            }

            MouseArea {
                id: knobMouseAreaRight
                property int knobMargin: (root.orientation == Qt.Horizontal ? (rightSliderKnob.width/2) : (rightSliderKnob.height/2))

                anchors.fill: parent
                anchors.margins: root.dragAreaMargin
                scale: root.thumbMouseAreaScale

                drag {
                    target: rightSliderKnob ;
                    axis: (root.orientation === Qt.Horizontal) ? Drag.XAxis : Drag.YAxis;
                    minimumX: leftSliderKnob.x + leftSliderKnob.width/2 + 15; minimumY: _minpos - knobMargin
                    maximumX: _maxpos - rightSliderKnob.width/2 ; maximumY: rightSliderKnob.y
                }

                onPressed: {
                    rightSliderKnob.anchors.left = undefined
                    rightSliderKnob.anchors.bottom = undefined
                    leftSliderKnob.selected = false
                    rightSliderKnob.selected = true
                    buzzer.buttonPress()
                } // un-anchor to allow drag
                onReleased: {
                    if(root.orientation == Qt.Horizontal)
                        rightSliderKnob.anchors.left = rightSliderKnob.parent.left
                    else
                        rightSliderKnob.anchors.bottom = rightSliderKnob.parent.bottom
                    root.released()
                } // re-anchor at end of drag

                onMouseXChanged: {
                    if( root.orientation == Qt.Horizontal )
                        root.finishValue = setValue(valueFromPosition(rightSliderKnob.x + rightSliderKnob.width/2))
                }
                onMouseYChanged: {
                    if( root.orientation != Qt.Horizontal )
                        setValue(valueFromPosition(_maxpos - _minpos - rightSliderKnob.y - rightSliderKnob.height/2))
                }

                Rectangle{
                    opacity: 0.3
                    color: main.colorKnobStimulusArea
                    anchors.fill: parent
                    visible: main.showStimulusArea
                }
            }
        }
    }

    states: [
        State {
            name: "Horizontal"
            when: orientation == Qt.Horizontal

            PropertyChanges {
                target: track
                width: root.trackWidth
                height: root.height
            }

            AnchorChanges {
                target: track
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
            }

            PropertyChanges {
                target: track_line
                height: 3
                width: parent.width
            }

            AnchorChanges {
                target: leftSliderKnob
                anchors { verticalCenter: parent.verticalCenter ; left: parent.left }
            }
            AnchorChanges {
                target: rightSliderKnob
                anchors { verticalCenter: parent.verticalCenter ; left: parent.left }
            }
        },
        State {
            name: "Vertical"
            when: orientation == Qt.Vertical

            PropertyChanges {
                target: track
                height: root.height
                width: root.width
            }

            AnchorChanges {
                target: track
                anchors {
                    left: undefined
                    right: undefined
                    top: root.top
                    bottom: root.bottom
                    horizontalCenter: root.horizontalCenter
                    verticalCenter: undefined
                }
            }

            PropertyChanges {
                target: track_line
                height: track.height;
                width: 3
                opacity: 0.5
            }

            AnchorChanges {
                target: leftSliderKnob
                anchors { verticalCenter: undefined ; left: undefined ; bottom: parent.bottom ; horizontalCenter: parent.horizontalCenter }
            }
        }
    ]

}
