
import QtQuick 2.12
import Process 0.1
import "."
import "../Television"
import ".."
import FileIO 1.0
import Components 1.0

Item {
    id: root
    property bool isUsbDeviceLoaded: storageMgr.usbDeviceLoaded
    property bool isSetupWizard: false
    property bool isActive: (main.state === "DiagnosticState" && diagnosticsMain.currentSelectedIndex >= 0) || root.isSetupWizard
    property string cursorState: "setup" // setup or next
    property bool refreshing: false

    anchors.fill: parent
    Component.onCompleted: netConnections.refreshList()

    function updateChannelList() {

        if (sapManager.item.channelList.count <= 0) {
            console.log("no SAP channels found. making no changes to TV list")
            return
        }

        var found = {}
        // Update all existing channels
        for (var i = 0; i < sapManager.item.channelList.count; i++) {
            var channelItem = sapManager.item.channelList.get(i)

            // SAP items are keyed by name
            // If item exists, just update it
            var id = "sap-" + channelItem.name
            // Store for later
            found[id] = channelItem

            var item = {}
            var index = tv.channelList.findChannel(id)
            if (index >= 0) {
                item = tv.channelList.channels.get(index)
            }

            item.id = id
            item.source = "sap"
            item.mrl = channelItem.mrl
            item.category = channelItem.category
            item.name = channelItem.name

            tv.channelList.addChannel(item)
        }

        // Now delete all channels that were not in this batch
        for (var i = tv.channelList.channels.count - 1; i >= 0; i--) {
            var chanItem = tv.channelList.channels.get(i)
            if (chanItem.source === "sap" && found[chanItem.id] === undefined) {
                tv.channelList.removeChannels(chanItem.id)
            }
        }

        // Finally, re-sort list by category
        tv.channelList.sortByCategory()
    }

    TextLabel {
        id: title
        text:  qsTr("IPTV SETUP")
        color: main.colorFontDark
        anchors.top: parent.top
        anchors.topMargin: 30 * main.vScaleFactor
        anchors.horizontalCenter: parent.horizontalCenter
        font.pixelSize: 24 * main.vScaleFactor
    }

    ChannelListBox {
        id: channelListBox

        visible: options.iptvEnabled
        width: 0.7 * root.width
        height: 0.65 * root.height
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.horizontalCenterOffset: -rightControlArea.width/2
        anchors.top: rightControlArea.top

        Text {
            id: refreshText
            text: qsTr("Refreshing") + "..." + application.emptyString
            font.pixelSize: 18* main.vScaleFactor
            anchors { left: parent.left; top: parent.top; margins: 20 * main.scaleFactor}
            visible: autoRefreshTimer.running
        }
    }

    Rectangle {
        id: rightControlArea
        visible: options.iptvEnabled

        width: 0.25 * root.width
        height: channelListBox.height
        color: "#7f8082"
        anchors { left: channelListBox.right; top: parent.top; topMargin: 198 * main.vScaleFactor }
    }

    WifiBrowser {
        id: netBrowser
        hideWireless: true
        visible: false
        z: 999999999

        onCanceled: visible = false
        anchors { horizontalCenter: parent.horizontalCenter ; verticalCenter: parent.verticalCenter }
        onSelected: {
            visible = false
            var args = []
            if (options.iptvEnabled) {
                args.push("-b")
            }

            args = args.concat([accessPoint.interface, accessPoint.essid, accessPoint.password])
            configureNetworkProcess.arguments = args

            // IPTV allows multiple connections
            // So we don't need to prompt the user
            configureNetworkProcess.start()

        }
    }

    MessagePopup {
        id: reconfiguringNotice

        message: qsTr("Reconfiguring Network Settings...") + application.emptyString
    }

    Process {
        id: configureNetworkProcess

        onStarted: { reconfiguringNotice.show() }
        onFinished: {
            netConnections.refreshList()
            reconfiguringNotice.hide()
            dataExport.getNetworkAddresses()
            sapLoader.reload()
        }

        program: "scripts/configure_network.sh"
        arguments: ["enp2s0"]
    }

    Text {
        id: channelsFoundText
        text: {
            if (root.refreshing || refreshTimer.running)
                return ""
            else if (tv.channelList.channels.count > 0)
                return qsTr("Number of Channels:") + " " + tv.channelList.channels.count + application.emptyString
            else
                return qsTr("No Channels Configured") + application.emptyString
        }
        anchors.bottom: channelListBox.top
        anchors.bottomMargin: 7* main.vScaleFactor
        anchors.left: channelListBox.left
        font { pixelSize: 18* main.vScaleFactor; bold: true }
    }

    Column {
        id: controlList

        anchors {
            top: rightControlArea.top;
            topMargin: spacing
            left: rightControlArea.left
            leftMargin: 0.05 * width
        }
        width: rightControlArea.width
        height: rightControlArea.height
        spacing: 20 * main.vScaleFactor
        z: 100

        StandardButton {
            id: refreshButton

            enabled: netConnections.wiredEnabled
            buttonText: qsTr("Refresh List") + application.emptyString
            width: 0.9 * parent.width
            height: 51 * main.vScaleFactor
            label.font.pixelSize: 12 * main.vScaleFactor

            onClicked:{
                tv.channelList.clearChannels()
                root.refreshing = true
                refreshTimer.restart()
                if (sapManager && sapManager.item) {
                    sapManager.item.refreshSources()
                } else {
                    console.log("cannot refresh: sapManager not created yet")
                }

                //console.log("finished refresh", JSON.stringify(sapManager.channelList))
                root.updateChannelList()
                root.refreshing = false
                tv.channelList.doSave()
                tv.channelList.readChannels(tv.channelFile)
            }
        }

        // Disable IPTV button
        StandardButton {
            id: disableButton

            enabled: true
            buttonText: qsTr("Disable IPTV") + application.emptyString
            width: 0.9 * parent.width
            height: 51 * main.vScaleFactor
            label.font.pixelSize: 12 * main.vScaleFactor

            onClicked:
            {
                options.iptvEnabled = false
                tv.channelList.clearChannels()
            }
        }

        Item {
            id: configureNetworkItem
            height: 105 * main.vScaleFactor
            width: 0.9 * parent.width

            StandardButton {
                id: configNetButton

                enabled: !netConnections.wireDisconnected

                height: 0.5 * parent.height
                width: parent.width
                anchors.top: parent.top
                anchors.left: parent.left

                buttonText: qsTr("Configure Network") + application.emptyString

                onClicked: {
                    netBrowser.visible = true
                }
            }

            Row {
                id: netConfigRow
                anchors.top: configNetButton.bottom
                anchors.left: configNetButton.left
                height: 0.5 * parent.height
                width: parent.width
                spacing: 10 * main.hScaleFactor

                ScalableImage {
                    id: warningIcon
                    visible: !netConnections.wiredConfigured || netConnections.wireDisconnected || !netConnections.wiredEnabled

                    sourceName: "Icon_Warning_Small";
                }

                Text {
                    id: netConfig
                    color: warningIcon.visible ? "#c0c0c0" : main.colorFontDark
                    font { pixelSize: 14 * main.vScaleFactor }
                    text: {
                        if (!netConnections.wiredConfigured) {
                            return qsTr("Not Configured") + application.emptyString
                        }  else if (netConnections.wireDisconnected) {
                            return qsTr("Wire Disconnected") + application.emptyString
                        } else if (!netConnections.wiredEnabled) {
                            return qsTr("Not Connected") + application.emptyString
                        } else {
                            return qsTr("Connected: Wired Ethernet") + application.emptyString
                        }
                    }
                }
            }

        }
    }

    MessagePopup {
        id: autofindPopup
        message: qsTr("Looking for channels...") + application.emptyString
        visible: autoRefreshTimer.running
        dismissable: true
    }

    Timer {
        id: autoRefreshTimer
        interval: 5000
        running: tv.channelList.channels.count === 0 && netConnections.wiredConfigured && !netConnections.wireDisconnected
        repeat: true
        onTriggered: {
            refreshButton.clicked()
        }
    }

    Timer {
        // Display loading image for at least 1 second
        id: refreshTimer
        interval: 1000
        repeat: false; running: false
    }

}
