import QtQuick 2.12
import Process 0.1
import FileIO 1.0
import Components 1.0

import ".."
import "../Television"
import "../Keypad"
import "../Networking"

Item{
    id: root
    anchors.fill: parent
    anchors.leftMargin: 25 * main.hScaleFactor

    property bool debug: false
    property bool iptvPopupHasBeenShown: false

    property bool isUsbDeviceLoaded: storageMgr.usbDeviceLoaded
    property string wifiAdapter: (tv.proIdiomPresent || tv.tunerType === "H1265") ? "wlp6s0" : "wlp5s0"
    onIsUsbDeviceLoadedChanged: print("USB DEVICE: " + isUsbDeviceLoaded)

    function filterCountries(tuner) {
        if (tuner === "DVB-C") {
            tuner = "DVB"
        }
        var newModel = country_codes

        if (tuner === "MGT") {
            // Show all for MGT
        } else if (tuner === "H975") {
            // Show all except DMB and ISDB for H975
            for( var i = country_codes.count-1 ; i >= 0 ; i--) {
                if ( newModel.get(i).tuner == "DMB" || newModel.get(i).tuner == "ISDB") {
                    newModel.remove(i)
                }
            }
        } else {
            for( var i = country_codes.count-1 ; i >= 0 ; i--) {
                if ( newModel.get(i).tuner !== tuner ) {
                    newModel.remove(i)
                }
            }
        }
        countryListPopup.model = newModel
    }

    function  maybeShowWifiConfig() {
        if (!netConnections.wiredConfigured && !netConnections.wirelessConfigured) {
            wifiBrowser.visible = true
        }
    }

    function debugLog(string) {
        if(root.debug)
            print("network: " + string)
    }

    LayoutMirroring.enabled: main.rightToLeft
    LayoutMirroring.childrenInherit: true

    Component.onCompleted: {
        netConnections.refreshList()
        filterCountries(tv.tunerType)
        homeUrlEdit.text = options.homeURL
    }

    Process{
        id: ifconfigProcess
        program: "ifconfig"
        arguments: ["enp2s0"]

        onReadyReadStandardOutput: {
            var output = new String(readAllStandardOutput())
            debugLog( "output: " + output)
            parseString(output, "enp2s0")
        }

        onReadyReadStandardError: {
            var output = new String(readAllStandardError())
            debugLog( "ifconfig error output: " + output)
        }

        onFinished: {
            if (!root.success) {
                ifconfigProcessWireless.start()
            }
        }
    }

    Process {
        id: iwconfigProcess
        program: "iwconfig"
        arguments: root.wifiAdapter

        onReadyReadStandardOutput: {
            var output = new String(readAllStandardOutput())
            debugLog( "iwconfig output: " + output)
            if(output.match(/root.wifiAdapter/)) {
                var essidSearch = output.match(/ESSID:"(.+?)"\s/)
                root.currentESSID = essidSearch ? essidSearch[1] : "not connected"
            }
        }
    }

    Process {
        id: configureNetworkProcess

        onStarted: { reconfiguringNotice.show() }
        onFinished: {
            netConnections.refreshList()
            netConnectionList.refresh()
            dataExport.getNetworkAddresses()
            reconfiguringNotice.hide()
            sapLoader.reload()
        }

        program: "scripts/configure_network.sh"
        arguments: ["enp2s0"]
    }

    NetworkConnectionList {
        id: netConnectionList
        preventEthDisconnect: root.iptvPopupHasBeenShown
        anchors {
            top: parent.top
            left: parent.left
            topMargin: 20 * main.vScaleFactor
        }

        onDisconnectRequest: {
            console.log("Disconnect from", interfaceName)
            if (interfaceName === "enp2s0" && options.iptvEnabled) {
                cannotRemovePopup.show()
                cannotRemovePopup.startTimeout(2)
                // Now disable the X button
                root.iptvPopupHasBeenShown = true
                return
            }

            // Prevent unclosed SSH connections
            if (netConnections.list.count <= 1) {
                remoteAccess.isEnabled = false
            }

            disconnectPopup._show()
            disconnectNetwork.arguments = [interfaceName]
            disconnectNetwork.start()
        }
    }

    MessagePopup {
        id: cannotRemovePopup
        message: qsTr("This connection is in use by IPTV. Please disable IPTV feature to remove this connection.")
        visible: false
    }

    MessagePopup {
        id: disconnectPopup
        message: qsTr("Disconnecting...") + application.emptyString
        property int minDisplayTime: 2000
        property bool done: false

        Timer {
            id: minDisplayTimer
            interval: disconnectPopup.minDisplayTime
            repeat: false
            running: false
            onTriggered: {
                if (disconnectPopup.done) {
                    disconnectPopup.hide()
                }
            }
        }

        function _show() {
            done = false
            minDisplayTimer.restart()
            show()
        }

        function _hide() {
            done = true
            if (minDisplayTimer.running) {
                return
            }
            disconnectPopup.hide()
        }
    }

    Process {
        id: disconnectNetwork
        program: "scripts/disconnect_network.sh"
        arguments: ["enp2s0"]
        onFinished: {
            disconnectPopup._hide()
            netConnections.refreshList()
            netConnectionList.refresh()
        }
    }

    StandardButton {
        id: interfaceButton

        width: 150 * main.hScaleFactor
        height: 73 * main.vScaleFactor
        anchors.right: parent.right
        anchors.rightMargin: 20 * main.hScaleFactor
        anchors.top: parent.top
        anchors.topMargin: 20 * main.vScaleFactor

        buttonText: qsTr("setup connection") + application.emptyString
        onClicked: {
            wifiBrowser.visible = true
        }
    }

    /*Grid {
        id: labels

        columns: 2
        spacing: itemSpacing
        anchors {
            left: parent.left
            leftMargin: 200
            right: parent.right
            rightMargin: 200
        }
        anchors {
            verticalCenter: parent.verticalCenter
            verticalCenterOffset: topOffset
        }

        Row {
            spacing: 170

        }

        // Bluetooth MAC
//        Text{
//            text: qsTr("Bluetooth Address") + application.emptyString
//            font.pointSize: fontSize
//        }
//        Text{
//            text: bluetoothAdapter.address ? bluetoothAdapter.address : qsTr("Not Installed") + application.emptyString
//            font.pointSize: fontSize
//        }
    }*/

    Checkbox {
        id: browserEnableRow
        label: qsTr("Web Browser Enabled") + application.emptyString
        width: netConnectionList.width
        value: options.browserEnabled
        anchors {
            top: netConnectionList.bottom
            topMargin: 30 * main.vScaleFactor
            left: netConnectionList.left
        }
        onValueChanged: {
            options.browserEnabled = value ? true : false
        }
    }

    MouseArea {
        id: webEnableMouseArea

        anchors.fill: browserEnableRow
        onClicked: {
            buzzer.buttonPress()
            if (browserEnableRow.value === true)
                browserEnableRow.value = false
            else
                webTermsPrompt.show()
        }
        Rectangle {
            anchors.fill: parent
            color: main.colorGenericStimulusArea
            opacity: 0.3
            visible: main.showStimulusArea
        }
    }

    Row {
        id: homeUrlRow

        anchors {
            top: browserEnableRow.bottom
            topMargin: 10 * main.vScaleFactor
            left: netConnectionList.left
            right: parent.right
            rightMargin: 20 * main.hScaleFactor
        }

        spacing: 5 * main.hScaleFactor
        visible: options.browserEnabled

        TextLabel {
            id: urlLabel

            text: qsTr("Homepage:")
            width: paintedWidth + 10 * main.hScaleFactor
            height: paintedHeight
            color: main.colorFontDark
        }

        TextInput {
            id: homeUrlEdit

            width: parent.width - urlLabel.width - 90 * main.hScaleFactor
            autoScroll: false
            clip: true
            inputMethodHints: Qt.ImhUrlCharactersOnly
            onEditingFinished: options.homeURL = homeUrlEdit.text

            Rectangle {
                color: "transparent"
                border.color: main.colorDiagnosticEdges
                anchors.fill: parent
                z: parent.z-1
            }
        }
    }

    Row {
        id: regionRow
        visible: options.browserEnabled
        anchors {
            top: homeUrlRow.bottom;
            topMargin: 30 * main.vScaleFactor
            left: netConnectionList.left
            right: parent.right
            rightMargin: 20 * main.hScaleFactor
        }
        spacing: 40 * main.hScaleFactor

        TextLabel {
            id: linksLabel
            text: qsTr("Region") + ":" + application.emptyString
            color: main.colorFontDark
        }

        Rectangle {
            color: "#cccccc"

            radius: main.defaultRadius
            width: 200 * main.hScaleFactor
            height: 30 * main.vScaleFactor

            Text {
                id: countryValue
                text: countryListPopup.currentIndex >= 0 ? qsTranslate("TelevisionRegionList", countryListPopup.selectedData.label) + application.emptyString : qsTranslate("TelevisionRegionList", "Select Country")
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter

                font.bold: false
            }
            MouseArea {
                anchors.fill: parent
                onClicked: {
                    countryListPopup.visible = true
                    buzzer.buttonPress()
                }
                Rectangle {
                    anchors.fill: parent
                    color: main.colorGenericStimulusArea
                    opacity: 0.3
                    visible: main.showStimulusArea
                }
            }
        }
    }

    CountryListPopup {
        id: countryListPopup

        function saveSelection(code) {
            options.browserCountryCode = code
        }

        onModelChanged: {
            if (!model)
                return

            // Initialize to current value
            for (var i = 0; i < model.count; i++) {
                if (model.get(i).code === options.browserCountryCode) {
                    currentIndex = i
                    return
                }
            }
            currentIndex = -1
        }

        anchors {
            left: parent.left
            top: parent.top
            topMargin: 20 * main.vScaleFactor
        }
        model: country_codes
        modelTranslationContext: "TelevisionRegionList"
        onCurrentIndexChanged: {
            if (!countryListPopup.visible) {
                // Avoid responding to programmatic changes to current index
                return
            }

            if (countryListPopup.selectedData) {
                saveSelection(countryListPopup.selectedData.code)
            }
        }
        onItemSelected: {
            var code = countryListPopup.model.get(index).code
            saveSelection(code)
            close()
        }
        onClose: {
        }
    }

    TelevisionRegionList {
        id: country_codes
    }

    MessagePopup {
        id: reconfiguringNotice

        message: qsTr("Reconfiguring Network Settings...") + application.emptyString
    }

    WifiBrowser {
        id: wifiBrowser

        visible: false
        onSelected: {
            visible = false
            var args = []
            if (options.iptvEnabled) {
                args.push("-b")
            }
            args = args.concat([accessPoint.interface, accessPoint.essid, accessPoint.password])

            configureNetworkProcess.arguments = args

            if (options.iptvEnabled) {
                // IPTV allows multiple connections
                // So we don't need to prompt the user
                configureNetworkProcess.start()
            } else {
                if (netConnections.wiredConfigured || netConnections.wirelessConfigured) {
                    // They already have a connection. Check if they want to disconnect first
                    disconnectPrompt.show()
                } else {
                    // Add new connection
                    configureNetworkProcess.start()
                }
            }

            return

        }
        onCanceled: visible = false
        anchors {
            horizontalCenter: parent.horizontalCenter
            verticalCenter: parent.verticalCenter
        }
    }

    ConfirmationPrompt {
        id: disconnectPrompt
        property string network: netConnections.wiredConfigured ? "Wired Ethernet" : netConnections.currentESSID

        visible: false
        prompt: qsTr("This will disconnect you from ") + network + ". " + qsTr("Are you sure?") + application.emptyString
        onNoClicked: {
            configureNetworkProcess.arguments = []
            hide()
        }
        onYesClicked: {
            // Prevent unclosed SSH connections
            remoteAccess.isEnabled = false
            configureNetworkProcess.start()
            hide()
        }
    }

    PopupWebAgreement {
        id: webTermsPrompt

        Component.onCompleted: hide()

        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter

        onYesClicked: {
            hide()
            browserEnableRow.value = true
            maybeShowWifiConfig()
        }

        onNoClicked: {
            hide()
            browserEnableRow.value = false
        }
    }

    TrueKeyboard {
        id: keyboard
        width: root.width
        anchors.horizontalCenter: root.horizontalCenter
        z: 999999
        active: homeUrlEdit.activeFocus
        useMain: false
    }
}
