import QtQuick 2.12
import ".."
import Components 1.0
import TrueTouch 1.0

Item {
    id: root

    anchors {
        fill: parent
        margins: 10  * main.hScaleFactor
    }

    state: main.isCSA ? "EllipticalStride" : machineinfo.productTypeString

    property int rpm
    property int watts
    property int contactHR
    property int telemetryHR
    property int fanSpeed
    property int buttonNumber
    property int strideAdc
    property int striderpm


    onWattsChanged: wattDisplay.value = watts
    onRpmChanged: rpmDisplay.value = rpm
    onContactHRChanged: contactHRDisplay.value = contactHR
    onTelemetryHRChanged: telemetryHRDisplay.value = telemetryHR
    onButtonNumberChanged: keypressDisplay.value = buttonNumber
    onStrideAdcChanged: strideAdcDisplay.value = strideAdc
    onStriderpmChanged: strideRpmDisplay.value = striderpm

    Connections {
        target: nucleus
        /* keyFunc is the button function */
        onKeypadButtonPressed: keypressDisplay.value = keyFunc
        onKeypadButtonReleased: keypressDisplay.value = 0
    }

    Connections {
        target: heartrate
        onTelemetryHeartRateChanged: telemetryHR = telemetryHeartRate
        onContactHeartRateChanged: contactHR = contactHeartRate
    }

    Connections {
        target: machinecontrol
        onEstopChanged: {
            estopDisplay.value = estop
            // stop change diagnostic speed control to zero target when safety key opened
            if(!estop)
                speedControl.value = 0
        }
        onPedalRPMChanged: {root.rpm = pedalRPM; root.striderpm = pedalRPM}
        onStrideLengthAdcActualChanged: strideAdc = machinecontrol.strideLengthAdcActual
    }
    Component.onCompleted: {
        if(main.isTreadmill || main.isStepmill) {
            if(machineinfo.systemConfig === "NUCLEUS") {
                nucleus.getStaticBrakeState()

            }
        }
        autoChannelChangeControl.value = options.autoChannelChangeInterval
        autoVideoChangeControl.value = options.autoVideoChangeInterval/1000
        workoutAutoRestartIntervalControl.value = options.workoutAutoRestartInterval
    }

    Component.onDestruction: {
        machinecontrol.treadmillSpeedTarget = 0
        /* When exiting from this screen, engage the static brake if measured
         * speed is 0
         */
        if(machinecontrol.treadmillSpeedActual === 0) {
            machinecontrol.staticBrakeEngage()
        }
    }

    Flickable {
        id: scrollColumn
        clip: true
        flickableDirection: Flickable.VerticalFlick

        anchors.fill: parent

        contentHeight: controlList.height

        Column {
            id: controlList
            spacing: 5 * main.vScaleFactor
            width: parent.width

            SliderControlLabeled{
                id: contactHRDisplay
                headingString: qsTr("Contact Heart Rate: ") + application.emptyString
                value:contactHR
                displayValue: (value ? value + qsTr("BPM") : qsTr("NONE")) + application.emptyString
                sliderVisible: false
            }

            SliderControlLabeled{
                id: telemetryHRDisplay
                headingString: qsTr("Telemetry Heart Rate: ") + application.emptyString
                value:telemetryHR
                displayValue: (value ? value + qsTr("BPM") : qsTr("NONE")) + application.emptyString
                sliderVisible: false
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0
            }

            SliderControlLabeled{
                id: fanControl
                headingString: qsTr("Fan Speed Control") + application.emptyString
                //value: fanSpeed
                minValue: 0
                maxValue: 100
                onValueChanged: fan.fanSpeed = fanControl.value;
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0
            }

            SliderControlLabeled{
                id: keypressDisplay
                headingString: qsTr("Keypress: ") + application.emptyString
                value:buttonNumber
                sliderVisible: false
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0

            }

            SliderControlLabeled{
                id: rpmDisplay
                headingString: qsTr("Pedal RPM: ") + application.emptyString
                value:root.rpm
                sliderVisible: false
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0

            }

            SliderControlLabeled{
                id: wattDisplay
                headingString: options.en957Enabled ? qsTr("Work: ") + application.emptyString : qsTr("Watts: ") + application.emptyString
                value:root.watts
                sliderVisible: false
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0

            }

            ButtonControlLabeled {
                id: safetyCircuitControl
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0
                width: 0.8 * parent.width
                height: visible && opacity > 0 ? 60 * main.vScaleFactor : 0

                labelString: qsTr("Safety Circuit Control") + application.emptyString

                /* Only show button on a nucleus system. */
                buttonEnabled: machineinfo.systemConfig === "NUCLEUS" &&
                               (statusString === "Engaged" ||
                                statusString === "Disengaged")

                statusString: if(machinecontrol.estop === true) {
                                  return qsTr("Engaged") + application.emptyString
                              } else {
                                  return qsTr("Disengaged") + application.emptyString
                              }

                onButtonPressed: {
                    if(statusString === "Engaged") {
                        machinecontrol.safetyCircuitOpen()
                    } else if(statusString === "Disengaged") {
                        machinecontrol.safetyCircuitClose()
                    }
                }

                Connections {
                    target: machinecontrol
                    onSafetyCircuitDisengaged: safetyCircuitControl.statusString = qsTr("Disengaged") + application.emptyString;
                    onSafetyCircuitDisengaging: safetyCircuitControl.statusString = qsTr("Disengaging") + application.emptyString;
                    onSafetyCircuitEngaged: safetyCircuitControl.statusString = qsTr("Engaged") + application.emptyString;
                    onSafetyCircuitEngaging: safetyCircuitControl.statusString = qsTr("Engaging") + application.emptyString;
                }
            }

            ButtonControlLabeled {
                id: staticBrakeToggle
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0
                width: 0.8 * parent.width
                height: visible && opacity > 0 ? 60 * main.vScaleFactor : 0

                labelString: qsTr("Static Brake Toggle") + application.emptyString

                statusString: if(machinecontrol.isStaticBrakeEngaged === true) {
                                  return qsTr("Engaged") + application.emptyString
                              } else {
                                  return qsTr("Disengaged") + application.emptyString
                              }

                buttonEnabled: (machinecontrol.treadmillSpeedTarget === 0)

                onButtonPressed: {
                    if(machinecontrol.isStaticBrakeEngaged === true) {
                        machinecontrol.staticBrakeDisengage()
                    } else {
                        machinecontrol.staticBrakeEngage()
                    }
                }

                Connections {
                    target: machinecontrol
                    onIsStaticBrakeEngagedChanged: {
                        if(isStaticBrakeEngaged === true) {
                            staticBrakeToggle.statusString = qsTr("Engaged") + application.emptyString
                        } else {
                            staticBrakeToggle.statusString = qsTr("Disengaged") + application.emptyString
                        }
                    }
                }
            }

            SliderControlLabeled{
                id: speedControl
                property bool valueUpdateOnly: false
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0
                headingString: (main.isStepmill ? qsTr("Motor Speed") : qsTr("Belt Speed") ) + application.emptyString
                displayValue: main.isStepmill ?
                                  machinecontrol.stepmillSpeedActual.toFixed(0) + " / " +  value.toFixed(0) + qsTr(" SPM") :
                                 // machineinfo.stepmillSpeedActual.toFixed(0) + " / " +  value.toFixed(0) + qsTr(" SPM") :
                                  (machinecontrol.treadmillSpeedActual/10.0).toFixed(1) + " / " + (value / 10.0).toFixed(1) + qsTr(" MPH")
                minValue: 0
                maxValue: main.isStepmill ? machineinfo.stepmillSpeedMaxSpm : machineinfo.treadmillSpeedMaxMph
                onValueChanged: {
                    if(valueUpdateOnly === false) {
                        if(main.isStepmill)
                            machinecontrol.stepmillSpeedTarget = speedControl.value
                        else
                            machinecontrol.treadmillSpeedTarget = speedControl.value
                    } else {
                        valueUpdateOnly = false
                    }
                }

                Component.onCompleted: value = (main.isStepmill ? machinecontrol.stepmillSpeedTarget : machinecontrol.treadmillSpeedTarget)

                Connections {
                    target: errorManager

                    onNucleusFaultCleared: {
                        if(speedControl.value !== 0) {
                            speedControl.valueUpdateOnly = true
                            speedControl.value = machinecontrol.treadmillSpeedActual
                        }
                    }
                }
            }

            SliderControlLabeled{
                id: inclineControl
                property bool manualControl: false
                property bool valueUpdateOnly: false
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0
                headingString: qsTr("Incline") + application.emptyString
                value: 0
                displayValue: "(" +
                              machinecontrol.treadmillInclineADCActual +
                              ") / " +
                              (machinecontrol.treadmillInclinePercentGradeActual / 10).toFixed(1) +
                              " / " + (value / 10.0).toFixed(1) + " %"
                minValue: machineinfo.treadmillInclineMinPercentGrade
                maxValue: machineinfo.treadmillInclineMaxPercentGrade
                stepValue: 5
                onValueChanged: {
                    if(valueUpdateOnly === false) {
                        machinecontrol.treadmillInclineTarget = inclineControl.value;
                    } else {
                        valueUpdateOnly = false
                    }
                }

                Component.onCompleted: value = machinecontrol.treadmillInclinePercentGradeActual
                Connections {
                    target: inclineUpControl
                    onSelectedChanged: {
                        if(inclineUpControl.selected ) {
                            inclineControl.manualControl = true
                        }
                    }
                }
                Connections {
                    target: inclineDownControl
                    onSelectedChanged: {
                        if(inclineDownControl.selected ) {
                            inclineControl.manualControl = true
                        }
                    }
                }

                Connections {
                    target: machinecontrol
                    onIsInclineMovingChanged: {
                        if(inclineControl.manualControl === true && machinecontrol.isInclineMoving === false) {
                            inclineControl.manualControl = false
                            inclineControl.valueUpdateOnly = true
                            inclineControl.value = machinecontrol.treadmillInclinePercentGradeActual
                        }
                    }
                    onSafetyCircuitDisengaging: {
                        inclineControl.valueUpdateOnly = true
                        inclineControl.value = machinecontrol.treadmillInclinePercentGradeActual
                    }
                }
            }

            ButtonControlLabeled {
                id: inclineUpControl
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0
                width: 0.8 * parent.width
                height: visible && opacity > 0 ? 60 * main.vScaleFactor : 0

                labelString: qsTr("Incline Force Up") + application.emptyString

                statusString: ""

                buttonEnabled: (safetyCircuitControl.statusString === "Engaged")

                onButtonPressed: {
                    if(selected) {
                        machinecontrol.treadmillInclineUp()
                        selected = false
                    } else {
                        machinecontrol.treadmillInclineStop()
                        selected = true
                    }
                }
            }

            ButtonControlLabeled {
                id: inclineDownControl
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0
                width: 0.8 * parent.width
                height: visible && opacity > 0 ? 60 * main.vScaleFactor : 0

                labelString: qsTr("Incline Force Down") + application.emptyString

                statusString: ""

                buttonEnabled: (safetyCircuitControl.statusString === "Engaged")

                onButtonPressed: {
                    if(selected) {
                        machinecontrol.treadmillInclineDown()
                        selected = false
                    } else {
                        machinecontrol.treadmillInclineStop()
                        selected = true
                    }
                }
            }

            Checkbox {
                id: runModeCheckbox
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0
                label: qsTr("Constant Power Mode") + application.emptyString

                width: parent.width

                value: {
                    switch(machinecontrol.machineMode){
                    case MachineControl.MODE_CONSTANT_POWER: return true;
                    case MachineControl.MODE_CONSTANT_TORQUE: return false;
                    default: return false;
                    }
                }
                onPressed: {
                    value ? machinecontrol.machineMode = MachineControl.MODE_CONSTANT_POWER : machinecontrol.machineMode = MachineControl.MODE_CONSTANT_TORQUE
                }
            }

            SliderControlLabeled{
                id: torqueControl
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0
                headingString: qsTr("Torque Control") + application.emptyString
                minValue: 0
                maxValue: 470
                onValueChanged: {
                    machinecontrol.machineMode = MachineControl.MODE_CONSTANT_TORQUE
                    machinecontrol.workload = torqueControl.value;
                }
                //value: root.resistance
            }

            SliderControlLabeled{
                id: wattControl
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0
                headingString: options.en957Enabled ? qsTr("Work Control") + application.emptyString : qsTr("Watt Control") + application.emptyString
                minValue: 0
                maxValue: 600
                onValueChanged: {
                    machinecontrol.machineMode = MachineControl.MODE_CONSTANT_POWER
                    machinecontrol.workload = wattControl.value
                }
                //value: root.resistance
            }

            SliderControlLabeled{
                id: estopDisplay
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0
                headingString: qsTr("Safety Switch: ") + application.emptyString
                value: machinecontrol.estop
                displayValue: (value ? qsTr("inserted") : qsTr("removed")) + application.emptyString
                sliderVisible: false
            }

            // Stride Adjust Elliptical
            SliderControlLabeled{
                id: strideAdjustControl
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0
                headingString:qsTr("Stride Length") + application.emptyString
                minValue: machineinfo.adjustableStrideEllipticalMinStrideIn/10
                maxValue: machineinfo.adjustableStrideEllipticalMaxStrideIn/10
                onValueChanged: {
                    machinecontrol.strideLengthTarget = value * 10;
                }
                Component.onCompleted: value = machinecontrol.strideLengthTarget/10
            }

            SliderControlLabeled{
                id: strideTorqueControl
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0
                headingString:qsTr("Torque Control") + application.emptyString
                minValue: 0
                maxValue: 1000
                onValueChanged: {
                    machinecontrol.machineMode = MachineControl.MODE_CONSTANT_TORQUE
                    machinecontrol.workload = strideTorqueControl.value
                }
                //value: root.resistance
            }

            SliderControlLabeled{
                id: strideWattsControl
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0
                headingString: options.en957Enabled ? qsTr("Work Control") + application.emptyString : qsTr("Watt Control") + application.emptyString
                minValue: 0
                maxValue: 600
                onValueChanged: {
                    machinecontrol.machineMode = MachineControl.MODE_CONSTANT_POWER
                    machinecontrol.workload = strideWattsControl.value
                }
                //value: root.resistance
            }

            SliderControlLabeled{
                id: strideRpmDisplay
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0
                headingString: qsTr("Pedal RPM: ") + application.emptyString
                value:root.rpm
                sliderVisible: false
            }

            SliderControlLabeled{
                id: strideAdcDisplay
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0
                headingString: qsTr("Stride ADC value: ") + application.emptyString
                value:root.strideAdc
                sliderVisible: false
            }

            SliderControlLabeled{
                id: autoChannelChangeControl
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0
                headingString: qsTr("Automatic Channel Change Interval:") + application.emptyString
                minValue: 0
                maxValue: 600000
                stepValue: 1000
                displayValue: value
                onValueChanged:
                {
                    options.autoChannelChangeInterval = value
                    tvTestHud.channelChangeTime = value

                    }

                }

            SliderControlLabeled{
                id: autoVideoChangeControl
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0
                headingString: qsTr("Automatic OI Video Change Interval:") + application.emptyString
                minValue: 0
                maxValue: 300
                stepValue: 1
                displayValue: value
                onValueChanged:
                {
                    if(value > 0 && autoChannelChangeControl.value > 0) {
                        // Can't run TV channel change test at the same time as OI video channel change test
                        autoChannelChangeControl.value = 0
                    }

                    options.autoVideoChangeInterval = value*1000
                }
            }

            SliderControlLabeled {
                id: workoutAutoRestartIntervalControl
                opacity: diagnosticsMain.accessLevel >= 1 ? 1 : 0
                headingString: qsTr("Trigger Automatic Workout Restart") + application.emptyString

                width: parent.width

                minValue: 0
                maxValue: 60
                stepValue: 1
                displayValue: value

                onValueChanged: {
                    options.workoutAutoRestartInterval = value
                    if(options.workoutAutoRestartInterval > 0 ) {
                        if (options.workoutSummaryTimeout > 45) {
                            options.workoutSummaryTimeout = 45
                        }
                    } else if (options.workoutSummaryTimeout < options.defaults.workoutSummaryTimeoutDefault){
                        options.workoutSummaryTimeout = options.defaults.workoutSummaryTimeoutDefault
                    }
                }
            }
        }

        Rectangle {
            id: sliderBar

            property double baseHeight: scrollColumn.height
            property double contentHeight: scrollColumn.contentHeight

            anchors.right: parent.right
            anchors.rightMargin: 2 * main.hScaleFactor
            width: 15 * main.hScaleFactor
            height: baseHeight * baseHeight/contentHeight
            visible: baseHeight < contentHeight
            color: "darkgrey"
            radius: main.defaultRadius
            y: scrollColumn.y + (baseHeight  - height) * scrollColumn.contentY / (contentHeight - baseHeight)
        }
    }

    states: [
        State {
            name: "EllipticalStride"
            PropertyChanges {target: estopDisplay ; visible:false }
            PropertyChanges {target: rpmDisplay ; visible:false }
            PropertyChanges {target: wattDisplay ; visible:false }
            PropertyChanges {target: speedControl ; visible:false }
            PropertyChanges {target: inclineControl ; visible:false }
            PropertyChanges {target: runModeCheckbox ; visible: true }
            PropertyChanges {target: torqueControl ; visible:false }
            PropertyChanges {target: wattControl ; visible: false }
            PropertyChanges {target: strideAdjustControl; visible: true }
            PropertyChanges {target: strideTorqueControl; visible: !runModeCheckbox.value }
            PropertyChanges {target: strideWattsControl; visible: runModeCheckbox.value }
            PropertyChanges {target: strideRpmDisplay; visible: true }
            PropertyChanges {target: strideAdcDisplay; visible: true }
            PropertyChanges {target: staticBrakeToggle ; visible: false }            
            PropertyChanges {target: safetyCircuitControl ; visible: false }
            PropertyChanges {target: inclineUpControl ; visible: false }
            PropertyChanges {target: inclineDownControl ; visible: false }



        },
        State {
            name: "Elliptical"
            PropertyChanges {target: estopDisplay ; visible:false }
            PropertyChanges {target: rpmDisplay ; visible:true }
            PropertyChanges {target: wattDisplay ; visible:true }
            PropertyChanges {target: speedControl ; visible:false }
            PropertyChanges {target: inclineControl ; visible:false }
            PropertyChanges {target: runModeCheckbox ; visible: true }
            PropertyChanges {target: torqueControl ; visible:!runModeCheckbox.value}
            PropertyChanges {target: wattControl ; visible: runModeCheckbox.value}
            PropertyChanges {target: strideAdjustControl; visible: false }
            PropertyChanges {target: strideTorqueControl; visible: false }
            PropertyChanges {target: strideWattsControl; visible: false }
            PropertyChanges {target: strideRpmDisplay; visible: false }
            PropertyChanges {target: strideAdcDisplay; visible: false }
            PropertyChanges {target: staticBrakeToggle ; visible: false }
            PropertyChanges {target: safetyCircuitControl ; visible: false }
            PropertyChanges {target: inclineUpControl ; visible: false }
            PropertyChanges {target: inclineDownControl ; visible: false }
        },
        State {
            name: "Lateral Trainer"
            PropertyChanges {target: estopDisplay ; visible:false }
            PropertyChanges {target: rpmDisplay ; visible:true }
            PropertyChanges {target: wattDisplay ; visible:true }
            PropertyChanges {target: speedControl ; visible:false }
            PropertyChanges {target: inclineControl ; visible:false }
            PropertyChanges {target: runModeCheckbox ; visible: true }
            PropertyChanges {target: torqueControl ; visible:!runModeCheckbox.value}
            PropertyChanges {target: wattControl ; visible: runModeCheckbox.value}
            PropertyChanges {target: strideAdjustControl; visible: false }
            PropertyChanges {target: strideTorqueControl; visible: false }
            PropertyChanges {target: strideWattsControl; visible: false }
            PropertyChanges {target: strideRpmDisplay; visible: false }
            PropertyChanges {target: strideAdcDisplay; visible: false }
            PropertyChanges {target: staticBrakeToggle ; visible: false }
            PropertyChanges {target: safetyCircuitControl ; visible: false }
            PropertyChanges {target: inclineUpControl ; visible: false }
            PropertyChanges {target: inclineDownControl ; visible: false }
        },
        State {
            name: "Bike"
            PropertyChanges {target: estopDisplay ; visible:false }
            PropertyChanges {target: rpmDisplay ; visible:true }
            PropertyChanges {target: wattDisplay ; visible:true }
            PropertyChanges {target: speedControl ; visible:false }
            PropertyChanges {target: inclineControl ; visible:false }
            PropertyChanges {target: runModeCheckbox ; visible: true }
            PropertyChanges {target: torqueControl ; visible:!runModeCheckbox.value}
            PropertyChanges {target: wattControl ; visible: runModeCheckbox.value}
            PropertyChanges {target: strideAdjustControl; visible: false }
            PropertyChanges {target: strideTorqueControl; visible: false }
            PropertyChanges {target: strideWattsControl; visible: false }
            PropertyChanges {target: strideRpmDisplay; visible: false }
            PropertyChanges {target: strideAdcDisplay; visible: false }
            PropertyChanges {target: staticBrakeToggle ; visible: false }
            PropertyChanges {target: safetyCircuitControl ; visible: false }
            PropertyChanges {target: inclineUpControl ; visible: false }
            PropertyChanges {target: inclineDownControl ; visible: false }
        },
        State {
            name: "Stepmill"
            PropertyChanges {target: estopDisplay ; visible:true }
            PropertyChanges {target: rpmDisplay ; visible:false }
            PropertyChanges {target: wattDisplay ; visible:true }
            PropertyChanges {target: speedControl ; visible:true }
            PropertyChanges {target: inclineControl ; visible:false }
            PropertyChanges {target: runModeCheckbox ; visible: false}
            PropertyChanges {target: torqueControl ; visible: false}
            PropertyChanges {target: wattControl ; visible: false}
            PropertyChanges {target: strideAdjustControl; visible: false }
            PropertyChanges {target: strideTorqueControl; visible: false }
            PropertyChanges {target: strideWattsControl; visible: false }
            PropertyChanges {target: strideRpmDisplay; visible: false }
            PropertyChanges {target: strideAdcDisplay; visible: false }
            PropertyChanges {target: staticBrakeToggle ; visible: false }
            PropertyChanges {target: safetyCircuitControl ; visible: true }
            PropertyChanges {target: inclineUpControl ; visible: false }
            PropertyChanges {target: inclineDownControl ; visible: false }
        },
        State {
            name: "Treadmill"
            PropertyChanges {target: estopDisplay ; visible:true && machineinfo.systemConfig === "BRAINBOARD"}
            PropertyChanges {target: rpmDisplay ; visible:false }
            PropertyChanges {target: wattDisplay ; visible:false }
            PropertyChanges {target: speedControl ; visible:true }
            PropertyChanges {target: inclineControl ; visible:true }
            PropertyChanges {target: runModeCheckbox ; visible: false }
            PropertyChanges {target: torqueControl ; visible:false }
            PropertyChanges {target: wattControl ; visible:false }
            PropertyChanges {target: strideAdjustControl; visible: false }
            PropertyChanges {target: strideTorqueControl; visible: false }
            PropertyChanges {target: strideWattsControl; visible: false }
            PropertyChanges {target: strideRpmDisplay; visible: false }
            PropertyChanges {target: strideAdcDisplay; visible: false }
            PropertyChanges {target: staticBrakeToggle ; visible: true }
            PropertyChanges {target: safetyCircuitControl ; visible: true }
            PropertyChanges {target: inclineUpControl ; visible: true }
            PropertyChanges {target: inclineDownControl ; visible: true }

        }
    ]

}
