// Diagnostics Software Update Page

import QtQuick 2.12
import QtQuick.Controls 2.12
import QtQuick.Controls.Styles 1.4
import StorageMgr 1.0
import Process 0.1
import ".."
import Components 1.0

Item {
    id: root

    anchors.fill: parent

    property int fontSize: 20 * main.vScaleFactor
    property string firmwareVersion: nucleusfirmwareloader.firmwareVersionString
    property string firmwareUpdateFile: nucleusfirmwareloader.updateFile
    property bool firmwareUpdating: nucleusfirmwareloader.isUpdating
    property bool firmwareUpdateAvailable: nucleusfirmwareloader.isUpdateAvailable
    property bool softwareUpdating: false
    property bool checkingForUpdates: checkUpdates.processState == Process.Running
    property bool softwareUpdatesAvailable: checkUpdates.packages.length > 0
    /* This property indicates the combined status of firmware and software update progress */
    property bool updateInProgress: softwareUpdating || firmwareUpdating

    onFirmwareUpdatingChanged: {
        if(firmwareUpdating)
            updatingDialog.show()
        else if (!softwareUpdating) {
            updatingDialog.hide()
            restartMsg.visible = true
        }
    }

    Connections {
        target: nucleusfirmwareloader
        onFault: {
            print("Diagnostics Update: " + message)
            updatingDialog.hide()
            bootloadErrorReceivedDialogText.text = message
            bootloadErrorReceivedDialog.show()
            main.restartRequired = true
            main.rebootRequired = false
        }
        onStatus: {
            updatingDialog.text = message
        }
        onUpdateSuccessful: {
            main.restartRequired = true
            main.rebootRequired = false
        }
    }

    signal close()

    ModalDialog{
        id:bootloadErrorReceivedDialog
        blockKeys: true
        dismissable: true

        Item {
            z: bootloadErrorReceivedDialog.z + 1
            anchors.fill: parent

            Text {
                id: bootloadErrorReceivedDialogText
                anchors.fill: parent
                anchors.verticalCenter: parent.verticalCenter
                color: main.colorFontLight
                anchors.horizontalCenter: parent.horizontalCenter
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                wrapMode: Text.WordWrap
                font.bold: false
                font.pixelSize: 50 * main.vScaleFactor

            }
            MouseArea{
                anchors.fill: parent
                onClicked: {
                    bootloadErrorReceivedDialog.hide()
                    buzzer.buttonPress()
                }
                Rectangle {
                    anchors.fill: parent
                    color: main.colorGenericStimulusArea
                    opacity: 0.3
                    visible: main.showStimulusArea
                }
            }
        }
        Component.onCompleted: hide()
    }

    Item{
        id: informationSection

        width: parent.width
        height: 100*main.vScaleFactor
        anchors.left: parent.left
        anchors.leftMargin: 50 * main.vScaleFactor
        anchors.top: root.top
        anchors.topMargin: 50 * main.vScaleFactor

        Grid{
            anchors.fill: parent
            columns: 2
            rows: 2
            spacing: 30 * main.hScaleFactor

            Text{
                text: qsTr("Application Version") + application.emptyString
                font.pixelSize: fontSize
            }

            Text{
                text: application.appVersion
                font.pixelSize: fontSize
            }

            Text{
                text: qsTr("Nucleus Firmware Version") + application.emptyString
                font.pixelSize: fontSize
            }

            Text{
                text: machineinfo.nucleusFirmwareVersion === "" ? qsTr("NOT AVAILABLE") + application.emptyString : machineinfo.nucleusFirmwareVersion
                font.pixelSize: fontSize
            }
        }
    }

    Item{
        id: firmwareUpdateSection

        width: parent.width
        height: childrenRect.height
        anchors {
            top: informationSection.bottom ; topMargin: 30 * main.vScaleFactor
            left: parent.left ; leftMargin: 80 * main.hScaleFactor
        }

        Text {
            id: restartMsg
            visible: false
            text: qsTr("Software updated. Restart console.") + application.emptyString
            font.pixelSize: fontSize
            anchors {
                left: checkForUpdatesButton.right
                leftMargin: 15 * main.hScaleFactor
                verticalCenter: checkForUpdatesButton.verticalCenter
            }
        }

        Text {
            id: updateCheckingText
            visible: checkUpdates.processState == Process.Running
            text: qsTr("Checking for software updates...") + application.emptyString
            font.pixelSize: fontSize
            anchors {
                left: packageUpdateButton.right
                leftMargin: 15 * main.hScaleFactor
                verticalCenter: packageUpdateButton.verticalCenter
            }
        }

        Text {
            id: packageUpdateText
            visible: checkUpdates.packages.length > 0
            text: qsTr("Software updates are available.") + application.emptyString
            font.pixelSize: fontSize
            anchors {
                left: packageUpdateButton.right
                leftMargin: 15 * main.hScaleFactor
                top: packageUpdateButton.top
            }
        }

        Text{
            id: hexFileStatusText
            font.pixelSize: fontSize
            anchors {
                left: packageUpdateButton.right ;
                leftMargin: 15 * main.hScaleFactor;
                bottom: packageUpdateButton.bottom
            }
            visible: firmwareUpdateAvailable && !restartMsg.visible && !updateCheckingText.visible
            text: {
                return qsTr("Nucleus firmware to load: ") + application.emptyString + firmwareVersion
            }
        }
        
        StandardButton {
            id: packageUpdateButton

            visible: (root.softwareUpdatesAvailable || root.firmwareUpdateAvailable) && !restartMsg.visible

            width: 150 * main.hScaleFactor
            height: 78 * main.vScaleFactor
            anchors.left: firmwareUpdateSection.left
            anchors.top: firmwareUpdateSection.top

            buttonText: qsTr("Update Software") + application.emptyString
            onClicked: {
                // clear list of packages before starting upgrade so button disappears
                checkUpdates.packages = ""
                //updateVideoFiles.start()
                updatePackages.start()
                if (firmwareUpdateAvailable) {
                    logger.logAlways("Diagnostics Update: Loading firmware:" + firmwareUpdateFile)
                    nucleusfirmwareloader.startFirmwareLoad(firmwareUpdateFile)
                }
            }
        }

        // check for updates button visible only while not already checking or update button shown
        StandardButton {
            id: checkForUpdatesButton

            visible: (checkUpdates.processState != Process.Running) && !packageUpdateButton.visible

            width: 150 * main.hScaleFactor
            height: 78 * main.vScaleFactor
            anchors.left: firmwareUpdateSection.left
            anchors.top: firmwareUpdateSection.top

            buttonText: qsTr("Check for Updates") + application.emptyString
            onClicked: {
                checkUpdates.packages = ""
                checkUpdates.start()
            }
        }

        StandardButton {
            id: gemUpdateButton
            visible: diagnosticsMain.accessLevel >= 1
            buttonText: qsTr("Pair with GEM Update App") + application.emptyString
            selected: false

            label.wrapMode: Text.WordWrap

            width: 200 * main.hScaleFactor
            height: 78 * main.vScaleFactor
            anchors {
                left: parent.left
                top: packageUpdateButton.bottom
                topMargin: 20 * main.vScaleFactor
            }

            onClicked: {
                if(!selected) {
                    console.log("Calling  gem.startAdvertising()")
                    gem.startAdvertising()
                    selected = true
                } else {
                    console.log("Calling  gem.stopAdvertising()")
                    gem.stopAdvertising()
                    selected = false
                }
            }
            Connections {
                target: gem
                onConnectedChanged: {
                    if(!gem.connected === 1 && gem.advertisedName === "")
                        gemUpdateButton.selected = false
                }
            }
        }

        Text {
            visible: diagnosticsMain.accessLevel >= 1
            text: qsTr("Pairing Name: ") + (gemUpdateButton.selected ? gem.advertisedName : qsTr("None")) +
                  "\n" + qsTr("Firmware Version: ") + gem.firmwareVersion + application.emptyString
            font.pixelSize: fontSize
            anchors {
                verticalCenter: gemUpdateButton.verticalCenter
                left: gemUpdateButton.right
                leftMargin: 30 * main.hScaleFactor
            }
        }
    }

    Text {
        id: instructions

        width: parent.width - (100 * main.hScaleFactor)

        anchors { bottom: parent.bottom ; bottomMargin: 90 * main.vScaleFactor ;  horizontalCenter: parent.horizontalCenter }
        horizontalAlignment: Text.AlignLeft
        wrapMode: Text.WordWrap
        visible: !restartMsg.visible
        text: {
            if(root.firmwareUpdating || root.softwareUpdating)
                return qsTr("Software is now being updated.  DO NOT power off the machine while software update is taking place.")
            else if(root.checkingForUpdates)
                return qsTr("Checking the network and connected USB devices for software updates.  This may take several seconds.  If updates are found, options for updating will be available above.")
            else if(root.softwareUpdatesAvailable || root.firmwareUpdateAvailable)
                return qsTr("Use the controls above to begin updating software.  Updates may take several minutes, and the machine power must not be interrupted.")
            else
                return qsTr("The machine software and firmware appear to be current.  Insert a software USB memory device and press the button above to check for updates.")
        }
    }

    StorageMgr {
        id: storage
    }

    ModalDialog {
        id: updatingDialog
        blockKeys: true
        visible: false  // shown when update process starts
        property alias text: bootLoadStatusText.text

        Text {
            id: dialogText
            width: 501 * main.hScaleFactor
            height: 99 * main.vScaleFactor
            color: main.colorFontLight
            text: qsTr("Updating software...") + application.emptyString
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.top: parent.top
            anchors.topMargin: 35 * main.vScaleFactor
            horizontalAlignment: Text.AlignHCenter
            wrapMode: Text.WordWrap
            font.bold: false
            font.pixelSize: 40 * main.vScaleFactor
            z: updatingDialog.z + 1
        }

        BusyIndicator {
            id: control
            z: parent.z + 1
            anchors {
                horizontalCenter: parent.horizontalCenter
                verticalCenter: parent.verticalCenter
                verticalCenterOffset: parent.height/4
            }
            width: 65
            height: 65

            running: parent.visible
            contentItem: Item {
                 implicitWidth: 64
                 implicitHeight: 64

                 Item {
                     id: item
                     x: parent.width / 2 - 32
                     y: parent.height / 2 - 32
                     width: 64
                     height: 64
                     opacity: control.running ? 1 : 0

                     Behavior on opacity {
                         OpacityAnimator {
                             duration: 250
                         }
                     }

                     RotationAnimator {
                         target: item
                         running: control.visible && control.running
                         from: 0
                         to: 360
                         loops: Animation.Infinite
                         duration: 1250
                     }

                     Image {
                         visible: control.running
                         source: "../graphics/ui3_0/Components/busyIndicator.png"
                         fillMode: Image.PreserveAspectFit
                         RotationAnimator on rotation {
                             running: control.running
                             loops: Animation.Infinite
                             duration: 2000
                             from: 0
                             to: 360
                         }
                     }
                 }
             }
        }

        Item {
            z: parent.z + 1
            anchors.fill: parent
            visible: firmwareUpdateAvailable

            Text{
                id: bootLoadStatusText
                text: ""
                wrapMode: Text.WordWrap
                color: main.colorFontLight
                font.bold: false
                font.pixelSize: 50 * main.vScaleFactor
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.bottom: bootloadProgress.top
                anchors.bottomMargin: 5 * main.vScaleFactor
            }

            ProgressBar {
                id: bootloadProgress
                width: 0.5 * parent.width
                height: 0.15 * parent.height
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
                anchors.verticalCenterOffset: 0.5*bootLoadStatusText.height
                value: nucleusfirmwareloader.progress
            }
        }
    }

    // The following process gathers a list of packages that may be updated
    Process {
        id: checkUpdates
        property string packages

        program: "bash"
        arguments: [ "scripts/list_upgrades.sh" ]
        onReadyReadStandardOutput: {
            packages += readAllStandardOutput()
        }
        onStarted: {
            nucleusfirmwareloader.findHexFile()
            console.log("checking updates")
        }
        onFinished: logger.logAlways("packages upgradable:" + packages)
    }

    // The following process updates packages using apt-get
    Process {
        id: updatePackages

        program: "bash"
        arguments: ["scripts/do_upgrades.sh"]

        onReadyReadStandardOutput: {
            console.log(readAllStandardOutput())
        }
        onReadyReadStandardError: {
            console.log(readAllStandardError())
        }
        onStarted: {
            updatingDialog.show()
            root.softwareUpdating = true
            logger.logAlways("Updating software packages...")
        }
        onFinished: {
            checkUpdates.packages = ""
            checkUpdates.start()
            logger.logAlways("Diagnostics Update: software package update complete.")
            root.softwareUpdating = false
            if(!main.restartRequired) 
                main.rebootRequired = true
            if(!firmwareUpdating){
                updatingDialog.hide()
                restartMsg.visible = true
            }
        }
    }
    
    // The following process updates packages using apt-get
    Process {
        id: updateVideoFiles

        program: "bash"
        arguments: ["scripts/video_updater/c360-videos-spectrum/sync_and_install.sh","-n"]

        onReadyReadStandardOutput: {
            console.log(readAllStandardOutput())
        }
        onReadyReadStandardError: {
            console.log(readAllStandardError())
        }
        onStarted: {
            console.debug("updating Video files...")
            //updatingDialog.show()
            //root.softwareUpdating = true
            //logger.logAlways("Updating video files...")
        }
        onFinished: {
            //checkUpdates.packages = ""
            //checkUpdates.start()
           // logger.logAlways("Diagnostics Update: video files updated.")
//            if(!firmwareUpdating){
//            updatingDialog.hide()
//         }
        }
    }

    Component.onCompleted: {
        console.log("starting software update check...")
        checkUpdates.start()
    }
}
