import QtQuick 2.12
import QtQuick.Layouts 1.12
import Components 1.0
import "../LanguageMenu/language.js" as LanguageJS
import ".."

/**
 * @brief Item that contains the contents of the home screen of the application.
 */
Item {
    id: home_screen
    anchors.fill: parent

    onVisibleChanged: nucleus.querySystemReady()

    property int sidebarWidth: 369 * main.hScaleFactor
    property int rightMarginWidth: 25 * main.hScaleFactor

    /**
     * @brief type:bool Boolean that determines if the Cardio360 button is to be displayed.
     */
    property bool showCardio360: false

    /**
     * @brief type:bool Boolean that determines if the buttons are enabled.
     */
    property bool buttonsEnabled: true

    /**
     * @brief type:bool Boolean that determines if the monuments button is to be displayed.
     */
    property bool showMonumentsButton: false

    /**
     * @brief Signal for when the login button has been pressed.
     */
    signal loginPressed()

    /**
     * @brief Signal for when the register button has been pressed.
     */
    signal registerPressed()

    /**
     * @brief Signal for when the Quick Start button has been pressed.
     */
    signal quickstartPressed()

    /**
     * @brief Signal for when the Cardio 360 Quick Start button has been pressed.
     */
    signal c360QuickstartPressed()

    /**
     * @brief Signal for when the monuments button has been pressed.
     */
    signal monumentsPressed()

    /**
     * @brief Signal for when the Signout button has been pressed.
     */
    signal signoutPressed()

    /**
     * @brief Updates the language icon's image source.
     * @param type:string lang The new langauge flag to be used.
     */
    function updateLanguageIcon(lang) {
        var base = LanguageJS.getLanguageIcon(lang);
        language_button.sourceName = base;
    }

    Item { // Left menu button area
        id: leftMenu
        width: home_screen.sidebarWidth
        anchors {
            left: parent.left
            top: parent.top
            bottom: parent.bottom
        }

        ScalableImage {
            id: background
            z: -2
            anchors.centerIn: parent
            height: parent.height
            width: parent.width
            sourceName: "StandardPage/View_Background"
            fillMode: Image.PreserveAspectCrop
        }

        ScalableImage {
            id: logoImage
            anchors {
                left: parent.left
                leftMargin: 73 * main.hScaleFactor
                top: parent.top
                topMargin: 25 * main.vScaleFactor
            }
            width: 213 * main.hScaleFactor
            height: 29 * main.vScaleFactor
            enabled: (visible && opacity > 0.0)
            visible: true
            fillMode: Image.PreserveAspectFit

            sourceName: "HomeScreen/TRU_Home_Logo";

            /**
         * @brief type:bool Boolean that determines if the True Fitness logo is blinking.
         */
//            property bool blinking

            /**
         * @brief Starts the True Fitness logo blinking routine.
         */
//            function startBlink() {
//                logo_blink_timer.start();
//                logoImage.blinking = true;
//            }

            /**
         * @brief Stops the True Fitness logo blinking routine and makes the True Fitness logo visible.
         */
//            function cancelBlink() {
//                blinking = false;
//                logo_blink_timer.stop();
//                logoImage.visible = true;
//                logoImage.opacity = 1.0;
//            }

            MouseArea {
                onPressAndHold: accessKeyPad.visible = true
                anchors {
                    top: parent.top;
                    topMargin: -1 * 25 * main.vScaleFactor
                    left: parent.left
                    leftMargin: -1 * 25 * main.hScaleFactor
                }
                width: logoImage.width + (30 * main.hScaleFactor)
                height: logoImage.height + (30 * main.vScaleFactor)

                Rectangle {
                    anchors.fill: parent
                    color: main.colorGenericStimulusArea
                    opacity: 0.3
                    visible: main.showStimulusArea
                }

//                Timer {
//                    id: logo_blink_timer

//                    /**
//                 * @brief type:int Property that holds how many times the True Fitness logo has blinked during the blinking routine.
//                 */
//                    property int blink_count: 0

//                    running: false
//                    repeat: true
//                    interval: 1000
//                    onTriggered: {
//                        logoImage.opacity = !logoImage.opacity;
//                        blink_count++;
//                    }
//                    onBlink_countChanged: {
//                        if(blink_count > 10) {
//                            logoImage.blinking = false;
//                            logoImage.opacity = 1.0;
//                            blink_count = 0;
//                            logo_blink_timer.stop();
//                        }
//                    }
//                }
            }
        }

        Text {
            id: welcomeBackLabel
            visible: userData.userName !== ""
            anchors {
                horizontalCenter: parent.horizontalCenter
                top: logoImage.bottom
                topMargin: 30 * main.vScaleFactor
            }
            width: 230 * main.hScaleFactor
            horizontalAlignment: Text.AlignHCenter
            color: main.colorFontGreen
            text: qsTr("Welcome Back ") + userData.userName + "!" + application.emptyString
            font.pixelSize: 18 * main.vScaleFactor
            font.bold: false
            wrapMode: Text.WordWrap
        }

        ColumnLayout {
            id: homeColumn
            anchors {
                left: parent.left
                leftMargin: 92 * main.hScaleFactor
                top: parent.top
                topMargin: (options.enableNFC ? 100 : 150) * main.vScaleFactor
                right: parent.right
                rightMargin: 1092 * main.hScaleFactor
                bottom: parent.bottom
                bottomMargin: (options.enableNFC ? 100 : 150) * main.vScaleFactor
            }

            /*StandardButton {
                id: testButton // Quick Start button
                width: 181 * main.hScaleFactor
                height: 92 * main.vScaleFactor
                buttonText: qsTr("LED TEST") + application.emptyString
                textColor: main.colorFontGreen
                onClicked: nucleus.setCenterpodLEDTest(1,1)
                Layout.preferredWidth: width
                Layout.preferredHeight: height
                Layout.alignment: Qt.AlignCenter
                icon.sourceName: "HomeScreen/QuickStartIcon"
                enabled: true
            }*/

            StandardButton {
                id: qsButton // Quick Start button
                width: 181 * main.hScaleFactor
                height: 92 * main.vScaleFactor
                buttonText: qsTr("QUICK START") + application.emptyString
                textColor: main.colorFontGreen
                onClicked: quickstartPressed()
                Layout.preferredWidth: width
                Layout.preferredHeight: height
                Layout.alignment: Qt.AlignCenter

                icon.sourceName: "HomeScreen/QuickStartIcon"
                enabled: machinecontrol.isBeltMoving === false &&
                         machinecontrol.isInclineMoving === false &&
                         bluetoothAdapter.connecting == false &&
                         nucleus.systemReady === true || (nucleus.systemReady === false && options.manufacturingTestModeEnabled === true)
            }

            StandardButton {
                id: cardio360qsButton // cardio360 Quick Start button
                width: 181 * main.hScaleFactor
                height: 92 * main.vScaleFactor
                enabled: showCardio360 && bluetoothAdapter.connecting === false
                visible: showCardio360
                wideImage: true
                buttonText: qsTr("QUICK START") + application.emptyString
                textColor: main.colorFontGreen
                onClicked: c360QuickstartPressed()
                onPressAndHold: demoC360Notice.showWithTimeout(2)
                Layout.preferredWidth: width
                Layout.preferredHeight: height
                Layout.alignment: Qt.AlignCenter

                icon.sourceName: "HomeScreen/C360_QuickStart_Icon"

                MessagePopup {
                    id: demoC360Notice
                    message: qsTr("Cardio360 Demo") + application.emptyString
                    onTimedOut: {
                        hide()
                        main.quickStart("Cardio_360_Demo.xml")
                    }
                }
            }

            StandardButton {
                id: monumentsButton
                width: 181 * main.hScaleFactor
                height: 92 * main.vScaleFactor
                enabled: showMonumentsButton && bluetoothAdapter.connecting === false
                visible: showMonumentsButton
                buttonText: qsTr("MONUMENTS") + application.emptyString
                onClicked: monumentsPressed()
                Layout.preferredWidth: width
                Layout.preferredHeight: height
                Layout.alignment: Qt.AlignCenter

                icon.sourceName: "WF/WF_Button_Monuments"
            }

            StandardButton {
                id: wfButton // Workout Finder button
                width: 181 * main.hScaleFactor
                height: 92 * main.vScaleFactor
                buttonText: qsTr("WORKOUT FINDER") + application.emptyString
                onClicked: main.state = "MenuState";
                Layout.preferredWidth: width
                Layout.preferredHeight: height
                Layout.alignment: Qt.AlignCenter
                enabled: bluetoothAdapter.connecting === false

                icon.sourceName: "HomeScreen/WorkoutFinderIcon"
            }

            StandardButton {
                id: sirButton // Sign-in / Register button
                width: 181 * main.hScaleFactor
                height: 92 * main.vScaleFactor
                enabled: main.loginAvailable
                visible: main.loginAvailable
                buttonText: qsTr("SIGN IN / REGISTER") + application.emptyString
                onClicked: main.state = "LoginState"
                Layout.preferredWidth: width
                Layout.preferredHeight: height
                Layout.alignment: Qt.AlignCenter

                icon.sourceName: "HomeScreen/SignInRegisterIcon"
            }

            Item { // Apple Watch Connection Menu
                width: childrenRect.width
                height: childrenRect.height
                visible: !gem.appleWatchConnected && options.enableNFC

                StandardIcon {
                    id: applewatch_button
                    width: 181 * main.hScaleFactor
                    height: 62*main.vScaleFactor
                    onClicked: {
                        main.showAppleWatchConnectsInfo = true
                    }

                    sourceName: "Apple/Apple_Watch_Connects_Marketing_Badge_RGB_lg_121019";
                    useSVG: true

                    fillMode: Image.PreserveAspectFit
                }
            }

            StandardButton { // EGym Connected
                id: eGym_connection
                visible: eGym.enabled
                width: 181 * main.hScaleFactor
                height: 92 * main.vScaleFactor
                Layout.preferredWidth: width
                Layout.preferredHeight: height
                Layout.alignment: Qt.AlignCenter
                border.color: "transparent"

                icon.sourceName: "eGym/EGYMLogoIcon"
                onClicked: eGymConnectsInfo.show()
            }
        }

        Row {
            id: iconLayout
            anchors {
                left: parent.left
                leftMargin: (gem.appleWatchConnected ? 50 : 72) * main.hScaleFactor
                top: parent.top
                topMargin: 706 * main.vScaleFactor
                right: parent.right
                rightMargin: 1072 * main.hScaleFactor
                bottom: parent.bottom
                bottomMargin: 25 * main.vScaleFactor
            }
            spacing: (gem.appleWatchConnected ? 10 : 34.5) * main.hScaleFactor


            StandardIcon { // Lock Icon
                id: lock_icon
                width: 51 * main.hScaleFactor
                height: 37 * main.vScaleFactor
                onPressAndHold: screen_lock_overlay.open()

                sourceName: "HomeScreen/LOCK_ICON";
            }

            StandardIcon { // Language Selection Menu
                id: language_button
                width: 51 * main.hScaleFactor
                height: 37 * main.vScaleFactor
                fillMode: Image.PreserveAspectCrop
                onClicked: main.state = "LanguageMenuState"
                onVisibleChanged: {
                    if(visible && _hideLanguageButton) {
                        visible = false;
                    }
                }

            }

            StandardIcon { // Bluetooth Device Connection Menu
                id: bluetooth_button
                width: 51 * main.hScaleFactor
                height: 37 * main.vScaleFactor
                onClicked: main.showMediaBrowser = true

                sourceName: "Bluetooth/BluetoothIcon";
            }

            StandardIcon { // Apple Watch Connection Menu
                id: applewatch_connection
                visible: gem.appleWatchConnected && options.enableNFC
                width: 51 * main.hScaleFactor
                height: Math.max( 30*main.vScaleFactor, 40) // 40px minimum height required by Apple

                sourceName: "Apple/Apple_Watch_connection_icon_green_RGB_090717"
                useSVG: true
            }
        }
    }

    ModalDialog {
        id: eGymConnectsInfo
        visible: false
        dismissAnywhere: true
        dismissable: true
        countDown: 20
        z: 999

        onVisibleChanged: {
            if(visible) {
                startTimeout(20)
            }
        }

        onTimedOut: {
            eGymConnectsInfo.hide()
        }

        CloseButton {
            id: closeButton

            z: parent.z + 1
            anchors {
                right: parent.right ; rightMargin: 80 * main.hScaleFactor
                top: parent.top ; topMargin: 40 * main.vScaleFactor
            }
            onClicked: eGymConnectsInfo.hide()
        }

        Column {
            z: parent.z + 1
            anchors.fill: parent
            anchors {
                topMargin: 50 * main.scaleFactor
                bottomMargin: 50 * main.scaleFactor
                leftMargin: 120 * main.scaleFactor
                rightMargin: 120 * main.scaleFactor
            }
            spacing: 20 * main.vScaleFactor

            ScalableImage {
                id: eGymConnect_icon
                width: parent.width
                height: 40 * main.vScaleFactor
                fillMode: Image.PreserveAspectFit
                sourceName: "eGym/EGYMLogoIcon"
            }

            TextLabel {
                text: qsTr("Connect to eGym")
                font.pixelSize: 25 * main.vScaleFactor
                width: parent.width
                horizontalAlignment: Text.AlignHCenter
            }

            TextLabel {
                text: qsTr("Hold your RFID near the reader below to login to eGym.") + application.emptyString
                font.pixelSize: 20 * main.vScaleFactor
                wrapMode: Text.WordWrap
                width: 0.9 * parent.width
                horizontalAlignment: Text.AlignHCenter
                anchors.horizontalCenter: parent.horizontalCenter
            }

            ScalableImage {
                id: nfc_icon
                width: parent.width
                height: 60 * main.vScaleFactor
                fillMode: Image.PreserveAspectFit
                sourceName: "Apple/WatchNFCArrowIcon"
            }
        }
    }

    // screen lock which is opened when lock icon is pressed
    ScreenLock {
        id: screen_lock_overlay
        z:99
        lockTime: 20
        visible: false
    }

    ScalableImage {
        id: alpineRunnerLogo

        anchors {
            left: leftMenu.right
            leftMargin: -45 * main.hScaleFactor
            top: parent.top
            topMargin: 28 * main.vScaleFactor
        }
        width: 467 * main.hScaleFactor * .85
        height: 29 * main.vScaleFactor * .85
        fillMode: Image.PreserveAspectFit
        visible: main.isAlpineRunner

        sourceName: "HomeScreen/AlpineRunner_Logo";
    }

    ScalableImage {
        id: rightMarginBackground
        z: -2
        anchors {
            right: parent.right
            top: parent.top
            bottom: parent.bottom
        }
        width: home_screen.rightMarginWidth
        sourceName: "StandardPage/View_Background"
        fillMode: Image.PreserveAspectCrop
    }

    onOpacityChanged: {
        if (opacity == 0) {
            main.showAppleWatchConnectsInfo = false
        }
    }

} // end home_screen Item
