import QtQuick 2.12
import QtQuick.Layouts 1.12
import Components 1.0

import "../"

/**
 * @brief Item that represents a keypad that is used for entering numeric information.
 */
Item {
    id: root

    //width: numRow * keyWidth + (numCol-1)*keyHSpacing + 2 * sideMargins
    // height: numCol * (keyHeight + keyVSpacing) + textEntryHeight + 2 * topMargins
    property real scaleFactor: 1.0
    property real topMargins: scaleFactor * 20 * main.vScaleFactor
    property real sideMargins: scaleFactor * 40 * main.vScaleFactor

    property real keyWidth:  66*main.hScaleFactor
    property real keyHeight:  60 * main.vScaleFactor
    property real keyHSpacing: 7 * main.hScaleFactor
    property real keyVSpacing: 7 * main.vScaleFactor
    property real textEntryMinWidth: 0
    property real textEntryHeight: scaleFactor * 65 * main.vScaleFactor
    property real delegateWidth
    property real delegateHeight


    property int numCol: 6
    property int numRow: 2

    property bool isColorInverted: false

    property string labelForLeftArrow: qsTr("BACK")
    property string labelForRighArrow: qsTr("ENTER")

    /**
     * @brief Signal for when a value has been entered.
     * @param type:string val The value that has been entered.
     */
    signal valueEntered (string val)

    /**
     * @brief Signal for when a key has been pressed.
     * @param type:string key The key that has been pressed.
     */
    signal keyPress(string key)

    /**
     * @brief Signal for when the keypad is to be closed.
     */
    signal close()

    /**
     * @brief type:variant A property that holds the number_keys model.
     */
    property variant keys_model: number_keys

    /**
     * @brief type:string Property that determines the unit type of the value entered.
     */
    property string units: ""

    /**
     * @brief type:bool Property that determines if decimals are allowed.
     */
    property bool allow_decimal: false

    /**
     * @brief type:int Property that determines how many decimal places are allowed.
     */
    property int decimalPlaces:0

    /**
     * @brief type:int Property that determines how many digits are allowed.
     */
    property int length: 3

    /**
     * @brief type:bool Property that determines if the keyboard may be closed.
     */
    property bool allow_close: true

    /**
     * @brief type:bool Property that determines if the keys will be shown.
     */
    property bool show_keys: true

    /**
     * @brief type:bool Property that determines if an entry field appears above the keyboard.
     */
    property bool show_entry: false

    /**
     * @brief type:bool Property that determines if the entry is cleared after Enter key is pressed.
     */
    property bool clear_after_enter: true

    /**
     * @brief type:string Property that contains instructions on what the user is to perform with the keyboard.
     */
    property string instructions: ""

    /**
     * @brief type:string Property that holds the current text entry.
     */
    property variant keyEntryText: ""

    /**
     * @brief type:string Property that holds the entry to be summitted.
     */
    property string value: decimalPlaces ? decimalFormat(keyEntryText) : keyEntryText

    /**
     * @brief type:string Property that contains an image for a key?
     */
    property string icon: ""

    /**
     * @brief type:bool Property that determines if the enter button has been pressed.
     */
    property bool _enterPressed: false

    /**
     * @brief Clears the current text entry.
     */
    function clear() {
        _enterPressed = false
        keyEntryText = ""
    }

    /**
     * @brief Sets and enters the current entry.
     * @param type:string val The value to be added to the current text entry.
     */
    function setValue(val) {
        val = val.toFixed(root.decimalPlaces);
        keyEntryText = val
        _enterPressed = true  // next keypress will first clear entry
    }

    /**
     * @brief Submits the current entry.
     */
    function submit() {
        add(">")
    }

    onKeyPress: {
        add(key);
        keyEmitter.emitKey( getKeyValue(key) )
    }

    // re-translate the strings when the language changes
    Connections {
        target: application
        onLanguageChanged: {
            root.labelForLeftArrow= qsTr("BACK")
            root.labelForRighArrow= qsTr("ENTER")
        }
    }

    /**
     * @brief Adds a key value to the key entry.
     * @param type:string val The value to add to the entry text.
     */
    function add(val) {
        // clear the entry first if the old value is still shown after pressing enter
        if (keyEntryText[0] === "-" ) {
            keyEntryText = "";
        }

        if (val[0] === "<" ) {
            keyEntryText = keyEntryText.slice(0, -1);
            val = "";
        } else if (val === ">" ) {
            valueEntered(value);

            if (root.clear_after_enter) {
                keyEntryText = "";
            }

            root._enterPressed = true;
            return;
        } else {
            if(root._enterPressed) {
                clear();
            }

            keyEntryText = rollover(keyEntryText + val);
        }
    }

    /**
     * @brief Formats the text utilizing the decimal point.
     * @param type:string text The text to be formatted.
     */
    function decimalFormat(text) {
        var str = text;

        if(str.length === 0) {
            str = "0";
        }

        var val = parseInt(str, 10);
        val = val / Math.pow(10, root.decimalPlaces);
        val = val.toFixed(root.decimalPlaces);
        return val;
    }

    /**
     * @brief Rolls over the text.
     * @param type:string text The text to roll over.
     * @param type:string character Unused within this function.
     * @return type:string The new roll overed string.
     */
    function rollover(text, character) {
        var string = text;
        var newstr;

        if(text.length > root.length) {
            newstr = text.slice(1);
        } else {
            newstr = text;
        }

        return newstr;
    }

    RowLayout
    {
        anchors.fill: parent

        // stack top entry box next to keypad
        Item {
            id: keypad_box

            Layout.fillHeight: false
            Layout.fillWidth: true
            Layout.alignment: Qt.AlignVCenter
            height: childrenRect.height

            MouseArea {
                id: keypadBodyCover
                anchors.fill: parent
            }

            Grid {
                id: keygrid
                rows: root.numRow
                columns: root.numCol
                rowSpacing: root.keyVSpacing
                columnSpacing: root.keyHSpacing

                Repeater {
                    model: keys_model
                    delegate: keypad_button
                }
            }

            // delegate for keypad button
            Component {
                id: keypad_button
                StandardKeyboardButton {
                    singleWidth: root.keyWidth
                    singleHeight:root.keyHeight
                    backgroundColor: isColorInverted !== isNumber ? main.colorDialogBackground : main.colorToolbarBackground
                    buttonText: value == "<" ? labelForLeftArrow : (value == ">" ? labelForRighArrow : value)
                    fontsize: scaleFactor * size
                    enabled: true
                    isNumberLabel: isNumber
                    onClicked: keyPress(value)
                }
            }
        }

        Item {
            id: keypad_top
            visible: show_entry

            Layout.fillHeight: true
            Layout.preferredWidth: 200 * main.hScaleFactor

            RowLayout {
                id: toplayout
                anchors.fill: parent

                // rectangle where entered data is displayed
                Rectangle {
                    id: entrybox
                    Layout.preferredHeight: 73 * main.hScaleFactor
                    Layout.fillWidth: true
                    radius: main.defaultRadius
                    border.color: main.colorUIBorder
                    border.width: main.widthBorder
                    Layout.alignment: Qt.AlignHCenter | Qt.AlignVCenter

                    property real minWidth: (root.textEntryMinWidth > 0) ? root.textEntryMinWidth
                                                                         : (!root.allow_close) ? keypad_box.width
                                                                                               : keypad_box.width-close_button.width-toplayout.spacing

                    ScalableImage {
                        id:keypadIcon
                        //Layout.fillHeight: true
                        //Layout.fillWidth: true
                        //Layout.top:
                        //Layout.alignment: Qt.AlignLeft
                        anchors.verticalCenter: entrybox.verticalCenter
                        anchors.left: entrybox.left
                        anchors.leftMargin: scaleFactor * 15 * main.hScaleFactor
                        height: entrybox.height*0.67
                        fillMode: Image.PreserveAspectFit
                        sourceName: (root.icon != "") ? root.icon.replace(/\.[^/.]+$/, "") : ""
                    }

                    NumberLabel {
                        id: entrytext
                        text: root.value
                        font.pixelSize: 45 * main.vScaleFactor
                        // font.pixelSize: entrybox.height*0.9
                        ///Layout.fillHeight: true
                        //Layout.fillWidth: true
                        anchors.verticalCenter: parent.verticalCenter
                        anchors.left: keypadIcon.right
                        anchors.leftMargin: scaleFactor * 30 * main.hScaleFactor
                        anchors.right: parent.right
                        anchors.rightMargin: scaleFactor * 45 * main.hScaleFactor
                        verticalAlignment: Text.AlignVCenter
                        horizontalAlignment: Text.AlignRight
                        color: main.colorFontDark
                        font.bold: true
                    }

                    // optionally display units
                    Text {
                        id: units_text
                        text: root.units
                        font.bold: false
                        font.pixelSize: 15 * main.vScaleFactor
                        //font.pixelSize: entrybox.height*0.15*main.vScaleFactor
                        // Layout.fillHeight: true
                        // Layout.fillWidth: true
                        verticalAlignment: Text.AlignVCenter
                        horizontalAlignment: Text.AlignRight
                        anchors.right: parent.right
                        anchors.rightMargin: scaleFactor*10 * main.hScaleFactor
                        anchors.verticalCenter: parent.verticalCenter
                        // show units only when some numbers are displayed
                        visible: keyEntryText && keyEntryText[0] !== '-' ? true : false
                    }
                }

                StandardButton {
                    id: close_button
                    visible: root.allow_close
                    enabled: root.allow_close
                    onClicked: root.close()
                    Layout.preferredHeight: Layout.preferredWidth
                    Layout.preferredWidth: scaleFactor * 30 * main.hScaleFactor
                    buttonText: "X"
                }
            }
        }
    }

    // model of keypad keys
    ListModel {
        id: number_keys
        ListElement { value: "1";   size: 35; isNumber: true}
        ListElement { value: "2";   size: 35; isNumber: true}
        ListElement { value: "3";   size: 35; isNumber: true}
        ListElement { value: "4";   size: 35; isNumber: true}
        ListElement { value: "5";   size: 35; isNumber: true}
        ListElement { value: "<";   size: 14; isNumber: false}
        ListElement { value: "6";   size: 35; isNumber: true}
        ListElement { value: "7";   size: 35; isNumber: true}
        ListElement { value: "8";   size: 35; isNumber: true}
        ListElement { value: "9";   size: 35; isNumber: true}
        // this is the backspace key.  The '<' value indicates to the entry function to delete previous character

        ListElement { value: "0";   size: 35; isNumber: true}
        ListElement { value:">";size: 14; isNumber: false}
    }

    /**
     * @brief Gets the value of the key pressed.
     * @return type:Key Returns the Qt.Key pressed if matched, otherwise returns Qt.Key_unknown.
     */
    function getKeyValue( label ) {
        switch( label ) {
        case "1" :
            return Qt.Key_1
        case "2" :
            return Qt.Key_2
        case "3" :
            return Qt.Key_3
        case "4" :
            return Qt.Key_4
        case "5" :
            return Qt.Key_5
        case "6" :
            return Qt.Key_6
        case "7" :
            return Qt.Key_7
        case "8" :
            return Qt.Key_8
        case "9" :
            return Qt.Key_9
        case "0" :
            return Qt.Key_0
        case "<" :
            return Qt.Key_Backspace
        case ">" :
            return Qt.Key_Enter
        }

        return Qt.Key_unknown
    }
}
