import QtQuick 2.12
import Components 1.0
import WorkoutRunData 1.0
import WorkoutManager 1.0
import "../string_format.js" as StrFormat
import ".."

Item {
    id: root

    // REQUIRED property: for translation to work (via qsTranslate), it must know where the external model is stored.
    // Without this you will likely see blank strings in your data table.
    property string sourceFile: "FitTestSummaryTable"

    visible: isBikeFitnessTest || isGerkinTest || isWFITest || isMilitaryTest

    property string workoutName: qsTr("FITNESS TEST") + application.emptyString

    property int tableWidth: 0.8 * width - tableHMargin
    property int tableHMargin: 20 * main.hScaleFactor
    property int numColumns: 4
    property int columnWidth: tableWidth/numColumns
    property int headingPointSize: 8 * main.vScaleFactor
    property int pointSize: 20 * main.vScaleFactor

    property int scoreColWidth: 136 * main.hScaleFactor
    property real scorePercent: (fitTestScore-testMin)/(testMax-testMin)

    property bool isBikeFitnessTest: false
    property bool isGerkinTest: false
    property bool isWFITest: false
    property bool isMilitaryTest: false
    property real fitTestScore: 0
    property real militaryTestScore: 0
    property real testMax: 1.0
    property real testMin: 0.0
    property string genderF: qsTr("F", "Female Gender Intial")
    property string genderM: qsTr("M", "Male Gender Intial")

    property string speedUnitsLabel: main.isStepmill ? qsTr("SPM") : workoutScreen.unitState === "ENGLISH" ? qsTr("MPH") : qsTr("KPH")

    function convertUnits(speedMiles) {
        return workoutScreen.unitState === "METRIC" ? Globals.mileToKm(speedMiles) : speedMiles;
    }

    function convertMeters(feet) {
        return workoutScreen.unitState === "METRIC" ? Globals.feetToMeters(feet).toFixed(0) : feet;
    }

    function formatHR(hr) {
        if (hr > 0)
            return hr.toFixed(0);
        else
            return "-";
    }

    function setData(workoutData) {
        root.workoutName = qsTr(workoutData.programName)
        root.genderF = qsTr("F", "Female Gender Intial")
        root.genderM = qsTr("M", "Male Gender Intial")

        labelsModel.clear();
        workoutModel.clear();
        coolDownModel.clear();
        totalModel.clear();

        var scoreRange = qsTr("ESTIMATED VO2");
        root.testMax = 1.0;
        root.testMin = 0.0;

        root.isBikeFitnessTest = workoutData.isBikeFitnessTest();
        if (root.isBikeFitnessTest) {
            root.fitTestScore = workoutData.getBikeFitTestScore();
        }

        root.isGerkinTest = workoutData.isGerkinTest();
        if (root.isGerkinTest) {
            root.fitTestScore = workoutData.getGerkinTestScore();
        }

        root.isWFITest = workoutData.isWFITest();
        if (root.isWFITest) {
            root.fitTestScore = workoutData.getWFITestScore();
        }

        root.isMilitaryTest = workoutData.isMilitaryTest();
        if (root.isMilitaryTest) {
            if (workoutManager.workoutProgram.programIndex === WorkoutProgram.TREAD_AIR_FORCE_MILITARY_FITNESS_TEST_PROG) {
                root.militaryTestScore = (workoutData.getAirForceScore() / 10.0);
                root.testMax = 50;
                root.testMin = 0;
            } else if (workoutManager.workoutProgram.programIndex === WorkoutProgram.TREAD_ARMY_MILITARY_FITNESS_TEST_PROG) {
                root.militaryTestScore = workoutData.getArmyScore();
                root.testMax = 100;
                root.testMin = 0;
            } else if (workoutManager.workoutProgram.programIndex === WorkoutProgram.TREAD_MARINES_MILITARY_FITNESS_TEST_PROG) {
                root.militaryTestScore = workoutData.getMarinesScore();
                root.testMax = 100;
                root.testMin = 0;
            } else if (workoutManager.workoutProgram.programIndex === WorkoutProgram.TREAD_NAVY_MILITARY_FITNESS_TEST_PROG) {
                root.militaryTestScore = workoutData.getNavyScore();
                root.testMax = 100;
                root.testMin = 45;
            } else {
                root.militaryTestScore = 0;
                root.testMax = 100;
                root.testMin = 0;
            }
            scoreRange = qsTr("SCORE (") + root.testMin + "-" + root.testMax + ")";
            root.fitTestScore = root.militaryTestScore;
        }

        // Label Column Data
        labelsModel.append({ "label" : "ELAPSED TIME"})
        labelsModel.append({ "label" : "AVG HEART RATE"})
        labelsModel.append({ "label" : "MAX HEART RATE"})
        labelsModel.append({ "label" : (main.isTreadmill || main.isStepmill)
                           ? qsTr("AVERAGE SPEED (") + speedUnitsLabel + ")"
                           : qsTr("AVERAGE PACE (RPM)")})
        labelsModel.append({ "label" : (main.isTreadmill || main.isStepmill)
                           ? qsTr("MAX SPEED (") + speedUnitsLabel + ")"
                           : qsTr("MAX PACE (RPM)")})
        labelsModel.append({ "label" : "AGE"})
        if(!root.isWFITest)
            labelsModel.append({ "label" : "GENDER"})
        labelsModel.append({ "label" : scoreRange})

        // Workout Column Data
        workoutModel.append({ "label" : StrFormat.formatTimeString(workoutData.workoutTime)});   // Elapsed Time
        workoutModel.append({ "label" : formatHR(workoutData.averageHeartRate)});                // Average Heart Rate
        workoutModel.append({ "label" : formatHR(workoutData.maximumHeartRate)});                // Max Heart Rate

        if (main.isTreadmill || main.isStepmill) {
            workoutModel.append({ "label" : convertUnits(workoutData.averageSpeed).toFixed(1)}); // Average Speed
            workoutModel.append({ "label" : convertUnits(workoutData.maximumSpeed).toFixed(1)}); // Max Speed
        } else {
            workoutModel.append({ "label" : workoutData.averagePace.toFixed(0)});                // Average Pace
            workoutModel.append({ "label" : workoutData.maximumPace.toFixed(0)});                // Max Pace
        }

        workoutModel.append({ "label" : " "});                                                   // Blank
        if(!root.isWFITest)
            workoutModel.append({ "label" : " "});                                               // Blank
        workoutModel.append({ "label" : " "});                                                   // Blank

        // Cool Down Column Data
        coolDownModel.append({ "label" : StrFormat.formatTimeString(workoutData.coolDownTime)});          // Elapsed Time
        coolDownModel.append({ "label" : formatHR(workoutData.coolDownAverageHeartRate) });               // Average Heart Rate
        coolDownModel.append({ "label" : formatHR(workoutData.maximumCoolDownHeartRate) });               // Max Heart Rate

        if (main.isTreadmill|| main.isStepmill) {
            coolDownModel.append({ "label" : convertUnits(workoutData.coolDownAverageSpeed).toFixed(1)}); // Average Speed
            coolDownModel.append({ "label" : convertUnits(workoutData.maximumCoolDownSpeed).toFixed(1)}); // Max Speed
        } else {
            coolDownModel.append({ "label" : workoutData.coolDownAveragePace.toFixed(0)});                // Average Pace
            coolDownModel.append({ "label" : workoutData.maximumCoolDownPace.toFixed(0)});                // Max Pace
        }

        coolDownModel.append({ "label" : " "});                                                           // Blank
        if(!root.isWFITest)
            coolDownModel.append({ "label" : " "});                                                       // Blank
        coolDownModel.append({ "label" : " "});                                                           // Blank

        // Total Column Data
        totalModel.append({ "label" : StrFormat.formatTimeString(workoutData.totalTime)});          // Elapsed Time
        totalModel.append({ "label" : formatHR(workoutData.totalAverageHeartRate)});                // Average Heart Rate
        totalModel.append({ "label" : formatHR(workoutData.totalMaximumHeartRate)});                // Max Heart Rate

        if (main.isTreadmill || main.isStepmill) {
            totalModel.append({ "label" : convertUnits(workoutData.totalAverageSpeed).toFixed(1)}); // Average Speed
            totalModel.append({ "label" : convertUnits(workoutData.totalMaximumSpeed).toFixed(1)}); // Max Speed
        } else {
            totalModel.append({ "label" : workoutData.totalAveragePace.toFixed(0)});                // Average Pace
            totalModel.append({ "label" : workoutData.totalMaximumPace.toFixed(0)});                // Max Pace
        }

        if (root.isBikeFitnessTest || root.isGerkinTest) {
            totalModel.append({ "label" : workoutData.userProgAge.toFixed(0) });                    // Age
            totalModel.append({ "label" : workoutData.userGender === 1 ? genderF : genderM });      // Gender
            totalModel.append({ "label" : Globals.formatWithSeparator(parseFloat(workoutData.estVO2).toFixed(2)) }); // VO2
        } else if (root.isWFITest ) {
            totalModel.append({ "label" : workoutData.userProgAge.toFixed(0) });                    // Age
            totalModel.append({ "label" : Globals.formatWithSeparator(parseFloat(workoutData.estVO2).toFixed(2)) }); // VO2
        } else if (root.isMilitaryTest) {
            if(workoutProgram.userAgeRequired) {
                totalModel.append({ "label" : workoutData.userProgAge.toFixed(0) });                // Age
            } else {
                totalModel.append({ "label" : " " });                                               // Blank
            }

            if(!root.isWFITest)
                totalModel.append({ "label" : workoutData.userGender === 1 ? genderF : genderM });  // Gender
            totalModel.append({ "label" : root.militaryTestScore.toFixed(1) });                     // Military Test Score
        }
    }

    ListModel
    {
        id: labelsModel
        property string heading: application.emptyString
        property int headingPointSize: root.headingPointSize
        property int width: root.columnWidth
        property int hAlign: Text.AlignLeft
        property bool showDottedLines: true
        property bool showFill: false
        property int pointSize: root.headingPointSize
        property bool isNumeric: false
    }

    ListModel
    {
        id: workoutModel
        property string heading: qsTr("WORKOUT") + application.emptyString
        property int headingPointSize: root.headingPointSize
        property int width: root.columnWidth
        property int hAlign: Text.AlignHCenter
        property bool showDottedLines: true
        property int pointSize: root.pointSize
        property bool isNumeric: true
    }

    ListModel
    {
        id: coolDownModel
        property string heading: qsTr("COOL DOWN") + application.emptyString
        property int headingPointSize: root.headingPointSize
        property int width: root.columnWidth
        property int hAlign: Text.AlignHCenter
        property bool showDottedLines: true
        property int pointSize: root.pointSize
        property bool isNumeric: true
    }

    ListModel
    {
        id: totalModel
        property string heading: qsTr("TOTAL") + application.emptyString
        property int headingPointSize: root.headingPointSize
        property int width: root.columnWidth
        property int hAlign: Text.AlignHCenter
        property bool showDottedLines: true
        property int pointSize: root.pointSize
        property bool isNumeric: true
    }

    ListModel {
        id: scoreSpectrumModel
        ListElement{ label: QT_TR_NOOP("EXCELLENT"); icon: "FitTest/FitTest_Excellent" }
        ListElement{ label: QT_TR_NOOP("GOOD"); icon: "FitTest/FitTest_Good" }
        ListElement{ label: QT_TR_NOOP("ABOVE AVG"); icon: "FitTest/FitTest_AboveAvg" }
        ListElement{ label: QT_TR_NOOP("AVERAGE"); icon: "FitTest/FitTest_Average" }
        ListElement{ label: QT_TR_NOOP("BELOW AVG"); icon: "FitTest/FitTest_BelowAvg" }
        ListElement{ label: QT_TR_NOOP("POOR"); icon: "FitTest/FitTest_Poor" }
        ListElement{ label: QT_TR_NOOP("VERY POOR"); icon: "FitTest/FitTest_VeryPoor" }
    }

    Component {
        id: scoreDelegate

        Item {
            height: 48 * main.hScaleFactor
            width: root.scoreColWidth

            ScalableImage {
                id: scoreImage
                anchors.fill: parent
                sourceName: model.icon
            }

            TextLabel {
                id: scoreLabel
                anchors.left: scoreImage.left
                anchors.leftMargin: 20 * main.hScaleFactor
                anchors.verticalCenter: scoreImage.verticalCenter
                text: qsTranslate(root.sourceFile, model.label)
                horizontalAlignment: Text.AlignLeft
                verticalAlignment: Text.AlignVCenter
                font.pixelSize: 0.25 * scoreImage.height

                // re-translate the strings when the language changes
                Connections {
                    target: application
                    onLanguageChanged: { scoreLabel.text = qsTranslate(root.sourceFile, model.label) }
                }
            }
        }
    }


    Rectangle {
        id: tableWrap

        width: root.width
        height: root.height
        anchors.top: parent.top
        anchors.left: parent.left
        color: main.colorDialogBackground
        border.color: main.colorUIBorder
        border.width: main.widthBorder

        DataTable {
            id: dataTable
            sourceFile: root.sourceFile
            anchors.top: parent.top
            anchors.left: parent.left
            anchors.leftMargin: root.tableHMargin
            width: root.tableWidth
            height: tableWrap.height
            models: [ labelsModel, workoutModel, coolDownModel, totalModel ]
        }

        ListView {
            id: scoreSpectrum

            anchors.right: tableWrap.right
            anchors.top: tableWrap.top
            anchors.topMargin: tableHMargin
            anchors.bottom: tableWrap.bottom
            anchors.bottomMargin: tableHMargin

            width: root.scoreColWidth
            spacing: 18 * main.vScaleFactor
            interactive: false

            model: scoreSpectrumModel
            delegate: scoreDelegate

        }

        ScalableImage {
            id: scoreIndicator

            anchors.right: scoreSpectrum.left
            anchors.rightMargin: 2 * main.hScaleFactor
            y: (1-root.scorePercent) * (scoreSpectrum.height - 2*tableHMargin) + tableHMargin
            width: 34 * main.hScaleFactor
            height: 34 * main.vScaleFactor

            sourceName: "FitTest/ScoreIndicator"
        }
    }
}
