import QtQuick 2.12
import TouchAppEnumerations 1.0
import WorkoutRunData 1.0
import WorkoutManager 1.0
import QtQuick.Layouts 1.12
import Components 1.0
import "../string_format.js" as StrFormat
import ".."
import "../BlueTooth"

StandardPage {
    id: root
    width: main.width
    height: main.height
    title: root.workoutName + qsTr(" SUMMARY") + application.emptyString
    
    image_width: 225/1366
    showBackButton: false

    property string workoutName: qsTr("WORKOUT") + application.emptyString
    property string workoutElapsed: "00:00"

    property bool isUsbDeviceLoaded: storageMgr.usbDeviceLoaded
    property int timeLeft: 0
    property bool saveHasHappened: false
    property int advertiseTimeOout: 60
    property bool dialogVisible: false

    signal close()
    signal save()
    signal advertise()
    signal connected()
    signal disconnect()

    Connections {
        target: root
        onConnected: bleDialog.show()
        onAdvertise: {
            bleDialog.show()
            ble.startAdvertiseing()
        }
        onDisconnect: {
            ble.stopAdvertising()
            bleDialog.hide()
        }
    }

    onClose: {
        pointsHistoryPopup.hide()
        pointsPopup.hide()
        disconnect()
    }

    function doSave() {
        if(isUsbDeviceLoaded) {
            save();
        } else {
            advertise();
            ble.advertising = true;
        }
    }

    function convertUnits(speedMiles) {
        return workoutScreen.unitState === "METRIC" ? Globals.mileToKm(speedMiles) : speedMiles;
    }

    function convertMeters(feet) {
        return workoutScreen.unitState === "METRIC" ? Globals.feetToMeters(feet).toFixed(0) : feet;
    }

    function setData(workoutData) {
        dataSummaryTable.reset();
        console.log("Setting final data values in summary, Time " + aWorkoutRunData.totalTime +
                    ", Avg HR " + aWorkoutRunData.totalAverageHeartRate.toString() +
                    ", Act Cal " + gem.gemActiveCalories.toFixed(0) + ", Watts " + aWorkoutRunData.avgWatts.toFixed(0) +
                    ", Avg Pace " + aWorkoutRunData.totalAveragePace)
        dataSummaryTable.updateDataValues()
        fitTestDataTable.setData(workoutData)

        if(fitTestDataTable.visible)
            root.workoutName = fitTestDataTable.workoutName
        else
            root.workoutName = qsTr("WORKOUT") + application.emptyString

        // prepare data for bluetooth advertisement
        root.workoutElapsed = workoutData.totalTime
    }

    Item {
        id: contentArea
        width: 1100 * main.hScaleFactor
        height: root.height
        anchors {
            horizontalCenter: parent.horizontalCenter
            horizontalCenterOffset: 43 * main.hScaleFactor
            top: parent.top
        }

        Item {
            id: tableBackground

            anchors.left: toolbarRow.left
            anchors.top: parent.top
            anchors.topMargin: 120 * main.vScaleFactor
            anchors.right: homeScreenButton.right
            anchors.bottom: toolbarRow.top
            anchors.bottomMargin: 50 * main.vScaleFactor

            EGymCardioTestSummary {
                id: cardioTest
                anchors.fill: parent
            }

            DataSummaryTable {
                id: dataSummaryTable
                selectable: false
                tableWidth: width - 2*tableHMargin
                anchors.fill: parent
                visible: !fitTestDataTable.visible && !cardioTest.visible
            }

            FitTestSummaryTable {
                id: fitTestDataTable
                anchors.fill: parent
            }
        }

        Row {
            id: toolbarRow
            anchors.bottom: contentArea.bottom
            anchors.bottomMargin: 20 * main.vScaleFactor
            anchors.left: contentArea.left
//            anchors.leftMargin: 287 * main.hScaleFactor

            height: 93 * main.vScaleFactor
            spacing: 30 * main.hScaleFactor

            Rectangle {
                id: resetTimerButton
                visible: true
                width: 140 * main.hScaleFactor
                height: parent.height
                border.color: main.colorUIBorder
                border.width: main.widthBorder
                color: main.colorButtonBackground
                radius: main.defaultRadius

                NumberLabel {
                    id: countDownText
                    anchors {
                        top: parent.top
                        topMargin: 10 * main.vScaleFactor
                        horizontalCenter: parent.horizontalCenter
                    }
                    text: StrFormat.formatTimeString(root.timeLeft)
                    font.pixelSize: 36 * main.vScaleFactor
                }

                TextLabel {
                    id: resetTimerText
                    anchors {
                        bottom: parent.bottom
                        bottomMargin: 13 * main.vScaleFactor
                        horizontalCenter: parent.horizontalCenter
                    }
                    width: 0.9*parent.width
                    text: qsTr("RESET TIMER") + application.emptyString
                    font.pixelSize: 13 * main.vScaleFactor
                    wrapMode: Text.WordWrap
                    horizontalAlignment: Text.AlignHCenter
                }
            }

            StandardButton {
                id: exportDataButton
                width: 100 * main.hScaleFactor
                height: parent.height
                visible: false // Hide functionality until 3rd party apps (eg. North Pole) are developed to export data to
                buttonText: qsTr("Export") + application.emptyString
                onClicked: root.doSave()

                icon.sourceName: "Bluetooth/BluetoothIcon";
            }

            StandardButton {
                id: signInRegisterButton
                height: parent.height
                enabled: main.loginAvailable
                visible: main.loginAvailable
                buttonText: qsTr("SIGN IN / REGISTER") + application.emptyString
                onClicked: main.state = "LoginState"

                icon.sourceName: "HomeScreen/SignInRegisterIcon"
            }

            Column {
                id: signInText
                height: parent.height
                width: 250 * main.hScaleFactor
                spacing: 6 * main.vScaleFactor

                enabled: main.loginAvailable
                visible: main.loginAvailable

                TextLabel {
                    id: signInRegisterText
                    text: qsTr("SPEED UP YOUR WORKOUT") + application.emptyString
                    font.pixelSize: 18 * main.vScaleFactor
                }

                Row {
                    id: bulletRow1
                    height: implicitHeight
                    width: implicitWidth

                    Text {
                        id: bullet1
                        anchors.verticalCenter: signInRegisterTextBullet1.verticalCenter
                        text: "●"
                        color: main.colorFontLight
                        font.pixelSize: 4 * main.vScaleFactor
                    }

                    Text {
                        id: signInRegisterTextBullet1
                        anchors.top: parent.top
                        text: qsTr(" Automatic export to your app!")
                        color: main.colorFontLight
                        font.pixelSize: 12 * main.vScaleFactor
                    }
                }

                Row {
                    id: bulletRow2
                    height: implicitHeight
                    width: implicitWidth

                    Text {
                        id: bullet2
                        anchors.verticalCenter: signInRegisterTextBullet2.verticalCenter
                        text: "●"
                        color: main.colorFontLight
                        font.pixelSize: 4 * main.vScaleFactor
                    }

                    Text {
                        id: signInRegisterTextBullet2
                        anchors.top: parent.top
                        text: qsTr(" More workout statistics!")
                        color: main.colorFontLight
                        font.pixelSize: 12 * main.vScaleFactor
                    }
                }

                Row {
                    id: bulletRow3
                    height: implicitHeight
                    width: implicitWidth

                    Text {
                        id: bullet3
                        anchors.verticalCenter: signInRegisterTextBullet3.verticalCenter
                        text: "●"
                        color: main.colorFontLight
                        font.pixelSize: 4 * main.vScaleFactor
                    }

                    Text {
                        id: signInRegisterTextBullet3
                        anchors.top: parent.top
                        text: qsTr(" Sign up today!")
                        color: main.colorFontLight
                        font.pixelSize: 12 * main.vScaleFactor
                    }
                }
            }

            // Points History will be implemented later
//            StandardButton {
//                id: eGymPointsButton
//                height: parent.height
//                visible: eGym.enabled && eGym.connected
//                buttonText: (eGym.pointsHistoryItem.length > 0 ? qsTr("EGYM POINTS HISTORY") : qsTr("SYNCHRONIZING WITH EGYM") )  + application.emptyString
//                onClicked: pointsPopup.show()
//            }
        }

        StandardButton {
            id: homeScreenButton
            anchors.right: contentArea.right
            anchors.rightMargin: 70 * main.vScaleFactor
            anchors.top: toolbarRow.top
            width: 120 * main.hScaleFactor
            height: toolbarRow.height
            visible: true
            buttonText: qsTr("HOME") + application.emptyString
            onClicked: close()

            icon.sourceName: "Components/HomeIcon"
        }
    }

    BlueToothLowEnergy {
        id: ble
        Component.onCompleted: console.log("ble." +ble.dummy)
    }

    BluetoothTransmitPopup {
        id: bleDialog
    }

    ModalDialog {
        id: pointsPopup

//        Component.onCompleted: hide()
        visible: main.state === "WorkoutSummaryState" && eGym.enabled && eGym.connected
        z: 99
        onVisibleChanged: console.log("Points pop up visible " + visible)

        Connections {
            target: main
            onStateChanged: {
                if(main.state === "WorkoutSummaryState") {
                    pointsPopup.visible = main.state === "WorkoutSummaryState" && eGym.enabled && eGym.connected
                }
            }
        }

        Item {
            anchors.fill: parent
            z: parent.z+1

            CloseButton {
                id: closeButton

                anchors {
                    right: parent.right
                    rightMargin: 10 * main.hScaleFactor
                    top: parent.top
                    topMargin: 10 * main.vScaleFactor
                }
                onClicked: pointsPopup.hide()
            }

            Column {
                id: egymCurrPoints
                height: childrenRect.height
                width: parent.width
                anchors.centerIn: parent
                spacing: 6 * main.vScaleFactor
                visible: eGym.enabled && eGym.connected

                TextLabel {
                    anchors.horizontalCenter: parent.horizontalCenter
                    text: (eGym.pointsHistory.length > 0 ? qsTr("EGYM POINTS EARNED") : qsTr("SYNCHRONIZING WITH EGYM") )  + application.emptyString
                    font.pixelSize: 42 * main.vScaleFactor
                }

                NumberLabel {
                    anchors.horizontalCenter: parent.horizontalCenter
                    visible: eGym.pointsHistory.length > 0
                    text: eGym.workoutPoints
                    font.pixelSize: 42 * main.vScaleFactor
                }
            }
        }
    }

    ModalDialog {
        id: pointsHistoryPopup

        property var points: eGym.pointsHistory

        Component.onCompleted: hide()

        onVisibleChanged: {
            if(visible)
                pointsHistory.updatePointsList()
        }

        Item {
            id: pointsHistoryItem
            anchors.fill: parent
            z: parent.z+1

            CloseButton {
                id: closeButton2

                anchors {
                    right: parent.right
                    rightMargin: 10 * main.hScaleFactor
                    top: parent.top
                    topMargin: 10 * main.vScaleFactor
                }
                onClicked: pointsHistoryPopup.hide()
            }

            ListModel {
                id: pointsList
            }

            property int maxY: computeMaxY()
            property int numPts: computeNumPts()

            function computeNumPts() {
                // Only show last 10 points
                return Math.min(pointsHistoryPopup.points.length, 10)
            }

            function computeMaxY() {
                var mxY = 1
                var i
                for(i=0; i < pointsHistoryPopup.points.length; i++) {
                    if(mxY < pointsHistoryPopup.points[i]["points"])
                        mxY = pointsHistoryPopup.points[i]["points"]
                }
                mxY = Math.floor(mxY*1.2)
                return mxY
            }

            function computeY(target) {
                var percent = target / maxY
                var y = percent * pointsView.height
                return y
            }

            function computeTime(t) {
                var d = new Date(t)
                var ts = d.toUTCString().slice(4,10)
                return ts
            }

            function updatePointsList() {
                var minI = pointsHistoryPopup.points.length - pointsHistoryItem.numPts;

                pointsList.clear()
                var i, p
                for(i=0; i < pointsHistoryItem.numPts; i++) {
                    p = pointsHistoryPopup.points[i+minI]["points"]
                    if(p > 0) {
                        pointsList.append({"timeStr": computeTime(pointsHistoryPopup.points[i+minI]["timestamp"]),
                                           "pointsY": computeY(p),
                                           "points": p})
                    }
                }
            }

            TextLabel {
                anchors {
                    top: parent.top
                    topMargin: 5 * main.vScaleFactor
                    horizontalCenter: parent.horizontalCenter
                }

                text: qsTr("POINTS HISTORY") + application.emptyString
                font.pixelSize: 24 * main.vScaleFactor
            }

            ListView {
                id: pointsView

                anchors {
                    top: parent.top
                    bottom: parent.bottom
                    left: parent.left
                    right: closeButton2.left
                    margins: 30 * main.scaleFactor
                }
                model: pointsList
                orientation: ListView.Horizontal

                property int barW: pointsView.width/pointsHistoryItem.numPts

                delegate: Component {
                    Rectangle {
                        anchors {
                            bottom: parent.bottom
                        }

                        width: pointsView.barW
                        height: pointsY
                        color: main.colorHighlightBackground
                        border.color: main.colorUIBorder
                        border.width: main.widthBorder

                        TextLabel {
                            anchors {
                                bottom: parent.bottom
                                bottomMargin: 15 * main.vScaleFactor
                                horizontalCenter: parent.horizontalCenter
                            }

                            text: timeStr
                            font.pixelSize: 12 * main.hScaleFactor
                            verticalAlignment: Text.AlignBottom
                            horizontalAlignment: Text.AlignLeft
                            rotation: 270
                        }

                        TextLabel {
                            anchors {
                                bottom: parent.top
                                bottomMargin: 5 * main.vScaleFactor
                                horizontalCenter: parent.horizontalCenter
                            }

                            text: points
                            font.pixelSize: 20 * main.hScaleFactor

                        }
                    }
                }
            }

            Connections {
                target: eGym
                onPointsHistoryChanged: {
                    pointsHistoryItem.updatePointsList()
                }
            }
        }
    }
}
