/** @file main.qml
    @brief Manage main display and transitions to workout menus and run screens.
*/

import QtQuick 2.12
import QtQuick.Window 2.12 // For Screen
import QtQuick.Controls 1.4
import WorkoutManager 1.0
import StorageMgr 1.0
import StatisticManager 1.0
import ErrorManager 1.0
import Centerpod 1.0
import WorkoutRunData 1.0
import Process 0.1
import AMixer 1.0
import DataExport 1.0
import Bluetooth 1.0
import HeartRateManager 1.0
import RecoveryManager 1.0
import SapManager 1.0
import Components 1.0
import FileIO 1.0
import QtQuick.Layouts 1.12
import "LanguageMenu"
import "Diagnostics"
import "MediaBrowser"
import "string_format.js" as StrFormat
import "Television"
import "Keypad"
import "WorkoutSetup"
import "WorkoutRun"
import "WorkoutSummary"
import "Prototype"
import "HomeScreen"
import "Networking"
import "SetupWizard"
import "views"
import "WorkoutSetup/setup_params.js" as SetupJS

/**
 * @brief the base QML file for the application.
 */

Item {
    id: main

    /* Some global properties */
    property string colorUIBorder: "#00a0df"//"#BA3E3E" old red
    property string colorSelected: "#00435e"//"#401515"
    property string colorDialogBackground: "black"
    property string colorToolbarBackground: "#002638"//"#383230"
    property string colorButtonBackground: "transparent"
    property string colorGenericStimulusArea: "red"
    property string colorKnobStimulusArea: "blue"
    property string colorTrackMouseStimulusArea: "green"
    property string colorHighlightBackground: "#00435e"//"#401515"
    property string colorFontGreen: "#5FFF00"
    property string colorFontLight: "white"
    property string colorFontDark: "black"
    property string colorDiagnosticEdges: "black"

    property real opacityDialog: 0.9
    property int widthDialog: 800 * main.hScaleFactor
    property int heightDialog: 350 * main.hScaleFactor
    /* To make our lives easier, make border width an even number, or else child
     * elements will clip borders due to borders being rendered at half pixels
     */
    property real widthBorder: ((width * 0.001) < 2 ? 2 : width * 0.001)
    property real defaultRadius: 10 * main.scaleFactor

    /**
     * Use SVG format images in ScalableImages
     */
    property bool useSVG: false

    /**
     * Set width and height based on screen size
     */
    width: Screen.width
    height: Screen.height
    /**
     * Default to 1366x768 with a pixel density of 2.91251 px/mm
     */
    readonly property int defaultWidth: 1366
    readonly property int defaultHeight: 768
//    readonly property real defaultPixelDensity: 2.91251
    /**
     * Calculate scale factors based on ratio of current screen to default and density of pixels
     */
    property real hScaleFactor: (width/defaultWidth)//*(Screen.pixelDensity/defaultPixelDensity)
    property real vScaleFactor: (height/defaultHeight)//*(Screen.pixelDensity/defaultPixelDensity)
    property real scaleFactor: Math.min(hScaleFactor, vScaleFactor)

    property bool modestyMode: options.modestyMode

    /**
     * @brief type:bool Boolean that determines if the login button should be visible
     */
//    property bool loginAvailable: true
    property bool loginAvailable: false

    /**
     * @brief type:bool Boolean that determines if a user is logged in.
     */
    property bool loggedIn: false

    /**
     * @brief type:variant Property that contains the topmenu model.
     */
    property variant topmenu

    /**
     * @brief type:variant Property that contains the All Workouts menu.
     */
    property variant allWorkoutsMenu

    /**
     * @brief type:bool Property that determines if a dialog box is shown.
     */
    property bool _dialogShowing: false

    /**
     * @brief type:bool Property that determines if the block keys are shown.
     */
    property bool blockKeys: rebootRequired || restartRequired || dialogManager.anyAreBlockingKeys || popupManager.estopShown || main.quickStartLoading
    property bool workoutKeysDisabled: false

    /**
     * @brief type:bool Property that determines if the stimulus area is shown.
     */
    property bool showStimulusArea: options.showStimulusArea

    /**
     * @brief type:int Property that determines the volume level to use.
     */
    property int volume: 5

    /**
     * @brief type:real Property that determines the time factor.
     */
    property real timeFactor: 1.0

    /**
     * @brief type:string Property that contains the application's serial number.
     */
    property string serialNumber: application.serialNumber

    /**
     * @brief type:string Property that contains the application's product version.
     */
    property string productVersion: application.productVersion

    /**
     * @brief type:bool Property that determines if a treadmill is being used.
     */
    property bool isTreadmill: machineinfo.productTypeString === "Treadmill"

    /**
     * @brief type:bool Property that determines if the machines used is an alpine runner.
     */
    property bool isStepmill: machineinfo.productTypeString === "Stepmill"

    /**
     * @brief type:bool Property that determines if the application's layout is right to left.
     */
    property bool rightToLeft: Qt.application.layoutDirection === Qt.RightToLeft

    /**
     * @brief type:string Property that determines the currently selected channel.
     */
    property string channelSelected: ""

    /**
     * @brief type:bool Property that determines if a restart of the machine is required.
     */
    property bool restartRequired: false  // Show error message and make user power cycle machine
    property bool rebootRequired: false  // Show error message and give user option to reboot console
    property bool syncRequired:false

    /**
     * @brief type:bool Property that determines if a firmware update is required.
     */
    property bool firmwareUpdateRequired: false

    /**
     * @brief type:bool Property that determines if the language button is visible.
     */
    property bool _hideLanguageButton: false

    /**
     * @brief type:int Property that holds the current screen shot count.
     */
    property int screenShotCount: 0

    /**
     * @brief type:string Property that holds the current console type such as Transcent10.
     */
    property string consoleType: application.consoleType

    /**
     * @brief type:bool Property that determines if each key press will be logged.
     */
    property bool _debugKeys: true

    /**
     * @brief type:bool Property that determines if effective radiated power (ERP) is supported.
     */
    property bool erpIsSupported: true

    /**
     * @brief type:bool Property that determines if loading workout from clicking the Quick Start button.
     */
    property bool quickStartLoading: false

    /**
     * @brief type:bool Property that determines if the machine meets CSA certification?
     */
    property bool isCSA: machineinfo.modelString === "TS1000" || machineinfo.modelString === "XS1000"

    /**
     * @brief type:bool Property that determines if the machine used is a lateral trainer.
     */
    property bool isLateralTrainer: machineinfo.modelString === "TL1000" || machineinfo.modelString === "XL1000"

    /**
     * @brief type:bool Property that determines if the machines used is an alpine runner.
     */
    property bool isAlpineRunner: machineinfo.modelString === "TI1000"

    /**
     * @brief type:bool Property that determines if the media browser is shown.
     */
    property bool showMediaBrowser: false

    /**
     * @brief type:bool Property that determines connection should be terminated
     */
    property bool killOnConnect: false

    /**
     * @brief type:variant Property that holds the home screen.
     */
    property variant home_screen: home_screen_loader.item

    /**
     * @brief type:variant Property that holds the workout menu.
     */
    property variant workout_menu: workout_menu_loader.item

    /**
     * @brief type:string Property that holds the lower screen board type.
     */
    property string lowerSystem: ""

    /**
     * @brief type:bool Property that determines if a user may begin a workout.
     */
    property bool canStartWorkout: true

    /**
      * @brief type:bool Property that determines if the user wants to select a new workout in the middle of a workout
      */
    property bool isNewWorkoutSelected: false

    /**
      * @brief type:bool Property that determines if the console is the primary console in a group of replica consoles
      */
    property bool isPrimary: false

    property bool showAppleWatchConnectsInfo: false

    property bool eGymCardioTest: false
    property bool eGymCardioTest_IntervalPending: false

    // Diagnostic/Testing tools Data
    property double pedalRPM_multiplier: 1.0

    property int screenHeight: height
    property int screenWidth: width

    property bool beltHasHadTimeToAccelerate: false

    focus: state == "HomeState" || state == "LoginState" || state == "RegisterState" || state == "DiagnosticState"

    /**
     * @brief Signal that indicates the state is to be reset.
     */
    signal reset()

    Component.onCompleted: {
        // The translation of the string "QT_LAYOUT_DIRECTION" should be "RTL" or "LTR" in the touch_*.ts files depending on the orientation of the language
        QT_TRANSLATE_NOOP("QApplication", "QT_LAYOUT_DIRECTION");
        // start on the home screen
        print("***** Application Start v" + application.appVersion + " *****");
        state = "HomeState";
        print("Getting Bios Version...");
        getBiosVerProcess.start();
        var lang = options.defaultLanguage;
        application.selectLanguage(lang);
        home_screen.updateLanguageIcon(lang);
        if(qsTranslate("QApplication", "QT_LAYOUT_DIRECTION") === "RTL") {
            main.rightToLeft = true
        } else {
            main.rightToLeft = false
        }

        console.log("language default: " + lang + " with orientation " + (main.rightToLeft ? "RTL" : "LTR"));

        workout_menu.selectWorkoutMenu();
        buzzer.buzzerEnabled = !options.disableBuzzer

        /* On start of application, set simulationMode and ignoreFaults to false */
        if (!options.manufacturingTestModeEnabled) {
            options.simulationMode = false;
            options.ignoreFaults = false;
        } else {
            options.ignoreFaults = true;
        }

        if (application.crashRecovery) {
            logUploadPrompt.show();
            logUploadPrompt.start();
        }

        bluetoothDeviceAudioWatcher.start()

        if(options.workoutAutoRestartInterval > 0) {
            // Automatically start Quickstart for TV testing
            console.log("===================================> Quickstart starting in " + autostartTimer.interval + " ms")
            autostartTimer.start()
        }

        // Reset Salt minion in case it got corrupted somehow
        resetMinion.start()

        gem.appleWatchEnabled = gem.nfc_Installed && options.enableNFC
        gem.enableAntPlus = options.enableAntPlus
    }

    Process {
        id: resetMinion
        program: "sudo"
        arguments: ["bash", "/srv/scripts/reset_minion.sh"]
        onStarted: {
            console.log("Resetting salt minion")
        }
        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim();
            console.log("Reset Salt Minion: ", data);
        }
        onFinished: {
            console.log("Reset salt minion complete")
        }
    }

    Timer{
        id: autostartTimer
        // Wait 50 seconds to let full incline level off
        interval: 50000
        running: false
        repeat: false
        onTriggered:{
            if (main.rebootRequired || main.restartRequired)
                return

            if(nucleus.systemReady) {
                console.log("automatically restarting workout ")
                quickStart()
            } else {
                autostartTimer.restart()
            }
        }
    }

    Timer{
        id: autostopTimer
        interval: options.workoutAutoRestartInterval*60*1000  // convert minutes to ms
        running: false
        repeat: false
        onTriggered:{
            workoutScreen.stopWorkout();
            autostartTimer.start()
        }
    }

    Component.onDestruction: {
        if (bluetoothAdapter.connected)
            bluetoothAdapter.disconnect()
        if (bluetoothAdapter.connectedHeadset)
            bluetoothAdapter.disconnectHeadset()
        console.log("Main: Component destroyed")
    }

    // update the layout direction when the language changes
    Connections {
        target: application
        onLanguageChanged: {
            if(qsTranslate("QApplication", "QT_LAYOUT_DIRECTION") === "RTL") {
                main.rightToLeft = true
            } else {
                main.rightToLeft = false
            }
            console.log("language orientation changed to: " + (main.rightToLeft ? "RTL" : "LTR"));
        }
    }

    Binding { target: application; property: "swipeStartX"; value: 0; when: main.state != "RunState" }
    Binding { target: application; property: "swipeEndX"; value: main.width; when: main.state != "RunState" }
    Binding { target: application; property: "swipeStartY"; value: 0; when: main.state != "RunState" }
    Binding { target: application; property: "swipeEndY"; value: main.height; when: main.state != "RunState" }

    Process {
        id: getBiosVerProcess
        Component.onCompleted: start()
        program: "sudo"
        arguments: ["cat", "/sys/class/dmi/id/bios_version"]
        onReadyReadStandardOutput: {
            var version = String(readAllStandardOutput())
            version = version.trim()
            print("Bios version is " + version)
        }
    }

    NetworkConnections {
        id: netConnections
        onIsRefreshingChanged: {
            if (!isRefreshing) {
                console.log("NetworkConnections: Finished refreshing connections")
                netConnections.printStatus()
            } else {
                console.log("NetworkConnections: Started refreshing connections")
            }
        }
    }

    Connections {
        target: nucleus
        onNucleusConnectionChanged: {
            if(connection == true) {
                nucleus.startInitializationProcess()
            }
        }
        onInitializationProcessFinished: {
            if(result == true) {
                workout_menu.selectWorkoutMenu()
                console.log("Nucleus: Diagnostics = " + options.loggingEnabled)
                nucleus.setDiagnostics(options.loggingEnabled)
                nucleus.cabSetEnable(options.cabEnabled)

                logger.logAlways("Installed Nucleus Firmware Version " + nucleus.firmwareVersionString + " App Version " + application.appVersion)

                // Retrieve Nucleus error logs
                console.log("Retrieving Nucleus fault log")
                errorManager.retrieveFaultLogs()
            }
        }

        onSystemReadyChanged: {
            console.log("Nucleus: System Ready = " + nucleus.systemReady)
            if(nucleus.systemReady === true) {
                canStartWorkout = true
            } else {
                canStartWorkout = false;
            }
        }
    }

    Connections{
        target: tv
        onTunerTypeChanged: {
            switch(tv.tunerType){
            case "ATSC":
                erpIsSupported = true;
                break;
            default:
                erpIsSupported = true;
                break;
            }
        }
    }

    /**
     * @brief Prints debug keys to the console log.
     * @param type:string keyname The name of the key pressed.
     * @param type:string objectname The name of the object.
     * @param type:string notes Any addtional information on the key.
     */
    function debugKeys(keyname, objectname, notes) {
        if (main._debugKeys) {
            console.log("KEYPRESS: ",  keyname, objectname, notes);
        }
    }

    /**
     * @brief Puts the ERP to sleep.
     */
    function erpSleep() {
        if (lowerSystem === "NUCLEUS") {
            console.debug("SENDING ERP SLEEP COMMAND TO NUCLEUS")
            nucleus.erpSleep()
        }
    }

    DialogManager {
        id: dialogManager
    }

    Process { id: syncDisk
        program: "/bin/sync"
        block: true
        blockTimeout: 1000 * 60 * 60; // 1 hour
        onFinished: {
            syncNoBlockTimer.restart()
        }
    }

    Timer {
        id: syncNoBlockTimer
        interval: 1000
        repeat: false
        running: false
        onTriggered: {
            syncNoBlock.start()
        }
    }

    Process {
        id: syncNoBlock
        program: "/bin/sync"
    }

    MessagePopup {
        id: beltStoppingPopup
        blockKeys: false
        isTransparent: true
        z:999

        /**
         * @brief type:bool Property that determines if the pop-up window is to be displayed.
         */
        property bool showPopup: main.state === "HomeState" && popupManager.estopShown === false
                                 && !options.manufacturingTestModeEnabled &&!nucleusReadyStatus()
                                 && popupManager.errorShown === false
        onShowPopupChanged: {
            if (showPopup) {
                show();
                console.log("Nucleus: starting ready timer.")
                check_nucleus_ready_timer.start()
            }
            else {
                hide();
                check_nucleus_ready_timer.stop()
                console.log("Nucleus: stopping ready timer.")
            }
        }
        message: qsTr("Please wait...") + "\n" + qsTr("Resetting.") + application.emptyString
        Component.onCompleted: hide()
    }

    function nucleusReadyStatus() {
        if(isTreadmill){
            if(!machinecontrol.isBeltMoving && !machinecontrol.isInclineMoving && nucleus.systemReady){
                return true
            } else {
                return false
            }
        }
        if(isStepmill) {
            if(!machinecontrol.isBeltMoving && nucleus.systemReady){
                return true
            } else {
                return false
            }
        }
        return nucleus.systemReady
    }

    Timer {
        id: check_nucleus_ready_timer
        interval: 60*1000
        running: false
        repeat: true
        onTriggered: {
            // Check nucleus status again
            if (!nucleusReadyStatus()) {
                logger.logAlways("Nucleus still not ready after " + check_nucleus_ready_timer.interval/1000 + "s. Retrieving Nucleus ready status again.")
                nucleus.querySystemReady()
            }
        }
    }

    MessagePopup {
        id: blockUseMessage
        z:999

        /**
         * @brief type:bool Property that determines if the pop-up window is to be displayed.
         */
        property bool showPopup: main.state !== "DiagnosticState" && main.state !== "SetupWizardState" && (rebootRequired || restartRequired || home_screen.buttonsEnabled === false)

        onShowPopupChanged: {
            if (showPopup) {
                logger.logAlways("Show BlockUseMessage: " + blockUseMessage.message)
                show();
            }
            else {
                logger.logAlways("BlockUseMessage hidden")
                hide();
            }
        }
        message: {
            if (main.restartRequired || main.rebootRequired) {
                return (application.crashRecovery ? qsTr("Power Cycle Required") : qsTr("Power Cycle Required to Complete Update")) + application.emptyString;
            } else if (!machineinfo.isReady) {
                return qsTr("Starting up...") + application.emptyString;
            } else if (!home_screen.buttonsEnabled) {
                if ((machineinfo.productTypeString === "") && (options.ignoreFaults == false)) {
                    return qsTr("Console Configuration Required.") + application.emptyString;
                } else if ((machineinfo.productTypeString === "Treadmill") && (machinecontrol.isCalibrated === false) && (options.ignoreFaults === false)) {
                    return qsTr("Treadmill Calibration Required.") + application.emptyString;
                } else if ((machineinfo.productTypeString === "") && (options.ignoreFaults == true)) {
                    return qsTr("Product Configuration Required.") + application.emptyString;
                } else {
                    return qsTr("Complete Configuration to Enable Buttons") + application.emptyString;
                }
            } else {
                return qsTr("Complete Configuration to Enable Buttons") + application.emptyString;
            }
        }
        Component.onCompleted: if (showPopup) show(); else hide()
        isTransparent: true

        StandardButton {
            property bool showButton: true
            visible: main.rebootRequired && showButton
            z: parent.z + 1
            height: 50 * main.vScaleFactor
            anchors {
                horizontalCenter: parent.horizontalCenter
                bottom: parent.bottom
                bottomMargin: 25 * main.vScaleFactor
            }

            buttonText: qsTr("Reboot Now")
            onClicked: {
                main.startReboot()
                showButton = false
                blockUseMessage.message = qsTr("Rebooting now!")
            }
        }
    }

    MessagePopup {
        id: gemStatusPopup
        z: 999

        property bool showPopup: options.manufacturingTestModeEnabled && main.state === "HomeState"

        onShowPopupChanged: {
            if (showPopup) {
                logger.logAlways("Show GEM Status: " + gemStatusPopup.message)
                show();
            }
            else {
                logger.logAlways("GEM Status hidden")
                hide();
            }
        }

        message: {
            return qsTr("GEM Firmware version: ") + gem.firmwareVersion +
                    "\n NFC " + (gem.nfc_Installed ? qsTr("is installed") : qsTr("is not installed")) + application.emptyString
        }

        Timer {
            running: gemStatusPopup.showPopup
            interval: 10 * 1000
            onTriggered: {
                gemStatusPopup.showPopup = false
            }
        }

        Connections {
            target: gem
            onFirmwareVersionChanged : {
                gemStatusPopup.showPopup = options.manufacturingTestModeEnabled
            }
        }
    }

    MiscStats {
        id: miscStats
    }

    WorkoutStats {
        id: workoutStats
    }

    DataExport {
        id: dataExport
        workoutProgram: workoutProgram
        workoutData: aWorkoutRunData
        profile1: workoutProgram.profile1
        profile2: workoutProgram.profile2
        productModel: machineinfo.modelString
//        machineID: netpulse.machineID
        serialNumber: main.serialNumber
        runMiles: statisticManager.totalDistance
        runHours: statisticManager.runTime / 3600  // stats manager keeps "runTime" in seconds
        modelID: 0 //configurationData.csafeID
        version: application.appVersion
        Component.onCompleted: print("ip address:" + ipAddress)
    }

//    SignIn {
//        id: signin_window
//        z:99
//        visible: (main.state === "LoginState");
//    }

//    CreateProfile {
//        id: createprofile_window
//        visible: (main.state === "RegisterState");
//    }

    Connections{
        target: tv
        onTunerTypeChanged: {
            switch(tv.tunerType){
                case "ATSC":
                    erpIsSupported = true;
                    break;
                default:
                    erpIsSupported = true;
                    break;
            }
        }
    }

    // expire Login after 30 seconds with no activity on homescreen
    Timer {
        id: loginTimeoutTimer
        running: (main.state == "HomeState")
        interval: 30 * 1000
        onTriggered: { // TODO: write a logout function
            reset();
        }
        /**
         * @brief Calls the restart function if currently running.
         */
        function reset() {
            if(running) {
                restart();
            }
        }
    }

    Connections {
        // reset the login timer when there's a screen touch
        target: application
        onScreenTouched: loginTimeoutTimer.reset()
    }

    AMixer {
        id: amixer
    }

    StatisticManager {
        id:statisticManager
        consoleSerialNumber: main.serialNumber;

        onTotalDistanceChanged: errorManager.updateMileage(totalDistance)
        Component.onCompleted: {
            setNucleusObject(nucleus);
        }
    }

    MessagePopup {
        id: infoPopup
        z: 10000
        dismissable: true
        dismissAnywhere: true
        isTransparent: true
    }

    MessageBanner {
        id: infoBanner
        z: 10000
        anchors.horizontalCenter: main.horizontalCenter
        y:0
        dismissable: true
        dismissAnywhere: true
        isTransparent: true
    }

    SimpleMessageBanner {
        id: viewContainerBanner
        z: 10000
        hideWhen: !main.isCSA && !main.isLateralTrainer && workoutScreen.container.currentViewName === "C360_VIEW"
        x:185 * main.hScaleFactor
        y:170 * main.vScaleFactor
        width:1000 * main.hScaleFactor
        dismissable: true
        dismissAnywhere: true
        isTransparent: true
    }

    Connections {
        target: machinecontrol
        onEstopChanged: {
            logger.logAlways("MachineControl: Safety Key " + (machinecontrol.estop ? "In" : "Out"))
        }
        onSafetyKeyQuickToggle: {
            miscStats.incrementUsage("Safety Key Toggle")
            logger.logAlways("MachineControl: Safety key quick toggle detected")
            logger.dumpLastLog()
        }
        onTreadmillInclinePercentGradeTargetChanged: {
            workoutManager.treadIncline = machinecontrol.treadmillInclineTarget
            workoutManager.treadInclineGUI = machinecontrol.treadmillInclineTarget
        }
    }

    PopupManager{
        id: popupManager

//        safetyKeyOut: !machinecontrol.estop && (isTreadmill || isStepmill)
        safetyKeyOut: !machinecontrol.estop && isTreadmill
        // hide pop-ups while in diagnostic screen and other situations
        hidePopup: main.state === "DiagnosticState" || main.state === "SetupWizardState"  || main.restartRequired || main.rebootRequired || options.ignoreFaults || main.firmwareUpdateRequired
        treadStopped: machinecontrol.treadmillSpeedActual < 1
        onEstopPopupTimedout: console.debug("estop timed out")

        Connections {
            target: machinecontrol

            onEstopChanged: {
                if(main.lowerSystem === "NUCLEUS" && estop === true && (isTreadmill || isStepmill)) {
                    popupManager.safetyKeyOut = false
                }
            }

            onSafetyKeyPulled: {
                //console.debug("key pulled: " + n)
                if(main.lowerSystem === "NUCLEUS" && ((isTreadmill) || (isStepmill)) ) {
                    popupManager.safetyKeyOut = true
                }
            }
        }
    }

    Connections {
        target: nucleusfirmwareloader
        onIsUpdatingChanged: errorManager.isNucleusFirmwareUpdating = nucleusfirmwareloader.isUpdating
    }

    Centerpod{
        id:centerpod

        onStartPressChanged:
        {
            if(centerpod.startPress==true && quickStartLoading==false )
            {
               if( machinecontrol.isBeltMoving === false && machinecontrol.isInclineMoving === false && bluetoothAdapter.connecting == false && nucleus.systemReady === true || (nucleus.systemReady === false && options.manufacturingTestModeEnabled === true))
                quickStart();
             }
        }

        Component.onCompleted:
        {
            setNucleus(nucleus)
        }
    }

    ErrorManager{
        id:errorManager

        /**
         * @brief type:bool Property that determines if a fatal error has occured.
         */
        property bool fatalError: false

        /**
         * @brief type:bool Property that determines if an error occured that prevents the use of the machine.
         */
        property bool preventUse: false

        /**
         * @brief type:bool Property that determines if an error occured that results in a degraded use.
         */
        property bool degradedUse: false

        onErrorEvent: {
            if(options.ignoreFaults) {
                console.log("Ignoring error event: " + errorcode + " " + errormessage);
                return;
            }

            logger.logAlways("Error event: " + errorcode + " " + errormessage);
            logger.dumpLastLog();

            if(errorcode === "CN25") {
                main.firmwareUpdateRequired = true;
            }

            if(severity === ErrorManager.ERROR_FATAL) {
                fatalError = true;
            }

            if(severity >= ErrorManager.ERROR_PREVENTS_USE) {
                preventUse = true;
            } else if(severity >= ErrorManager.ERROR_DEGRADES_USE) {
                degradedUse = true;
            }

            if(!options.ignoreFaults) {
                var translatedErrorMessage = qsTranslate("ErrorManager", errormessage);
                popupManager.setErrorMessage(errorcode, translatedErrorMessage, severity);

                // briefly display new error at top of screen if pop-ups are hidden (as in diagnostic screens.)
                if(popupManager.hidePopup) {
                    infoBanner.showMessage(popupManager.currentMessage, 5);
                }

                if (errorcode === "X101" || errorcode === "X102") {
                    console.log("Stride Fault");
                    machinecontrol.strideFaulted = true;
                }
            }
        }

        onNucleusNonRecoverableFaultTriggered: {
            preventUse = true;
            popupManager.setNucleusFaultTriggeredMessage(msg);

            if(main.state === "RunState") {
                workoutScreen.stopWorkout();
            }

            if(popupManager.hidePopup) {
                infoBanner.showMessage(popupManager.currentMessage, 5);
            }
        }

        onNucleusRecoverableFaultTriggered: {
            preventUse = true;
            popupManager.setNucleusFaultTriggeredMessage(msg);

            if(main.state === "RunState") {
                /* To Do: faults not implemented in Stepmill yet */
                if(!isStepmill)
                    workoutScreen.stopButtonPressedHandler();
            }
        }

        onNucleusFaultRecovery: {
            preventUse = true;
            popupManager.setNucleusFaultRecoveringMessage(msg);
        }

        onNucleusFaultCleared: {
            preventUse = false;
            popupManager.reset()
            if(options.workoutAutoRestartInterval > 0 && workoutScreen.stopPrompt.visible) {
                // resume workout && Set incline to max
                console.log("Resuming Workout & automatically setting incline to " + workoutProgram.maxIncline)
                workoutManager.treadIncline = workoutProgram.maxIncline * 10

                workoutScreen.resumeWorkout()
                workoutScreen.stopPrompt.hide()
            }
        }

        onNucleusWarningTriggered: {
            infoBanner.showMessage(msg, 5);
        }

        onNucleusWarningCleared: {
            infoBanner.showMessage(msg, 5);
        }

        Component.onCompleted: {
            setNucleusObject(nucleus);
            setNucleusFaultInfoManager(nucleusFaultInfoMgr);
        }
    }

    StorageMgr{
        id:storageMgr
    }

    RemoteAccessDialog {
        id: accessDialog

        /**
         * @brief type:bool Property that determines if the remote access dialog is shown.
         */
        property bool showMe: main.state === "HomeState" && remoteAccess.isEnabled

        onShowMeChanged: {
            if (showMe) {
                show();
            } else {
                hide();
            }
        }
        isTransparent: true
    }

    RemoteAccess {
        id: remoteAccess
    }

    SoftwareUpdatePrompt {
        id: softwareUpdatePrompt

        /**
         * @brief type:bool Property that determines if the device is loaded.
         */
        property bool deviceLoaded: storageMgr.usbDeviceLoaded

        prompt: qsTr("Update Software Now?") + application.emptyString
        x: 300 * main.hScaleFactor
        y: 200 * main.vScaleFactor
        onNoClicked: hide()
        Component.onCompleted: hide()

        onDeviceLoadedChanged: {
            if (main.state != "HomeState") {
                return;
            }

            if (storageMgr.usbDeviceLoaded) {
                console.log("USB Device Loaded", storageMgr.usbDevicePath, storageMgr.usbBlockDevicePath);
                var hasPackages = storageMgr.checkForPackagesFile();
                console.log(hasPackages ? "Has Packages" : "No Packages");

                if (hasPackages) {
                    softwareUpdatePrompt.showPrompt();
                }
            } else {
                console.log("USB Device Removed", storageMgr.usbDevicePath, storageMgr.usbBlockDevicePath);

                if (softwareUpdatePrompt.visible) {
                    softwareUpdatePrompt.hide();
                }
            }
        }
    }

    Loader {
        id: screensaver
        z:1000
        anchors.fill: parent
        visible: false

        Text {
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.bottom: parent.bottom ; anchors.bottomMargin: 20 * main.vScaleFactor
            font.pointSize: 40
            color: "transparent"
            z:99
            text: qsTr("Touch to Begin") + application.emptyString
        }

        /**
         * @brief Starts the screen saver.
         */
        function start() {
            visible = true;
            screensaver.source = "HomeScreen/Slideshow.qml";
            screensaver.item.zoomEffect = true;
            screensaver.item.fadeEffect = true;
            screensaver.item.slideInterval = 10000;
        }

        /**
         * @brief Stops the screen saver.
         */
        function stop() {
            visible = false;
            screensaver.source = "";

            if ((machineinfo.modelString === "----" && !setupWizard.isNucleus) || (setupWizard.isNucleus && !options.setupWizardCompleted)) {
                main.state = "SetupWizardState";
            } else {
                main.state = "HomeState";
            }
        }

        Connections {
            target: machinecontrol
            // wake up out of screen saver when pedaled.
            onPedalRPMChanged: {
                if(pedalRPM > 0) {
                    if(main.state === "ScreenSaverState" || main.state === "SleepModeState") {
                        screensaver.stop();
                    } else if (main.state === "HomeState") {
                        main.lcdBrightness = options.lcdBrightness;

                        if (!options.screenSaveTimeout && options.dimDisplayTimeout) {
                            dim_timer.restart();
                        }
                    }
                }

                if(workoutScreen.workoutActive && !(main.isTreadmill || main.isStepmill)) {
                    // update the pace profile with current pace during a workout.
                    aWorkoutRunData.setRPM(pedalRPM_multiplier * pedalRPM);

                    if(!options.enableProtoGraphs)
                        workoutProgram.profile2.setProperty(workoutProgram.profile2.currentSegment, "segmentMagnitude", pedalRPM);
                }
            }
            onDisplayWattsChanged: {
                if(workoutScreen.workoutActive) {
                    if(displayWatts > 1000) {
                        console.log("Error: displayWatts out of range! " + dispayWatts)
                    }
                    if(isStepmill){
                        aWorkoutRunData.setWatts(displayWatts);
                        workoutManager.setWatts(displayWatts);
                        return
                    }

                    aWorkoutRunData.setWatts(pedalRPM_multiplier * displayWatts);
                    workoutManager.setWatts(pedalRPM_multiplier * displayWatts);
                }
            }
            onStrideLengthTargetChanged: {
                workoutManager.strideLevel = strideLengthTarget;
            }
        }

        MouseArea {
            anchors.fill: parent
            onClicked: screensaver.stop()
        }
    }

    /**
     * @brief Starts a workout using the quick start workout.
     * @param type:string filename The quick start file to load/use.
     */
    function quickStart(filename) {
        if ((isTreadmill || isStepmill) && machinecontrol.treadmillSpeedActual !== 0) {
            return;
        }

        if (main.state === "SleepModeState") return;
        if (softwareUpdatePrompt.active) return;
        if (logUploadPrompt.visible) return;
        if (blockUseMessage.visible) return;
        if (accessDialog.visible) return;
        if (main.quickStartLoading) return; // prevent multiple consecutive loads

        main.quickStartLoading = true;
        print("Quickstart " + (filename ? filename : ""));
        eGym.selectedExerciseName = "QuickStart"
        main.channelSelected = "";

        if( main.state === "HomeState" || main.state == "ScreenSaverState" || main.state === "MenuState" ) {
            if(main.state === "ScreenSaverState") {
                screensaver.stop();
            }

            ipod_window.close();

            if(isTreadmill) {
                if(!filename) {
                    filename ="Tread_Quick_Start.xml";
                }
            }else if (isStepmill) {
                if(!filename) {
                    filename ="Step_Quick_Start.xml";
                }
            } else {
                if(!filename) {
                    filename = "Quick_Start.xml";
                }
            }


            workoutStats.incrementUsage(qsTr("Quick Start"));
            workoutProgram.loadWorkoutFileAsync("Program_Files/" + filename);
            aWorkoutRunData.programName = qsTr("QUICK START");
            centerpod.setQuickStart(true);
            centerpod.setPausePress(false)
          }
    }

    /**
     * @brief Starts a given workout?
     * @param type:string item The workout item to load?
     */
    function startWorkout(item) {
        main.channelSelected = "";
        main.clearQLists();
        var programFile = item.program;
        workoutStats.incrementUsage(item.label);

        state = "LoadState";

        if(programFile === "Program_Files/eGymCardioTest.xml" ||
           programFile === "Program_Files/Tread_eGymCardioTest.xml" ||
           programFile === "Program_Files/Step_eGymCardioTest.xml") {
            main.eGymCardioTest = true
        }

        loadingProgramNotice.programItem = item;
        workoutProgram.loadWorkoutFileAsync(programFile);
    }

    Connections {
        target: workoutProgram
        onParsingFileChanged: {
            // saved programs are handled by the saved programs browser, so skip them
            if (!workoutProgram.parsingFile) {
                finishLoading(loadingProgramNotice.programItem);
            }
        }
    }

    /**
     * @brief Finishes loading a workout?
     * @param type:string The item to load?
     */
    function finishLoading(item) {
        if(!workoutProgram.isQuickStart) {
            mainProgItem.setProgramDetails(item);

            aWorkoutRunData.programName = item.label;
            main.state = "SetupState";
        } else {
            aWorkoutRunData.programName = qsTr("QUICK START");
            workoutStartManager.conditionalStart();
        }
    }

    /**
     * @brief Starts a workout conditionally?
     */
    function conditionalStart() {
        workoutManager.resetLevels();
        aWorkoutRunData.reset(main.isNewWorkoutSelected);
        workoutManager.setupWorkout();
        main.state = "RunState";
        workoutScreen.start();
        if(main.eGymCardioTest)
            eGym.testInProgress = true;

        if(main.isTreadmill || main.isStepmill) {
            main.workoutKeysDisabled = true
            testwarning.show();
        } else {
            workoutManager.start(main.eGymCardioTest);
        }
    }

    function delay(duration){
        var timeStart = new Date().getTime();

        while (new Date().getTime() - timeStart < duration){
            // do nothing
        }
    }


    Timer{
        id: initializationRefreshTimer
        interval: 4000
        running: false
        repeat: true
        onTriggered:{
            if(main.state === "HomeState"){
                if(isTreadmill){
                    if(machinecontrol.treadmillSpeedTarget != 0 ||
                            machinecontrol.treadmillSpeedActual !=0){
                        machinecontrol.treadmillSpeedTarget = 0;
                        console.debug("Belt really ran Away, refreshing init")
                        nucleus.startInitializationProcess();
                    }
                } else if (isStepmill){
                    if(machinecontrol.stepmillSpeedTarget != 0 ||
                            machinecontrol.stepmillSpeedActual !=0){
                        console.debug("Steps really ran Away, refreshing init")
                        nucleus.startInitializationProcess();
                    }
                }
            } else if (main.state === "RunState"){
                if(isTreadmill && beltHasHadTimeToAccelerate && !workoutScreen.stopPrompt.visible){
                    if(machinecontrol.treadmillSpeedTarget == 0 &&
                            machinecontrol.treadmillSpeedActual != 0){
                        machinecontrol.treadmillSpeedTarget = 0;
                    }
                    if(machinecontrol.treadmillSpeedTarget == 0 ||
                            machinecontrol.treadmillSpeedActual == 0){
                        console.debug("speed issue in workout, refreshing init")
                        nucleus.startInitializationProcess();
                    } else if (machinecontrol.treadmillSpeedTarget != 0 && machinecontrol.treadmillSpeedActual == 0){
                        workoutScreen.stopButtonPressedHandler();//stopPause workout
                    }
                } else if (isStepmill && beltHasHadTimeToAccelerate){
                        if(machinecontrol.stepmillSpeedTarget == 0 ||
                                machinecontrol.stepmillSpeedActual == 0){
                            console.debug("speed issue in workout, refreshing init")
                            nucleus.startInitializationProcess();
                        }
                }
                beltHasHadTimeToAccelerate = true;
            }
        }
    }

    TreadmillWorkoutCountdownWarning {
        id:testwarning
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        onJumpStart: {
            workoutManager.jumpStart();
            initializationRefreshTimer.start();
            //start init refresh timer (if target speed stays 0 for too long, re-init)
        }
        onTimedOut: {
            workoutManager.start(main.eGymCardioTest);
            hide();
        }
        onVisibleChanged: {
            if(visible) {
                delayTimer.start();
                tv.on=false
            } else {
                changeTvState.start()
                countDown = 3; // reset counter when hiding dialog
            }
        }
        onTriggered: buzzer.warningBeep()
        Component.onCompleted: hide()

        Connections {
            target: workoutScreen
            /* Hide the dialog, which stops count, when workout active status
             * changes to false during count down.
             */
            onWorkoutActiveChanged: {
                if(testwarning.visible) {
                    testwarning.hide();
                }
            }

            onWorkoutStarted: {
                nucleus.workoutStart(workoutProgram.programIndex, workoutProgram.userWeight,
                                     workoutManager.workoutData.timeRemaining, workoutManager.workoutData.distanceRemaining*10,
                                     workoutManager.workoutData.calories)
                gem.startWorkout()
            }
            onWorkoutStopped: {
                testwarning.hide()
            }
            onWorkoutPaused: {
                nucleus.workoutPause();
                gem.pauseWorkout()
                testwarning.hide()
            }
            onWorkoutResumed: {
                nucleus.workoutResume()
                gem.startWorkout()
            }
        }

        Timer{
            id: delayTimer
            interval: 350
            running: false
            repeat: false
            onTriggered: {
                testwarning.startTimeout(3);
                buzzer.warningBeep();
            }
        }
    }

    /**
     * @brief Clears all queue lists?
     */
    function clearQLists() {
        workoutManager.reset();
        mainProgItem.clearListView(); // Use to reset setup items
        aWorkoutRunData.reset(main.isNewWorkoutSelected);
    }

    Connections {
        target: machineinfo
        onModelStringChanged: {
            // Re-select workout menu
            workout_menu.selectWorkoutMenu()
            gem.equipmentType = machineinfo.productTypeString;
        }
    }

    ImageButton {
        id: screen_lock_button
        visible: false
        enabled: false
    }

    ScalableImage {
        id: background
        z: -2
        visible: main.state !== "SetupWizardState" && main.state !== "DiagnosticState"
        anchors.fill: parent
        sourceName: "StandardPage/View_Background"
    } // end Image background

    AttractVideo {
        id: videoRunner
        z: -2
        visible: main.state === "HomeState" && errorManager.preventUse == false && home_screen.buttonsEnabled === true && popupManager.estopShown === false
        anchors.fill: parent
    }

    Item {
        id: workoutStartManager

        /**
         * @brief type:bool Property that determines if the machine has been engaged such as a treadmill safety circuit?
         */
        property bool engaging: false

        /**
         * @brief Starts the workout conditionally?
         */
        function conditionalStart() {
            if((main.isTreadmill)  && lowerSystem === "NUCLEUS") {
                if(engaging === false) {
                    engaging = true;
                    machinecontrol.safetyCircuitClose();
                } else {
                    main.conditionalStart();
                }
            } else {
                main.conditionalStart();
            }
        }

        Connections {
            target: machinecontrol
            onSafetyCircuitEngaged: {
                if(workoutStartManager.engaging === true) {
                    workoutStartManager.engaging = false;
                    main.conditionalStart();
                }
            }
            onSafetyCircuitDisengaged: {
                if(workoutStartManager.engaging === true) {
                    workoutStartManager.engaging = false;
                    main.quickStartLoading = false;
                }
            }
            onSafetyKeyInserted: {
                if(workoutStartManager.engaging === false) {
                    workoutStartManager.engaging = true;
                }
            }
        }
    }

    Item {
        id: diagnoticScreenEntry
        anchors.fill: parent
        visible: (popupManager.estopShown === true || errorManager.preventUse === true || blockUseMessage.visible === true) && main.state === "HomeState"


        Rectangle {
            id: upperLeftCorner
            color: "transparent"
            height: 200
            width: height
            anchors.top: parent.top
            anchors.left: parent.left
            MouseArea {
                anchors.fill: parent
                onPressAndHold: {
                    buzzer.secretBeep(0)
                    accessKeyPad.visible = true
                }
                Rectangle {
                    anchors.fill: parent
                    color: main.colorGenericStimulusArea
                    opacity: 0.3
                    visible: main.showStimulusArea
                }
            }
        }
    }

    Loader {
        id: home_screen_loader
        sourceComponent: touch_home_screen_component
        anchors.fill: parent
        anchors.rightMargin: 0
        anchors.bottomMargin: 0
        anchors.leftMargin: 0
        anchors.topMargin: 0
        z: 4
        opacity: 0
        enabled: (visible && opacity > 0.0)
        Component.onCompleted: {
            item.updateLanguageIcon(options.defaultLanguage);
            diagnostics.enableUsbLog(options.usbLoggingEnabled)
        }

        Component {
            id: touch_home_screen_component

            HomeScreen {
                id: touch_home_screen
                x: 0
                y: 0
                opacity: 1
                visible: !application.crashRecovery && !popupManager.estopShown && !errorManager.preventUse && !blockUseMessage.visible && !accessDialog.visible
                /**********************************************************************
                  Truth table for buttonsEnabled
                  +-------------------+-------------------+------------++----------------+
                  | productTypeString | treadIsCalibrated | standalone || buttonsEnabled |
                  +-------------------+-------------------+------------++----------------+
                  |         ""        |     don't care    |   false    ||     false      |
                  |       "Bike"      |     don't care    |   false    ||     true       |
                  |    "Elliptical"   |     don't care    |   false    ||     true       |
                  |    "Treadmill"    |        true       |   false    ||     true       |
                  |    "Treadmill"    |        false      |   false    ||     false      |
                  |    "Stepmill"     |     don't care    |   false    ||     true       |
                  |         ""        |     don't care    |    true    ||     false      |
                  |       "Bike"      |     don't care    |    true    ||     true       |
                  |    "Elliptical"   |     don't care    |    true    ||     true       |
                  |    "Treadmill"    |        true       |    true    ||     true       |
                  |    "Treadmill"    |        false      |    true    ||     true       |
                  |    "Stepmill"     |     don't care    |    true    ||     true       |
                  +-------------------+-------------------+------------++----------------+
                 **********************************************************************/
                buttonsEnabled: !(((machineinfo.productTypeString === "") && (options.ignoreFaults === false)) ||
                    (main.isTreadmill && (machinecontrol.isCalibrated == false) && (options.ignoreFaults == false)) ||
                    ((machineinfo.productTypeString === "") && (options.ignoreFaults == true)))
                                || options.manufacturingTestModeEnabled === true
                showCardio360: machineinfo.productType === WorkoutProgram.ELLIPTICAL || machineinfo.productTypeString === "Lateral Trainer"
                showMonumentsButton: main.isAlpineRunner || main.isStepmill
                onQuickstartPressed: {
                    quickStart();
                    diagnostics.logDiagnotsticEvent("XXXX","APP","Button","QuickStart","");
                }
                onC360QuickstartPressed: {
                    if (main.isLateralTrainer) {
                        main.quickStart("Cardio_360_Quick_Start_LateralTrainer.xml")
                    } else {
                        main.quickStart("Cardio_360_Quick_Start.xml")
                    }
                }
                onMonumentsPressed: {
                    main.state = "MenuState";
                    workout_menu.selectMenu("MONUMENTS")
                }
                onLoginPressed: main.state = "LoginState"
                onRegisterPressed: main.state = "RegisterState"
                onSignoutPressed: {
                    main.state = "HomeState";
                }
            }
        }
    }

    // Display the workout program image while program loads from XML
    Item {
        id: loadingProgramNotice
        anchors.fill: parent
        opacity: 0
        enabled: (visible && opacity > 0.0)

        /**
         * @brief type:variant Property containing the items for a workout program.
         */
        property variant programItem : undefined

        TextLabel {
            id: loadingProgramLabel
            text: loadingProgramNotice.programItem ? qsTranslate("Menu", loadingProgramNotice.programItem.label) : ""
            font.pointSize: 14 * main.vScaleFactor
            anchors {
                horizontalCenter: parent.horizontalCenter
                verticalCenter: parent.verticalCenter
            }
        }

        ProgressBar {
            id: progressBar
            height: 40 * main.vScaleFactor
            width: 360 * main.hScaleFactor
            value: workoutProgram.parsingProgress
            anchors {
                horizontalCenter: parent.horizontalCenter
                top: loadingProgramLabel.bottom
                topMargin: 10 * main.vScaleFactor
            }
            opacity: 0.5
        }
    }

    Loader {
        id: workout_menu_loader
        sourceComponent: touch_workout_menu_component
        anchors.fill: parent
        focus: main.state === "MenuState"
        opacity: 0
        enabled: (visible && opacity > 0.0)

        Component {
            id: touch_workout_menu_component

            WorkoutFinder {
                id: touch_workout_menu
                anchors.fill: parent
                onWorkoutSelected: {
                    startWorkout(item);
                }
            }
        }
    }

    AudioManager {
        id: audioManager
        debug: true
        tvPlaying: tv.playing && !tv.isIpodInput
        ipodPlaying: false
        ipodPlayingVideo: false
        browserPlaying: browserAudioWatcher.soundIsPlaying
        btconnected: bluetoothAdapter.connected
        onAudioSourcesChanged: print("audio sources: " + audioSources)
    }

    Process {
        id: bluetoothDeviceAudioWatcher

        /**
         * @brief type:bool Property that determines if sound is playing from the web browser.
        */
        property bool available: false

        program: "scripts/watch_btdev_audio.sh"
        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim();
            console.log("bluetoothDeviceAudioWatcher stdout", data);

            if (data === "yes") {
                bluetoothDeviceAudioWatcher.available = true;
            } else {
                bluetoothDeviceAudioWatcher.available = false;
            }
        }
        onFinished: {
            console.log("bluetoothDeviceAudioWatcher finished");
            bluetoothDeviceAudioWatcher.available = false;
        }
    }

    Process {
        id: browserAudioWatcher

        /**
         * @brief type:bool Property that determines if sound is playing from the web browser.
        */
        property bool soundIsPlaying: false

        program: "scripts/watch_browser.sh"
        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim();
            console.log("browserAudioWatcher stdout", data);

            if (data === "yes") {
                browserAudioWatcher.soundIsPlaying = true;
            } else {
                browserAudioWatcher.soundIsPlaying = false;
            }
        }
        onFinished: {
            console.log("browserAudioWatcher finished");
            browserAudioWatcher.soundIsPlaying = false;
        }
    }

    Process {
        id: browserMute
        program: "scripts/browsermute.sh"

        property bool muted: false
        property bool _flag: false

        /**
                 * @brief Mutes the sound.
                 * @param type:string Flags for muting sound?
                 */
        function setMute(flag) {
            _flag = flag
            browserMute.arguments = [flag];
            browserMute.start();
        }
        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim();
            console.log("output from browsermute:", data);
            muted = _flag
        }
    }

    Process {
        id: vlcMute
        program: "scripts/vlcmute.sh"

        /**
         * @brief Mutes the sound.
         * @param type:string Flags for muting sound?
         */
        function setMute(flag) {
            vlcMute.arguments = [flag]
            vlcMute.start()
        }
        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim();
            console.log("output from vlcMute:", data);
        }
    }

    Connections {
        target: audioManager
        onVideoSourceChanged: {
            console.log("Video Source is: " + audioManager.videoSource);
        }
        onTvVideoAllowedChanged: {
            muteTimer.stop();
            muteTimer.start();
        }
        onVirtActiveVideoAllowedChanged: {
            muteTimer.stop();
            muteTimer.start();
        }
    }

    Timer {
        id: muteTimer
        running: false
        interval: 1000
        onTriggered: {
            running = false
            if(audioManager.tvVideoAllowed || audioManager.virtActiveVideoAllowed || audioManager.ipodVideoAllowed || (audioManager.ipodAllowed && bluetoothAdapter.connectedHeadset)) {
                console.log("Unmuting VLC");
                vlcMute.setMute(0);
            } else {
                console.log("Muting VLC");
                vlcMute.setMute(1);
            }
        }
    }

    Connections {
        target: audioManager
        onBtAllowedChanged: {
            if (audioManager.btAllowed) {
                btMute.setMute(0);
            } else {
                btMute.setMute(1);
            }
        }
    }

    Process {
        id: btMute
        program: "scripts/btmute.sh"

        /**
         * @brief Mutes the sound.
         * @param type:string Flags for muting sound?
         */
        function setMute(flag) {
            btMute.arguments = [flag];
            btMute.start();
        }

        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim();
            console.log("output from btmute:", data);
        }
    }

    Television {
        id: tv
        onClicked: {
            buzzer.buttonPress()
            tv.fullscreen = tv.fullscreen ? false: true
        }
        muted: false
        verbose: false

        Connections {
            target: audioManager
            onTvVideoAllowedChanged: {
                if(audioManager.tvVideoAllowed) {
                    tv.tuneChannelID(tv.currentChannel);
                } else {
                    tv.stop();
                }
            }

            onTvAllowedChanged: {
                tv.muted = !audioManager.tvAllowed;
                print("tv muted:" + audioManager.tvAllowed);
            }
        }
    }

    ModalDialog {
        id: rebootMessage

        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter

        blockKeys: true
        property bool showPopup: false
        onShowPopupChanged: {
            if (showPopup){
                show()
                rebootMessage.startTimeout(5)
            } else
                hide()
        }

        onTimedOut: {
            main.startReboot()
            hide()
        }

        onVisibleChanged: {
            if(!visible) {
                // reset counter when hiding dialog
                rebootMessage.countDown = 5
            }
        }
        onTriggered: buzzer.warningBeep()
        Component.onCompleted: hide()

        Item {
            anchors.fill: parent
            z: parent.z + 1

            TextLabel {
                id: warningText2

                width: 0.8 * parent.width
                height: 99 * main.vScaleFactor

                text: qsTr("TV Error Detected.") + "\n" + qsTr(" Console Rebooting in ...") + application.emptyString
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.top: parent.top
                anchors.topMargin: 35 * main.vScaleFactor
                horizontalAlignment: Text.AlignHCenter
                wrapMode: Text.WordWrap
                font.pixelSize: 40 * main.vScaleFactor
            }

            NumberLabel {
                // show count down to timeout of prompt (started by startTimeout(seconds) method of ModalDialog superclass
                id: countdownText2

                // display count down (with a leading 0 if remaining seconds < 10
                text: ":" + ((rebootMessage.countDown.toString().length < 2) ? "0" : "") + rebootMessage.countDown
                font.pixelSize: 96 * main.vScaleFactor

                anchors.horizontalCenter: parent.horizontalCenter
                anchors.bottom: parent.bottom
                anchors.bottomMargin: 73 * main.vScaleFactor
            }
        }
    }

    function startReboot () {
        console.log("************* Calling reboot!")
        rebootConsole.start()
    }

    Process { id: rebootConsole
        program: "scripts/export-diagnostics/reboot.sh"

        onStarted: {
            console.log("Reboot started")
        }

        onReadyReadStandardOutput: {
            var output = String(readAllStandardOutput())
            output = output.trim()
            console.log( "reboot process (STDOUT):" + output)
        }

        onReadyReadStandardError: {
            var output = String(readAllStandardError())
            output = output.trim()
            console.log( "reboot process (STDERR):" + output)
        }
    }


    /**
     * @brief type:alias Property that contains the manager for the SAP mode.
     */
    property alias sapManager: sapLoader.item

    Loader {
        id: sapLoader
        sourceComponent: sapManagerComponent

        /**
         * @brief Reloads the SAP manager.
         */
        function reload() {
            sapLoader.sourceComponent = undefined;
            sapLoader.sourceComponent = sapManagerComponent;
        }

        Component {
            id: sapManagerComponent

            Item {
                /**
                 * @brief type:alias Property that contains the manager for the SAP manager?
                 */
                property alias item: manager

                SapManager {
                    id: manager
                    onChannelFound: {
                    }
                }
            }
        }
    }


    TvRecordingManager {
        id: tvRecordingManager
    }

    TvRecordingMessage {
        id: tvRecordingMessage
        visible: tvRecordingManager.recording || tvRecordingManager.copying
        z: 9999999
    }


    Rectangle
    {
        id:fullscreenVideoFrame
        visible: false
        color: "black"
        anchors.fill:parent


        function showRect()
        {
          fullscreenVideoFrame.visible=true
          tv.changeParent(videoViewBackground2)
          }

ColumnLayout {
            id:fullscreenVideoLayout
           // visible: true
           anchors.fill: parent

            anchors.margins: 1.5 * main.widthBorder
            anchors.bottomMargin: 10
            spacing: 0

            Rectangle { // TV area
                id: videoViewBackground2
                Layout.fillWidth: true
                Layout.fillHeight: true
                Layout.leftMargin: 80
                Layout.alignment: Qt.AlignHCenter | Qt.AlignCenter
                border.color: main.colorUIBorder
                border.width: main.widthBorder

                Rectangle
                {
                    id:tvNotPlaying
                    anchors.fill: parent
                    color: "black"
                    //visible: true


                            }

            }



           Item {
                Layout.preferredHeight:  main.height*0.08
                Layout.fillWidth:  true
                Layout.alignment: Qt.AlignHCenter | Qt.AlignBottom
                //visible: true
                RowLayout
                {
                    anchors.fill: parent
                    anchors.bottomMargin: 10

             StandardButton
                {
                    id:tvFullScreenButton
                    Layout.fillHeight: true
                    Layout.preferredWidth: main.width*0.08
                    Layout.alignment: Qt.AlignLeft | Qt.AlignCenter
                    icon.sourceName: "Media/expand_icon_720"
                    enabled:true
                    onClicked:
                    {
                        fullscreenVideoFrame.visible=false
                        tv.fullscreen=false
                        tv.changeParent(tv.viewParent)

                    }
                }
             Item
             {
                 id:levelRect
                 Layout.fillHeight: true
                 Layout.preferredWidth: main.width*0.20
                 Layout.alignment: Qt.AlignCenter
                 RowLayout
                {
                    anchors.fill: parent
                    anchors.leftMargin: 10

                 TextLabel {
                     id: leftDataLabel
                     Layout.fillHeight: true
                     Layout.fillWidth: true
                     Layout.alignment: Qt.AlignCenter
                     Layout.topMargin: 10
                     font.capitalization: Font.AllUppercase
                     font.pixelSize: 30 * main.vScaleFactor
                     text: main.isTreadmill ? qsTr("INCLINE") : qsTr("LEVEL") + application.emptyString
                 }

                 NumberLabel {
                     id: leftDataValue
                     Layout.fillHeight: true
                     Layout.fillWidth: true
                     font.capitalization: Font.AllUppercase
                     font.pixelSize: 48 * main.vScaleFactor
                     text: main.isTreadmill ? (workoutManager.treadInclineGUI/10.0).toFixed(1) : workoutManager.workloadLevel
                   }
                 }

             }
             Item
             {
                 id:rightRect
                 Layout.fillHeight: true
                 Layout.preferredWidth: main.width*0.20
                 Layout.alignment: Qt.AlignCenter
                 RowLayout
                {
                    anchors.fill: parent
                    TextLabel {
                        id: rightDataLabel
                        Layout.fillHeight: true
                        Layout.fillWidth: true
                        Layout.alignment: Qt.AlignCenter
                        Layout.topMargin: 10
                        font.capitalization: Font.AllUppercase
                        font.pixelSize: 30 * main.vScaleFactor
                        text: ((main.isTreadmill || main.isStepmill) ? qsTr("SPEED") : qsTr("RPM")) + application.emptyString
                    }

                    NumberLabel {
                        id: rightDataValue
                        Layout.fillHeight: true
                        Layout.fillWidth: true
                        font.capitalization: Font.AllUppercase
                        font.pixelSize: 48 * main.vScaleFactor
                        text: {
                               if (main.isTreadmill) return (workoutManager.treadSpeedGUI /10.0).toFixed(1)
                               else if (main.isStepmill) return workoutManager.treadSpeedMetricGUI.toFixed(1)
                               else return machinecontrol.pedalRPM
                        }

                    }
                }

             }

            }

        }
        }
    }
    LanguageMenu {
        id: language_menu
        opacity: 0
        enabled: (visible && opacity > 0.0)
        z:99
        anchors.fill: parent
        focus: main.state === "LanguageMenuState"
        onLanguageSelected: {
            if(lang) {
                application.selectLanguage(lang);
                home_screen.updateLanguageIcon(lang);
            }

            main.state = "HomeState";
        }
    }

    // return to default language after a period of inactivity in the home screen
    Timer {
        id: lang_timer
        running: main.state === "HomeState"
        interval: 5 * 60 * 1000
        onTriggered: {
            if (main.state === "HomeState") {
                application.selectLanguage(options.defaultLanguage); // reset language after workout
                home_screen.updateLanguageIcon(options.defaultLanguage);
            }
        }
    }
    Item {
        id: tvTestHud

        property alias channelChangeTime: channelChangeTimeSlider.value
//        property int usbhubcount: 0
        property int count: 0
        property ListModel tuneTimeModel: tv.tuneTimes

        opacity: 0.5
        visible: enabled
        enabled: options.autoChannelChangeInterval > 99
        width: childrenRect.width
        height: childrenRect.height
        anchors { left: parent.left; leftMargin: 25 * main.hScaleFactor; bottom: parent.bottom; bottomMargin: 50 * main.vScaleFactor}
        z:99
        Rectangle {
            id: tvTestItemRect
            color:"green"
            width: childrenRect.width
            height: childrenRect.height

            Column {
                spacing: 3 * main.vScaleFactor
                Text {  color: "white"; text: "Time: " + (tv.tuneTimes.count > 0 ? tv.tuneTimes.get(tv.tuneTimes.count-1).time/1000.0  + "s" : "") }
                Row {
                    id: barGraph
                    height: 50 * main.vScaleFactor
                    width: 200 * main.hScaleFactor
                    spacing: 1 * main.hScaleFactor
                    Repeater {
                        model: tvTestHud.tuneTimeModel
                        delegate: Component {
                            Rectangle {
                                height: barGraph.height * (model.time / 5000.0)
                                width: barGraph.width / tvTestHud.tuneTimeModel.numberTimesToStore
                                anchors { bottom: parent ? parent.bottom : undefined }
                            }
                        }
                    }
                }

                Text { visible: true; color: "white"; text: qsTr("Interval: ") + (channelChangeTimeSlider.off ? qsTr("OFF") : tvTestHud.channelChangeTime) }
                Text { visible: true; color: "white"; text: qsTr("Count: ") + tvTestHud.count }

                SliderControl{
                    id: channelChangeTimeSlider
                    property bool off: channelChangeTimeSlider.value === 0 //channelChangeTimeSlider.maxValue
                    value: options.autoChannelChangeInterval //maxValue
                    anchors.horizontalCenter: parent.horizontalCenter
                    width: 0.9 * parent.width
                    height: 30 * main.vScaleFactor
                    minValue: 0
                    maxValue: 600 * 1000
                    stepValue: 1000
                    showValue: false
                    onValueChanged: {options.autoChannelChangeInterval = value}
                }

//                Text { text: "VLC Error Occured: " + tv.vlcErrorOccurred }
            }
        }
    }

    Item {
        id: oiTestHud

        property int count: 0

        opacity: 0.7
        visible: enabled
        enabled: options.autoVideoChangeInterval > 0
        width: 150 * main.hScaleFactor
        height: 75 * main.vScaleFactor
        anchors {
            right: parent.right
            rightMargin: 25 * main.hScaleFactor
            bottom: parent.bottom
            bottomMargin: 50 * main.vScaleFactor
        }
        z:100

        Rectangle {
            id: oiTestItemRect
            color:"green"
            anchors.fill: parent

            Column {
                anchors.fill: parent
                spacing: 3 * main.vScaleFactor

                Text {
                    anchors.left: parent.left
                    anchors.leftMargin: 5 * main.hScaleFactor
                    visible: true
                    text: qsTr("Interval: ") + (changeTimeSlider.off ? qsTr("OFF") : changeTimeSlider.value)
                }

                Text {
                    anchors.left: parent.left
                    anchors.leftMargin: 5 * main.hScaleFactor
                    visible: true
                    text: qsTr("Count: ") + oiTestHud.count
                }

                Slider {
                    id: changeTimeSlider

                    anchors.horizontalCenter: parent.horizontalCenter
                    value: options.autoVideoChangeInterval/1000
                    onValueChanged: {options.autoVideoChangeInterval = value * 1000}

                    width: 0.9 * parent.width
                    height: 20 * main.vScaleFactor
                    orientation: Qt.Horizontal
                    minimumValue: 0
                    maximumValue: 300
                    stepSize: 1
//                    snapMode: Slider.SnapAlways
                }
            }
        }
    }

    Item {
        id: debugOverlay

        /**
         * @brief type:string Property that contains all supported audio sources.
         */
        property string asources: ((audioManager.ipodPlaying ? "ip " : "") +
                                  (audioManager.ipodPlayingVideo ? "iv " : "") +
                                  (audioManager.virtActivePlaying ? "va " : "") +
                                  (audioManager.tvPlaying ? "tv " : "") +
                                  (audioManager.browserPlaying ? "web " : "") +
                                  (audioManager.btconnected ? "bt " : ""))

        /**
         * @brief type:string Property that contains all allowed audio devices.
         */
        property string aallowed: ((audioManager.ipodAllowed ? "ip " : "") +
                                  (audioManager.ipodVideoAllowed ? "iv " : "") +
                                  (audioManager.btAllowed ? "bt ": "") +
                                  (audioManager.virtActiveAllowed ? "va " : "") +
                                  (audioManager.virtActiveVideoAllowed ? "vv " : "") +
                                  (audioManager.tvAllowed ? "tv " : "") +
                                  (audioManager.tvVideoAllowed ? "tvv " : "") +
                                  (audioManager.browserAllowed ? "web " : ""))

        property string tvsettings: ((tv.on ? "on ": "off ") +
                                     "CC: " + (tv.captionTrack > 0 ? "on " : "off ") +
                                     ("lastCCTrack: " + tv.lastCaptionTrack))

        opacity: 0.5
        visible: false
        x: 40 * main.hScaleFactor
        y: 180 * main.vScaleFactor
        z:99

        Rectangle {
            color:"lightgray"
            width: childrenRect.width + 10 * main.hScaleFactor
            height: childrenRect.height + 10 * main.vScaleFactor

            Column {
                spacing: 3
                Text { visible: true;           text: qsTr("tv: ") + debugOverlay.tvsettings }
                Text { visible: true;           text: qsTr("audio playing:") + debugOverlay.asources }
                Text { visible: true;           text: qsTr("audio allowed:") + debugOverlay.aallowed }
                Text { visible: true;           text: qsTr("mux:") + audioManager.currentMuxSource }
                Text { visible: true;           text: qsTr("volume:") + audioManager.volume }
                Text { visible: true;           text: qsTr("mute:") + audioManager.mute }
                Text { visible: true;           text: qsTr("nucleus ready:") + nucleus.systemReady }
                Text { visible: isTreadmill; 	text: qsTr("static brake:") + machinecontrol.isStaticBrakeEngaged }
                Text { visible: isTreadmill; 	text: qsTr("target incline:") + machinecontrol.treadmillInclineTarget / 10.0 }
                Text { visible: isTreadmill; 	text: qsTr("actual incline:") + machinecontrol.treadmillInclinePercentGradeActual / 10.0 }
                Text { visible: isTreadmill; 	text: qsTr("target speed:") + machinecontrol.treadmillSpeedTarget / 10.0 }
                Text { visible: isTreadmill; 	text: qsTr("actual speed:") + machinecontrol.treadmillSpeedActual / 10.0 }
                Text { visible: isTreadmill; 	text: qsTr("actual rpm:") + machinecontrol.isBeltMoving }
                Text { visible: isStepmill; 	text: qsTr("actual speed:") + machinecontrol.stepmillSpeedActual}
                Text { visible: isStepmill; 	text: qsTr("target speed:") + machinecontrol.stepmillSpeedTarget}
                Text { visible: !isTreadmill && !isStepmill; 	text: qsTr("Workload:") + machinecontrol.workload }
                Text { visible: true;           text: qsTr("time warp:") + timeFactorControl.value + "x" }

                SliderControl {
                    id: timeFactorControl
                    value: main.timeFactor
                    width: 150 * main.hScaleFactor
                    x: 20 * main.hScaleFactor
                    minValue: 0
                    maxValue: 30
                    showValue: false
                    onValueChanged: main.timeFactor = value

                    thumbImage.sourceName: "Components/Red_Dot";
                }
            }
        }
    }

    Rectangle {
        id: languageDebug
        visible: false
        x: 20 * main.hScaleFactor
        y: 300 * main.vScaleFactor
        color: "#80659d32"
        width: 200 * main.hScaleFactor
        height: childrenRect.height
        z:99999999999999999999

        Column {
            spacing: 2
            Repeater {
                model: [
                    { lang: "en", caption: qsTranslate("LanguageMenu", "ENGLISH") + application.emptyString } ,
                    { lang: "ru", caption: qsTranslate("LanguageMenu", "RUSSIAN") + application.emptyString } ,
                    { lang: "fr", caption: qsTranslate("LanguageMenu", "FRENCH") + application.emptyString } ,
                    { lang: "zh", caption: qsTranslate("LanguageMenu", "CHINESE") + application.emptyString } ,
                    { lang: "de", caption: qsTranslate("LanguageMenu", "GERMAN") + application.emptyString } ,
                    { lang: "nl", caption: qsTranslate("LanguageMenu", "DUTCH") + application.emptyString } ,
                    { lang: "ar", caption: qsTranslate("LanguageMenu", "ARABIC") + application.emptyString } ,
                    { lang: "es", caption: qsTranslate("LanguageMenu", "SPANISH") + application.emptyString } ,
                    { lang: "pt", caption: qsTranslate("LanguageMenu", "PORTUGUESE") + application.emptyString } ,
                    { lang: "ko", caption: qsTranslate("LanguageMenu", "KOREAN") + application.emptyString } ,
                    { lang: "ja", caption: qsTranslate("LanguageMenu", "JAPANESE") + application.emptyString } ,
                    { lang: "it", caption: qsTranslate("LanguageMenu", "ITALIAN") + application.emptyString } ,
                ]
                delegate: Component {
                    Text {
                        font { pixelSize: 16 * main.vScaleFactor}
                        font.bold: application.currentLanguage === modelData.lang
                        color: main.colorFontLight
                        text: modelData.caption

                        MouseArea {
                            anchors.fill: parent
                            onClicked: {
                                buzzer.buttonPress()
                                var lang = modelData.lang
                                application.selectLanguage(lang)
//                                home_screen.updateLanguageIcon(lang)
                            }
                            Rectangle {
                                anchors.fill: parent
                                color: main.colorGenericStimulusArea
                                opacity: 0.3
                                visible: main.showStimulusArea
                            }
                        }
                    }
                }
            }
        }
    }

    Rectangle {
        id: gemDebug
        visible: false

        x: 20 * main.hScaleFactor
        y: 300 * main.vScaleFactor
        color: "#80659d32"
        width: 250 * main.hScaleFactor
        height: childrenRect.height
        z:99999999999999999999

        Column {
            spacing: 2

            Text { text: "GEM Advertised Name: " }
            Text { text: gem.advertisedName }
            Text { text: "GEM Connection Allowed: " + gem.connectionAllowed }
            Text { text: "GEM Connected: " + gem.connected }
            Text { text: "GEM Apple Watch Connected: " + gem.appleWatchConnected }
            Text { text: "GEM Heart Rate: " + gem.heartRate }
            Text { text: "GEM Average Heart Rate: " + gem.gemHeartRateAvg }
            Text { text: "GEM Total Calories: " + gem.gemTotalCalories }
            Text { text: "GEM Calories Per Hour: " + gem.gemCaloriesPerHour }
            Text { text: "GEM Active Calories: " + gem.gemActiveCalories }
            Text { text: "GEM User Name: " + gem.gemUserName }
            Text { text: "GEM Control Action Type: " + gem.gemControlActionType }
            Text { text: "GEM FW Version: " + gem.firmwareVersion }
        }
    }

    Rectangle {
        id: egymDebug
        visible: false

        x: 20 * main.hScaleFactor
        y: 300 * main.vScaleFactor
        color: "#80df3933"
        width: 250 * main.hScaleFactor
        height: childrenRect.height
        z:99999999999

        Column {
            spacing: 5 * main.vScaleFactor

            Text { text: "EGym enabled: " + eGym.enabled}
            Text { text: "EGym connected: " + eGym.connected}
            Text { text: "EGym userName: " + userData.userName}
            Text { text: "EGym RFID: " + eGym.rfid}
            Text { text: "EGym RFID Format: " + eGym.rfidFormat}
            Text { text: "EGym Exercise Name: " + eGym.selectedExerciseName}
            Text { text: "EGym Exercise ID: " + eGym.selectedExerciseId}
            Text { text: "EGym Exercise Plan ID: " + eGym.selectedPlanId}
            Text { text: "EGym Serial# :" + eGym.serialNumber}
            Text { text: "EGym Type: " + eGym.machineType}

            StandardButton {
                z: parent.z + 1
                width: parent.width * 0.9
                height: 50* main.vScaleFactor
                anchors.horizontalCenter: parent.horizontalCenter
                buttonText: "Login with RFID"
                enabled: eGym.enabled
                onClicked: {
                    console.log("EGym Login " + eGymRFIDs.selectedData.label)
                    eGym.login(eGymRFIDs.selectedData.label, "MIFARE")
                }
            }

            StandardButton {
                z: parent.z + 1
                width: parent.width * 0.9
                height: 50* main.vScaleFactor
                anchors.horizontalCenter: parent.horizontalCenter
                buttonText: "Logout"
                enabled: eGym.enabled
                onClicked: {
                    console.log("EGym Logout")
                    eGym.reset()
                }
            }

            ComboBox {
                id: eGymRFIDs
                z: parent.z + 1
                width: parent.width * 0.9
                height: 50 * main.vScaleFactor
                anchors.horizontalCenter: parent.horizontalCenter
                currentIndex: 0

                model: ListModel {
                    ListElement { label: "A2667D23"}
                    ListElement { label: "F2767A23"}
                    ListElement { label: "72448223"}
                    ListElement { label: "82C78423"}
                    ListElement { label: "F2088C23"}
                }
            }
        }
    }

    Rectangle {
        id: heartrateSimulator
        anchors {
            right: parent.right
            rightMargin: 60 * main.hScaleFactor
            bottom: parent.bottom
            bottomMargin: 150 * main.vScaleFactor
        }
        visible: false
        color: "red"
        opacity: 0.4
        width: 300 * main.hScaleFactor
        height: 100 * main.vScaleFactor
        z: 9999

        Column {
            spacing: 10 * main.vScaleFactor
            anchors.fill: parent
            anchors.topMargin: 15 * main.vScaleFactor

            Checkbox {
                label: "Enable Heart Rate Simulator"
                width: 0.9 * parent.width
                font.pixelSize: 15 * main.vScaleFactor
                anchors.horizontalCenter: parent.horizontalCenter
                value: false
                onPressed: {
                    hrSimSlider.enabled = value;
                }
            }

            SliderControl{
                id: hrSimSlider
                value: 80
                anchors.horizontalCenter: parent.horizontalCenter
                width: 0.9 * parent.width
                height: 30 * main.vScaleFactor
                minValue: 80
                maxValue: 200
                stepValue: 1
                showValue: true
                enabled: false
                onValueChanged: {
                    if(enabled) heartrate.simulateHeartRate(value)
                }
                onEnabledChanged: {
                    heartrate.simulateHeartRate(enabled ? value : 0)
                }
            }
        }
    }

    // Keys to control the simulator
    Keys.onPressed: {
        switch(event.key){
            case Qt.Key_Q: { // fake stride-up for testing
                keymapper.keyPressHandler(8);
                break;
            }
            case Qt.Key_A: { // fake stride-down for testing
                keymapper.keyPressHandler(3);
                break;
            }
            case Qt.Key_R: { // Ctrl+r allows recording a TV stream
                if (event.modifiers & Qt.ControlModifier) {
                    console.log("ctrl-r pressed");

                    if (!tvRecordingManager.recording) {
                        tvRecordingManager.startRecording();
                    }
                }

                break;
            }
            case Qt.Key_S: { // toggle on/off simulation of random pedaling / heart rate
//                main.loginAvailable = !main.loginAvailable;
//                pedalSim.simulating = !pedalSim.simulating;
                break;
            }
            case Qt.Key_H: {  // toggle holding heartrate grips
//                pedalSim.holdingGrips = !pedalSim.holdingGrips;
                heartrateSimulator.visible = !heartrateSimulator.visible
                break;
            }
            // must have simulation turned off to use the following controls
            case Qt.Key_BracketRight: { // ']' increase heartrate
//                pedalSim.heartRate++;
                break;
            }
            case Qt.Key_BracketLeft: { // '[' decrease heartrate
//                pedalSim.heartRate--;
                break;
            }
            case Qt.Key_Period: { // '.' increase pedal speed
                pedalRPM_multiplier += 0.1
//                pedalSim.pedalRPM++;
                break;
            }
            case Qt.Key_Comma: { // ',' decrease pedal speed
                pedalRPM_multiplier -= 0.1
                pedalRPM_multiplier = Math.max(pedalRPM_multiplier, 0.1)
//                pedalSim.pedalRPM--;
                break;
            }
            case Qt.Key_AsciiTilde: // '~' toggle debug HUD
                debugOverlay.visible = !debugOverlay.visible;
                break;
            case Qt.Key_QuoteDbl: {
                languageDebug.visible = !languageDebug.visible;
                break;
            }
            case Qt.Key_C: // 'C' toggle show mouse cursor
                application.showCursor = !application.showCursor;
                break;
            case Qt.Key_O: // 0 key also takes screen shot
            case Qt.Key_P: {// 'P' take screen shot (photo)
                buzzer.buttonPress();
                print("taking screenshot " + main.screenShotCount);
                application.screenShot("/media/usb/screenshot_" + main.screenShotCount + ".png");
                infoBanner.showMessage(qsTr("Screenshot ") + main.screenShotCount + qsTr(" saved."), 3);
                main.screenShotCount++;
                break;
            }
            case Qt.Key_K: { // toggle safety key simulation
//                pedalSim.safetyKeyIn = !pedalSim.safetyKeyIn;
//                print("estop:" + pedalSim.safetyKeyIn);
                break;
            }
            case Qt.Key_D: { // quick diagnostic entry
                accessKeyPad.visible = true;
                break;
            }
            case Qt.Key_T: {
                if (event.modifiers & Qt.ShiftModifier) {
                  tvTestHud.visible = !tvTestHud.visible
                }
                break
            }
            case Qt.Key_W: {
                gemDebug.visible = !gemDebug.visible;
                break;
            }
            case Qt.Key_E: {
                egymDebug.visible = !egymDebug.visible;
                break;
            }
            case Qt.Key_F1: {
                break;
            }
        }
    }

    Keys.onReleased: {
        switch (event.key) {
            case Qt.Key_Q: { // fake stride-up for testing
                keymapper.keyReleaseHandler(62);
                break;
            }
            case Qt.Key_A: { // fake stride-down for testing
                keymapper.keyReleaseHandler(63);
                break;
            }
        }
    }

    Connections {
        target: options
        onSimulationModeChanged: {
            console.debug("Simulation mode changed: " + options.simulationMode)
        }

        onEnableNFCChanged: {
            gem.appleWatchEnabled = gem.nfc_Installed && options.enableNFC
        }

        onEnableAntPlusChanged: {
            gem.enableAntPlus = options.enableAntPlus
            gem.reboot()
        }
    }

    Connections {
        target: heartrate
        onTelemetryHeartRateChanged: heartRateManager.setHeartRate("telemetry", telemetryHeartRate)
        onContactHeartRateChanged: heartRateManager.setHeartRate("contact", contactHeartRate)
        onGemHeartRateChanged: heartRateManager.setHeartRate("GEM", gemHeartRate)
    }

    Connections {
        target: gem
        onHeartRateChanged: {
            if(hr > 0)
                heartRateManager.setHeartRate("GEM", hr)
        }
        onGemControlActionTypeChanged: {
            switch (type) {
            case 1: // Start
                break;
            case 2: // Pause
                break;
            case 3: // Resume
                break;
            case 4: // Finish
                break;
            case 0: // Unused
            case 255: // Reset
            default:
                 break;
            }
        }
        onConnectedChanged: {
            if(gem.connected === 0) {
                if (main.state != "RunState" && main.state != "WorkoutSummaryState") {
                    gem.appleWatchConnected = false
                }
                if(gem.waitingForWorkoutEnd) {
                    workout_summary.setData( aWorkoutRunData )
                    gem.resetValues();
                    workoutManager.reset();
                    workoutManager.resetLevels();
                    gem.waitingForWorkoutEnd = false
                }
            }
        }
        onAppleWatchConnectedChanged: {
            if(gem.appleWatchConnected) {
                main.showAppleWatchConnectsInfo = false
            }
        }
        onNfc_InstalledChanged:{
            if(!gem.nfc_Installed)
                options.enableNFC = false
        }
    }

    RecoveryManager{
        id: recoveryManager
        onCompleted: {
            errorManager.preventUse = false
            errorManager.degradedUse = false
            popupManager.reset()
        }
    }

    Connections{
        target: errorManager
        onErrorEvent:{
            recoveryManager.errorTriggeredSlot(errorcode)
        }

        onConsoleDetectedFaultRecovered: {
            errorManager.preventUse = false
            errorManager.degradedUse = false
            popupManager.reset()
        }
    }

    function personPresentSupported()
    {
        return ( machineinfo.modelString === "TI1000" ||
                machineinfo.modelString === "TC900" ||
                machineinfo.modelString === "TC650" ||
                machineinfo.modelString === "TC400"  ||
                machineinfo.modelString === "ST1000")
    }

    PersonPresentDetectionManager{
        id:personPresentDetectionManager
        isEnabled: (options.ppdEnabled) && personPresentSupported() && main.state === "RunState";
    }

    KeyMapper{
        id: keymapper
        function handleStart() {
            if ((isTreadmill || isStepmill) && machinecontrol.treadmillSpeedActual !== 0) {
                return
            }

            if((isTreadmill || isStepmill) && canStartWorkout == false) {
                return
            }

            if (blockUseMessage.visible || errorManager.fatalError || errorManager.preventUse || main.state === "SleepModeState")
                return
            if (accessDialog.visible)
                return
            if (softwareUpdatePrompt.active)
                return
            if (logUploadPrompt.visible)
                return
            if (bluetoothAdapter.connecting)
                return

            quickStart()
            main.channelSelected = ""
        }
        onStartQuickStartPressed: {
            keymapper.handleStart()
        }
        onStartPressed: {
            keymapper.handleStart()
        }
    }

    WorkoutProgram {
        id: workoutProgram

        userWeightMax: {
            if(main.lowerSystem === "NUCLEUS") {
                return nucleus.maxUserWeightLbs
            } else {
                return 400
            }
        }
        metricUnits: options.metricUnits
        saveWorkoutEnabled: options.saveWorkoutEnabled
        property int maxSegments: Math.min( profile1.count, profile2.count)
        inclineTable: machineinfo.inclineTable
        speedTable: machineinfo.speedTable
        speedTableMetric: machineinfo.speedTableMetric
        brakeWattsTable: machineinfo.brakeWattsTable
        paceTable: machineinfo.paceTable
        brainboardProductTypeString: machineinfo.productTypeString
        isChangeToNewWorkout: main.isNewWorkoutSelected
    }

    WorkoutManager {
        id: workoutManager

        property bool withLimit: true

        productType: machineinfo.productType
        workoutData: aWorkoutRunData
        heartRate: heartRateManager.heartRate
        treadSpeedGUI: isStepmill ? machinecontrol.stepmillSpeedTarget : machinecontrol.treadmillSpeedTarget
        treadInclineGUI: machinecontrol.treadmillInclineTarget
        isEGymCardioTest: main.eGymCardioTest

        onWattsAt60Changed: {
            machinecontrol.workload = wattsAt60;
//            aWorkoutRunData.setWatts(wattsAt60)  // Why display target watts ever?
        }
      /*  onTreadInclineChanged:
        {
            //tv.showOverlayOnFullscreen()
        }

        onTreadSpeedChanged:
        {
            //tv.showOverlayOnFullscreen()
        }
        onWorkloadLevelChanged:
        {
           // tv.showOverlayOnFullscreen()
        }*/

        onSpeedChanged: {
            // allow non-zero speed only in RunState
            if(isTreadmill)
                machinecontrol.treadmillSpeedTarget = (main.state === "RunState") ? treadSpeed : 0
            else if(isStepmill) {
                machinecontrol.stepmillSpeedTarget = (main.state === "RunState") ? treadSpeed : 0
            }
        }
        onInclineChanged: {
            machinecontrol.treadmillInclineTarget = treadIncline
            if(main.lowerSystem === "NUCLEUS") {
                workoutData.incline = treadIncline / 10.0
            }
        }

        onStrideLevelChanged: machinecontrol.strideLengthTarget = strideLevel

        workoutProgram: workoutProgram

        timeFactor: main.timeFactor
        cooldownDuration: options.cooldownTime
        //treadInclineMax: configurationData.maxInclinePercentGrade
        //treadInclineMin: configurationData.minInclinePercentGrade

        onWorkoutModeChanged:
        {
            var modeStr = ""
            switch(mode) {
            case WorkoutRunData.WARM_UP_MODE: modeStr = "Warm Up"; break;
            case WorkoutRunData.COOL_DOWN_MODE: modeStr = "Cool Down"; break;
            case WorkoutRunData.WORKOUT_MODE: modeStr = "Workout"; break;
            case WorkoutRunData.SUMMARY_MODE: modeStr = "Summary"; break;
            }

            print("WorkoutManager: Workout Mode is " + modeStr)
            aWorkoutRunData.setWorkoutMode(mode);
            if (mode == WorkoutRunData.SUMMARY_MODE) {
                workoutScreen.stopWorkout()
                if(main.isTreadmill) {
                    treadSpeed = 0
                    machinecontrol.treadmillSpeedTarget = treadSpeed
                }
            }
            if (mode == WorkoutRunData.COOL_DOWN_MODE) {
                workoutScreen.showMessage(qsTr("COOL DOWN") + application.emptyString)
            }
        }
        onDisplayHRPrompt: workoutScreen.displayHRPrompt(index);

        onSelectTestEffortLevel: {
            eGymCardioTest_selectEffortLevel.show()
        }
    }

    WorkoutRunData {
       id: aWorkoutRunData

        workoutProgram: workoutProgram.programIndex
        productTypeString: machineinfo.productTypeString
        heartRate: heartRateManager.heartRate
        workloadLevel: workoutManager.workloadLevel
        paceLevel: workoutManager.paceLevel
        wattsAt60: workoutManager.wattsAt60;
        timeFactor: main.timeFactor
        strideLength: workoutManager.strideLevel
    }

    Connections {
        target: eGym

        onConnectedChanged: {

        }

        onTrainingPlansRecieved: {
            if(eGym.connected && userData.userName !== "") {
                main.state = "MenuState"
                workout_menu.selectMenu("EGYM")
            }
        }

        onCardioTestInitialized: {
            mainProgItem.doStart()
        }

        onTestNextIntervalChanged: {
            if(eGym.testInProgress) {
                main.eGymCardioTest_IntervalPending = false
                workoutManager.doEGymTestNextInterval(eGym.testNextInterval)
            } else {
                main.eGymCardioTest_IntervalPending = true
            }
        }

        onTestInProgressChanged: {
            if(eGym.testInProgress && main.eGymCardioTest_IntervalPending) {
                main.eGymCardioTest_IntervalPending = false
                workoutManager.doEGymTestNextInterval(eGym.testNextInterval)
            }
        }

        onShowTerms: {

        }

        onShowTestSetup: {

        }

        onShowTestResults: {
            workoutScreen.stopWorkout()
        }

        onTestAborted: {
            workoutScreen.stopWorkout()
        }
    }

    ModalDialog {
        id: eGymCardioTest_selectEffortLevel
        z: 999

        visible: false
        property int effortLevel: -1

        onVisibleChanged: {
            if(visible) {
                effortLevel = -1;
            } else {
                eGym.testIntervalCompleted = true
                eGym.effortLevel = eGymCardioTest_selectEffortLevel.effortLevel
                eGym.requestUpdateTest()
            }
        }
        Item {
            anchors.fill: parent
            z: parent.z + 1

            CloseButton {
                id: eGymCloseButton

                anchors {
                    right: parent.right ; rightMargin: 80 * main.hScaleFactor
                    top: parent.top ; topMargin: 40 * main.vScaleFactor
                }
                onClicked: parent.hide()
                enabled: eGymCardioTest_selectEffortLevel.effortLevel >= 0
            }

            TextLabel {
                id: testTitle
                anchors {
                    top: parent.top
                    topMargin: 20 * main.vScaleFactor
                    horizontalCenter: parent.horizontalCenter
                }

                text: qsTr("HOW DIFFICULT WAS THE LAST SEGMENT?")
            }

            Row {
                anchors {
                    centerIn: parent
                }

                height: 50 * main.vScaleFactor
                spacing: 10 * main.vScaleFactor

                Repeater {
                    model: 10
                    delegate: Component {
                        StandardButton {
                            width: 50 * main.hScaleFactor
                            height: 50 * main.vScaleFactor
                            buttonText: index
                            selected: eGymCardioTest_selectEffortLevel === index

                            onClicked: {
                                eGymCardioTest_selectEffortLevel.effortLevel = index
                                eGymCardioTest_selectEffortLevel.hide()
                            }
                        }
                    }
                }
            }
        }
    }

    MessagePopup {
        id: speedInclineLimitedPopup
        blockKeys: false
        isTransparent: true
        Component.onCompleted: hide()
        onVisibleChanged: {
        if(visible) {
            if(workoutScreen.showingTV)
            tv.on=false
            } else if(workoutScreen.showingTV)
            changeTvState.start()
    }
   }

    Connections {
        target: machinecontrol
        onSpeedLimited: {
          //  workoutManager.treadSpeed = machinecontrol.treadmillSpeedTarget
           var speed= SetupJS.returnSpeedIncline("incline",(workoutManager.treadSpeed / 10.0),(workoutManager.treadIncline/10.0),1)
            var msg = qsTr("Speed limited to ") + application.emptyString

          //  var speed = workoutManager.treadSpeed / 10.0
            if (workoutScreen.unitState === "ENGLISH") {
                msg += speed + " " + qsTr("MPH") + application.emptyString
            } else {
                msg += Globals.mileToKm(speed).toFixed(1) + " " + qsTr("KPH") + application.emptyString
            }

            speedInclineLimitedPopup.showMessage(msg, 3)
        }
        onInclineLimited: {
           // workoutManager.treadIncline = machinecontrol.treadmillInclineTarget
            var incline=SetupJS.returnSpeedIncline("speed",(workoutManager.treadSpeed / 10.0),(workoutManager.treadIncline/10.0),1)
           var msg = qsTr("Incline limited to ") + incline + " %";
            speedInclineLimitedPopup.showMessage(msg, 3)
        }
        onInclineInhibited: {
            workoutManager.treadIncline = machinecontrol.treadmillInclineTarget
            infoBanner.showMessage("Incline disabled", 5)
        }
    }

    ProgSetup {
        id: mainProgItem
        enabled: (visible && opacity > 0.0)

        focus: main.state === "SetupState"

        onStartProgram:{
            workoutProgram.setupComplete(aWorkoutRunData.totalTime, aWorkoutRunData.totalDistance)
            workoutScreen.updateAdjustModel()
            workoutStartManager.conditionalStart()
//            workoutManager.start();
//            main.state = "RunState";
//            workoutScreen.start();
        }

        onSaveCustomProgram: workoutProgram.saveProgram(programFileName,saveLocation);

        anchors.fill: parent
        opacity: 0
        z: 8
    }

    BlueToothLowEnergy {
        id: ble
    }

    WorkoutScreen {
        id: workoutScreen
        enabled: (visible && opacity > 0.0)

//        visible: main.state === "RunState" && !fullscreenVideoFrame.visible
        visible: main.state === "RunState" && !tv.fullscreen

        onWorkoutStopped: {
            gem.stopWorkout()
            browserAudioWatcher.kill()
            audioManager.reset()
            if(main.state==="RunState")
            main.state = "WorkoutSummaryState";
            workoutManager.stop();
            centerpod.setQuickStart(false)
            stopPrompt.hide()
            viewContainerBanner.hide()
            if (bluetoothAdapter.discovering) {
                bluetoothAdapter.stopDiscovery()
            }

            //prepare data for blutooth advertisment
            //ble.exportWorkoutDataForBleno(aWorkoutRunData,application)

            // add to statistics
            statisticManager.mileage += aWorkoutRunData.totalDistance * 100
            statisticManager.runTime += aWorkoutRunData.totalTime
            statisticManager.totalDistance += aWorkoutRunData.totalDistance * 100
            statisticManager.maxSpeed = Math.max(aWorkoutRunData.maximumSpeed*10, statisticManager.maxSpeed)
            statisticManager.averageSpeed = (statisticManager.mileage/100)/(statisticManager.runTime/3600)
            statisticManager.syncFromNucleus();

            // end of workout
            if(gem.connected === 0) {
                workout_summary.setData( aWorkoutRunData )
                workoutManager.reset()
                workoutManager.resetLevels()
                gem.waitingForWorkoutEnd = false
            } else {
                gem.waitingForWorkoutEnd = true
            }

            if(eGym.connected)
                eGym.stopWorkout();

            aWorkoutRunData.mode = WorkoutRunData.SUMMARY_MODE
            mainProgItem.clearListView();

            workoutScreen.container.destroyViews()

            if(!main.isNewWorkoutSelected) {
                audioManager.setMute(false)
                tv.currentChannel = -1

                // if bluetooth is connected, disconnect the bluetooth device
                if(bluetoothAdapter.connected){
                    bluetoothAdapter.disconnect()
                }
                if (bluetoothAdapter.connectedHeadset)
                    bluetoothAdapter.disconnectHeadset()
            }
            machinecontrol.autoStrideEnabled = false

            heartRateManager.reset();

            if(options.workoutAutoRestartInterval > 0) {
                console.log("Restarting autostart timer................................................................................. autostart in " + autostartTimer.interval + "ms")
                autostartTimer.restart()
            }
        }

        onWorkoutPaused: {
            workoutManager.pause()
        }
        onWorkoutStarted: {
            tv.firstTune = true;
            tv.disconnectwifi=true;
            quickStartLoading = false
            gem.waitingForWorkoutEnd = false

            if(options.workoutAutoRestartInterval > 0) {
                console.log("Restarting autostop timer......................................................................................autostop in "  + autostopTimer.interval +"ms.")
                autostopTimer.restart()
                // Set incline to max
                console.log("Automatically Setting incline to " + workoutProgram.maxIncline)
                workoutManager.treadIncline = workoutProgram.maxIncline * 10
            }
        }

        onUnitStateChanged: {
            aWorkoutRunData.metricUnits = (workoutScreen.unitState === "METRIC" ? true : false)
        }

        anchors.fill: parent
        opacity: 0


    }

    WorkoutSummary {
        id: workout_summary
        timeLeft: idle_timer.secondsLeft()
        opacity: 0
        enabled: (visible && opacity > 0.0)
        focus: main.state === "WorkoutSummaryState"
        onClose: {
            saveHasHappened = false
            if(main.eGymCardioTest) {
                main.eGymCardioTest = false
                main.state = "MenuState"
            } else {
                eGym.reset()
                main.state = "HomeState"
            }
        }
        onSave: {
            var success = aWorkoutRunData.saveResults()
            if(success) {
                saveHasHappened = true
                infoPopup.showMessage(qsTr("Summary has been saved.") + application.emptyString, 5);
            }
            else
                infoPopup.showMessage(qsTr("Error. Check your memory device.") + application.emptyString, 5);
        }
    }

    SetupWizard{
        id: setupWizard

        focus: main.state === "SetupWizardState"
        isNucleus: main.lowerSystem === "NUCLEUS" || main.lowerSystem === "NUCBOOT"
        nucleusUpdateAvailable: nucleusfirmwareloader.isUpdateAvailable

        opacity: 0
        enabled: (visible && opacity > 0.0)
        onSetupWizardComplete: {
            if(!main.restartRequired)
                main.rebootRequired = true

            main.state = "HomeState"
        }
        onLanguageSelected:{
            if(lang){
                application.selectLanguage(lang)
                home_screen.updateLanguageIcon(lang)
                options.defaultLanguage = lang
            }
        }

    }

    BlueToothHeartRate {
        id: bluetoothLEAdapter

        onScanningChanged: {
            if(scanning === true) {
                heartRateManager.currentBtAddress = ""
                heartRateManager.clearBluetoothHeartRateDeviceList()
                if (bluetoothLEAdapter.nextHrAddress !== "") {
                    bluetoothLEAdapter.getHrServiceHandle(bluetoothLEAdapter.nextHrAddress, bluetoothLEAdapter.nextHrAddressType)
                    bluetoothLEAdapter.nextHrAddress = ""
                    bluetoothLEAdapter.nextHrAddressType = ""
                }
            }
        }

        onHeartrateChanged: {
            bluetoothLEAdapter.connecting=false
            heartRateManager.setHeartRate("bluetooth", bluetoothLEAdapter.heartrate)
        }
        //onScanningChanged: {
            //if(bluetoothLEAdapter.scanning == true)

        //}
        onHrTimeout: {
            console.log("Heartrate timed out...now disconnecting")
            console.debug("Heartrate timed out...now disconnecting")

            bluetoothLEAdapter.disconnect(bluetoothLEAdapter.deviceMAC,bluetoothLEAdapter.hrHandle)
        }
        onConnected: {
            heartRateManager.currentBtAddress = bluetoothLEAdapter.deviceMAC

            /* Close media browser if showing on HR connected */
            if(main.state === "HomeState") {
                if(ipod_window === true) {
                    main.showMediaBrowser = false
                }
            }
        }
        onHrDisconnectingComplete: {
            heartRateManager.currentBtAddress = ""
            heartRateManager.clearBluetoothHeartRateDeviceList()
            if (bluetoothLEAdapter.nextHrAddress !== "") {
                bluetoothLEAdapter.getHrServiceHandle(bluetoothLEAdapter.nextHrAddress, bluetoothLEAdapter.nextHrAddressType)
                bluetoothLEAdapter.nextHrAddress = ""
                bluetoothLEAdapter.nextHrAddressType = ""
            }
        }
    }


    HeartRateManager{
        id: heartRateManager
        onBluetoothHRConnect: {
            if(heartRateManager.currentBtAddress !== address) {
                if (bluetoothLEAdapter.hrConnected) {
                    // Disconnect them and store address for later reconnect
                    bluetoothLEAdapter.nextHrAddress = address
                    bluetoothLEAdapter.nextHrAddressType = addressType
                    bluetoothLEAdapter.disconnect(bluetoothLEAdapter.deviceMAC, bluetoothLEAdapter.hrHandle)
                    return
                }

                bluetoothLEAdapter.getHrServiceHandle(address, addressType)
            }
        }
    }

    // Both SetupWizard and DiagnosticsMain use this, so put it here
    ModalDialog {
        id:productSelectedConfirmation
        property alias text: dialogText.text

        Text {
            id: dialogText
            anchors.fill: parent
            anchors.verticalCenter: parent.verticalCenter
            color: main.colorFontLight
            anchors.horizontalCenter: parent.horizontalCenter
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            wrapMode: Text.WordWrap
            font.bold: false
            font.pixelSize: 50 * main.hScaleFactor
            z: parent.z + 1

        }
        Component.onCompleted: hide()
        onTimedOut: hide()
    }

    MediaBrowser {
        id: ipod_window

        allowFullscreen: false
        z: 90
        opacity: visible ? 1.0 : 0.0
        enabled: visible
        visible: main.showMediaBrowser

        isDialogVisible: visible

        onClose: {
            main.showMediaBrowser = false
        }
        onVisibleChanged: {
            if(visible)
                shown()
            else
                hidden()
        }
        anchors {   horizontalCenter: parent.horizontalCenter
                    top: parent.top
                    topMargin: 140 * main.vScaleFactor
        }

        Behavior on opacity { NumberAnimation { duration: 200; easing.type: Easing.InOutQuad } }
    }

    DiagnosticsMain {
        id: diagnosticsMain
        opacity: 0
        enabled: (visible && opacity > 0.0)
        anchors.fill: parent

        Connections {
            target: diagnosticsMain
            ignoreUnknownSignals: true

            onClose: {
                main.state = "HomeState"
                if(machinecontrol.isCalibrated){
                    machinecontrol.treadmillInclineTarget = 0
                }
            }
        }
    }

    Keypad {
        id: accessKeyPad
        length: 5
        show_entry: true
        instructions: qsTr("ENTER PASSCODE") + application.emptyString
        textEntryMinWidth: 300 * main.hScaleFactor
        textEntryHeight: 42 * main.vScaleFactor
        z: 1000

        anchors.centerIn: parent
        width: 417 * main.hScaleFactor
        height: 436 * main.vScaleFactor
        radius: main.defaultRadius
        border.color: main.colorUIBorder
        border.width: main.widthBorder

        opacity: 1
        visible: false

        onClose: {
            diagnosticsMain.diagnosticAccessCode = 0
            accessKeyPad.visible = false
            accessKeyPad.clear()
        }

        onValueEntered: {
            if(val.length == 5) {
                diagnosticsMain.diagnosticAccessCode = Number(val)
                accessKeyPad.visible = false
                if (diagnosticsMain.accessLevel >= 0){
                   if(main.state==="RunState")
                   workoutScreen.stopWorkout()

                  main.state = "DiagnosticState"
                }
            }
        }

        // re-translate the strings when the language changes
        Connections {
            target: application
            onLanguageChanged: {
                accessKeyPad.instructions = qsTr("ENTER PASSCODE") + application.emptyString
            }
        }
    }

    MouseArea {
        // Capture clicks outside keypad to dismiss it
        anchors.fill: accessKeyPad
        enabled: accessKeyPad.visible
        z: accessKeyPad.z - 1
        onClicked: {}
    }

    MouseArea {
        // Capture clicks outside keypad to dismiss it
        anchors.fill: parent
        enabled: accessKeyPad.visible
        z: accessKeyPad.z - 2
        onClicked: accessKeyPad.close()
    }

    LogUploader {
        id: logUploader
    }

    LogUploadPrompt {
        id: logUploadPrompt
    }


    Options {
        id: options
        objectName: "options"
        onContactHRPriorityChanged: {
            heartRateManager.setContactPriority(contactHRPriority)
        }
        onLcdBrightnessChanged: main.lcdBrightness = lcdBrightness
        onCabEnabledChanged: {
            nucleus.cabSetEnable(cabEnabled)
        }
        onIgnoreFaultsChanged: {
            if (options.ignoreFaults) {
                popupManager.reset()
            }
        }
        onLoggingEnabledChanged: {
            logger.enabled = loggingEnabled
        }
        onDisableBuzzerChanged: {
            buzzer.setBuzzerEnabled(!options.disableBuzzer)
        }
    }

    property int lcdBrightness: options.lcdBrightness
    onLcdBrightnessChanged: {
        main.setLcdBrightness(lcdBrightness)
    }

    function setLcdBrightness(percent) {
         lcdBrightnessProcess.setBrightness(percent)
    }

    Process {
        id: lcdBrightnessProcess
        program: "scripts/set_brightness.sh"

        function setBrightness(percent) {
            lcdBrightnessProcess.arguments = [ (percent*6000/100).toFixed(0) ]
            lcdBrightnessProcess.start()
        }
    }

    Binding {
        target: application
        property: "showCursor" ; value: options.showCursor
    }

    // Idle timer
    // Time out interval is set by state transition table below.
    // The timer is reset by a screen touch (event sent by application object.)
    // Timeout action is defined in onTriggered event.  Many screens return to HomeScreen state.
    Timer {
        id: idle_timer

        repeat: true
        running: true
        interval: 10000
        onRunningChanged: idleSecondsTimer.reset()
        onIntervalChanged: idleSecondsTimer.reset()

        function secondsLeft() {
            var sec = (interval / 1000) - idleSecondsTimer.seconds
            if(sec < 0 )
                return 0
            else
                return sec
        }

        onTriggered: {
            switch(main.state) {
                case "HomeState":{  // idle timeout on home-screen goes to screen-saver or sleep
                    if(options.screenSaveTimeout) {  // screensaver enabled
                        main.state = "ScreenSaverState"
                        screensaver.start()
                    } // otherwise, go to sleep mode if enabled
                    else if(options.sleepModeTimeout) {
                        main.state = "SleepModeState"
                    }

                    if (remoteAccess.isEnabled) {
                        // Exit screensaver immediately
                        screensaver.stop()
                    }

                    // disconnect bluetooth device if connected
                    if (bluetoothAdapter.connected) {
                        bluetoothAdapter.disconnect()
                    }
                    if (bluetoothAdapter.connectedHeadset)
                        bluetoothAdapter.disconnectHeadset()

                    if(!options.screenSaveTimeout && !options.sleepModeTimeout && options.erpEnabled){
                        erpSleep()
                    }
                    break;
                }
                case "LoginState":
                case "RegisterState": {
                    main.state = "HomeState";
                    break;
                }
                case "WorkoutSummaryState": {
                    workout_summary.close() // This will make sure the bt advertise dialog closes on timeou
                    break
                }
                case "MenuState":
                case "SetupState":
                case "LanguageMenuState": {
                    // disconnect bluetooth device if connected
                    if (bluetoothAdapter.connected) {
                        bluetoothAdapter.disconnect()
                    }
                    if (bluetoothAdapter.connectedHeadset)
                        bluetoothAdapter.disconnectHeadset()
                    eGym.reset()
                    main.state = "HomeState";
                    break;
                }
                case "ScreenSaverState": {
                    if (options.sleepModeTimeout) {
                        main.state = "SleepModeState"
                    } else if(options.erpEnabled){
                        erpSleep()
                    }
                    break;
                }
                case "DiagnosticState":
                case "SetupWizardState":
                {
                    break;
                }
                case "SleepModeState":
                {
                    if(options.erpEnabled){
                        erpSleep()
                    }

                    break;
                }
            } // end switch
        } // end onTriggered
    }

    Timer {
        id: dim_timer
        interval: options.dimDisplayTimeout * 1000
        running: false
        repeat: false
        onTriggered: {
            if (main.state != "HomeState" && main.state != "ScreenSaverState")
                return

            // Set to 40% of brightness setting
            var brightnessLevel = parseInt(0.40 * options.lcdBrightness)
            main.lcdBrightness = brightnessLevel
            console.log("Dim timer fired. Set brightness to", brightnessLevel)
        }
    }

    Timer {
        id:idleSecondsTimer

        property int seconds: 0

        function reset() {
            idleSecondsTimer.seconds = 0
            idleSecondsTimer.restart()
        }

        interval: 1000
        running: true
        onRunningChanged: seconds = 0
        repeat: true
        onTriggered: {
            if( main.state !== "RunState" && main.state !== "DiagnosticState" ){
                if(machinecontrol.isBeltMoving){
//                    console.debug("BELT STATUS ON INSPECTION T-A "+machinecontrol.treadmillSpeedTarget+"-"+machinecontrol.treadmillSpeedActual)
                    machinecontrol.treadmillSpeedTarget = 0
                    console.log("MOVING BELT DETECTED, STOPPING")
                }
                if(isTreadmill){
//                    console.debug("BELT STATUS ON INSPECTION T-A "+machinecontrol.treadmillSpeedTarget+"-"+machinecontrol.treadmillSpeedActual)
                    if(machinecontrol.treadmillSpeedActual != 0  || machinecontrol.treadmillSpeedTarget != 0){
                        machinecontrol.treadmillSpeedTarget = 0
                        console.log("NON-ZERO TREAD SPEED DETECTED, STOPPING")
                    }
                } else if (isStepmill){
//                    console.debug("STEP STATUS ON INSPECTION T-A "+machinecontrol.stepmillSpeedTarget+"-"+machinecontrol.stepmillSpeedActual)
                    if(machinecontrol.stepmillSpeedActual != 0 || machinecontrol.stepmillSpeedTarget != 0){
                        machinecontrol.stepmillSpeedTarget = 0;
                        console.log("NON-ZERO STEPMILL SPEED DETECTED, STOPPING")
                    }
                }
            }

            seconds++
        }
    }

    Timer {
        id: playBluetooth
        interval: 1000; repeat: false; running: false

        onTriggered: {
            if (bluetoothAdapter.currentConnectedDevice) {
                if (bluetoothAdapter.currentConnectedDevice.player) {
                    bluetoothAdapter.currentConnectedDevice.player.play()
                }
            }
        }
    }

    TrueBlueAdapter{
        id: bluetoothAdapter

        onConnectedChanged: {
            if (connected) {
                btMute.setMute(0)
                audioManager.setVolume(audioManager.volumeDefault)
                playBluetooth.start()
            }
        }

        onConnectedHeadsetChanged: {
            if (tv.isIpodInput) {
                // restart tv if iPod is currently playing, so that it will start streaming the ipod
                // input to pulseaudio, which is required for the headset to function
                tv.mrl = ""
                tv.tuneChannelID("IPODINPUT")
            }
        }
    }
    // Expire bluetooth connection after 2 minutes without starting a workout
    Timer {
        id: bluetoothSessionTimer
        running: (main.state !== "RunState") && bluetoothAdapter.connected
        interval: 2 * 60 * 1000
        onTriggered: {
            ipod_window.close()
            if (bluetoothAdapter.connected)
                bluetoothAdapter.disconnect()
            if (bluetoothAdapter.connectedHeadset)
                bluetoothAdapter.disconnectHeadset()
        }
    }

    // Expire bluetooth Heartrate connection after 5 minutes without starting a workout
    Timer {
        id: bluetoothLESessionTimer
        running: (main.state !== "RunState") && bluetoothLEAdapter.hrConnected
        interval: 30 * 1000
        onTriggered: {
            if (bluetoothLEAdapter.hrConnected)
                console.debug("main HR timeout")
                bluetoothLEAdapter.disconnect()
        }
    }

    // Expire GEM->App connection after 1 minute without starting a workout
    Timer {
        id: gemAppTimer
        running: main.state !== "RunState" && gem.connected === 1
        interval: (gem.appleWatchConnected ? 3 : 1) * 60 * 1000
        onTriggered: {
            main.showMediaBrowser = false
            console.log("GEM connection timeout without starting workout. Resetting...")
            gem.reset()
        }
    }
    Timer{
        id:changeTvState
        interval: 5
        repeat: false
        onTriggered:
        {
      if(workoutScreen.state==="DataExpanded" || workoutScreen.state==="ToolsVisible" || workoutScreen.stopPrompt.visible===true || appleWatchConnect_popup.visible===true)
        tv.on=false
       else
       tv.on=true
     changeTvState.stop()
        }
    }

    // Expire eGym RFID connection after 1 minute without starting a workout
    Timer {
        id: eGymConnectionTimer
        running: main.state !== "RunState" && eGym.connected === 1
        interval: 60 * 1000
        onTriggered: {
            console.log("eGym connection timedout without starting workout. Resetting...")
            eGym.reset()
        }
    }

    ModalDialog {
        id: appleWatchConnect_popup
        visible: main.showAppleWatchConnectsInfo && (main.state == "HomeState" || main.state == "RunState")
        dismissAnywhere: true
        dismissable: true
        countDown: 20
        z: 999

        onVisibleChanged: {
            if(visible) {
                if(workoutScreen.showingTV)
                {
                    tv.on=false
                }

                startTimeout(20)
            }
            else if(workoutScreen.showingTV)
                changeTvState.start()
        }

        onTimedOut: {
            main.showAppleWatchConnectsInfo = false
        }

        CloseButton {
            id: closeButton

            z: parent.z + 1
            anchors {
                right: parent.right ; rightMargin: 80 * main.hScaleFactor
                top: parent.top ; topMargin: 40 * main.vScaleFactor
            }
            onClicked: main.showAppleWatchConnectsInfo = false
        }

        Column {
            z: parent.z + 1
            anchors.fill: parent
            anchors.margins: 10 * main.scaleFactor
            spacing: 10 * main.vScaleFactor

            ScalableImage {
                id: nfc_icon
                width: parent.width
                height: 60 * main.vScaleFactor
                fillMode: Image.PreserveAspectFit
                sourceName: "Apple/WatchNFCArrowIcon"
            }

            TextLabel {
                text: qsTr("How It Works")
                font.pixelSize: 25 * main.vScaleFactor
                width: parent.width
                horizontalAlignment: Text.AlignHCenter
            }

            TextLabel {
                text: qsTr("Hold your Apple Watch near the reader below and confirm the connection on your Apple Watch to automatically connect to this ")
                      + machineinfo.productTypeString + ". "
                      + qsTr("You can also open the Workout app on you Apple Watch then  hold near the reader to get started.")
                font.pixelSize: 20 * main.vScaleFactor
                isHairline: true
                wrapMode: Text.WordWrap
                width: 0.9 * parent.width
                horizontalAlignment: Text.AlignHCenter
                anchors.horizontalCenter: parent.horizontalCenter
            }

            ScalableImage {
                id: appleConnect_icon
                width: parent.width
                height: Math.max(40 * main.vScaleFactor, 40) // 40px minimum height required by Apple
                fillMode: Image.PreserveAspectFit
                sourceName: "Apple/Apple_Watch_Connects_Marketing_Badge_RGB_lg_121019"
                useSVG: true
            }

        }
    }

    Rectangle {
        id: touchSplat

        property int size: 10
        opacity: 0.0 // Hide until first touch
        width: size
        height: width
        radius: width
        z: 100

        transform: Translate { x:-touchSplat.size/2 ; y: -touchSplat.size/2 }

        color: main.colorUIBorder //"#0099FF"

        ParallelAnimation {
            id:splat
            NumberAnimation { target: touchSplat; properties: "size" ; from: 15 ; to: 0; duration: 500; easing.type: Easing.InOutQuad }
            NumberAnimation { target: touchSplat; properties: "opacity" ; from: 0.5 ; to: 0; duration: 500; easing.type: Easing.InOutQuad }
        }
    }

    Connections {
        // reset the idle timer when there's a screen touch
        target: application
        onScreenTouched: {
            splat.stop() // Stop in-progress splat
            idle_timer.restart()
            lang_timer.restart()
            touchSplat.x = x
            touchSplat.y = y
            touchSplat.opacity = 0.5
            touchSplat.size = 15
            if(main.state == "SleepModeState")
                main.state = "HomeState"
            else if (main.state === "HomeState") {
                main.lcdBrightness = options.lcdBrightness
                if (!options.screenSaveTimeout && options.dimDisplayTimeout)
                    dim_timer.restart()
            }
        }
        onMouseReleased: {
            touchSplat.x = x
            touchSplat.y = y
            splat.restart()
        }
    }

    Connections {
        target: machineinfo
        onDetectionComplete: {
            main.lowerSystem = lowerSystem
            if(lowerSystem === "NO_LOWERSYSTEM" && !options.ignoreFaults) {
                blockUseMessage.hide()
                errorManager.errorReceived(ErrorManager.ERRORNO_CONSOLE_USBCONNECTIONTERMINATED,statisticManager.totalDistance)
            }

            if (main.lowerSystem === "NUCLEUS" || main.lowerSystem === "NUCBOOT") {
                if (!options.setupWizardCompleted) {
                    main.state = "SetupWizardState"
                }
            }

        }

        onModelStringChanged: {
            var maxSpeed = main.isTreadmill ? machineinfo.treadmillSpeedMaxMph : main.isStepmill ? machineinfo.stepmillSpeedMaxSpm : 0
            var maxLevel = 20
            console.log("Setting EGym machine info: " + main.serialNumber + ", " + machineinfo.productTypeString)
            eGym.setMachineInfo(main.serialNumber, machineinfo.productTypeString, machineinfo.treadmillInclineMinPercentGrade, machineinfo.treadmillInclineMaxPercentGrade, maxSpeed, maxLevel)
        }
    }

    onStateChanged: {
        print("Main: UI state = " + main.state)

        if(state === "HomeState"){
            videoRunner.start()
            if(machineinfo.productType === WorkoutProgram.ELLIPTICAL) {
                machinecontrol.workload = 0
            }
            initializationRefreshTimer.start();
            beltHasHadTimeToAccelerate = false;
        }
    }

    Process {
        id: paVolControl
        program: "scripts/true-pa-volume-control.sh"
        property bool isMute
        property int _currentVolSetting: 0

        function setVolume(vol_setting){

            /* Only run the script if the vol setting is different */
            if(_currentVolSetting === vol_setting) {
                return
            }

            _currentVolSetting = vol_setting

            var vol = (6554 * vol_setting) + 6550
            var volStr = vol.toString();
            paVolControl.arguments = ["vol", volStr]
            paVolControl.start()
        }

        function mute() {
            paVolControl.arguments = ["mute"]
            paVolControl.start()
        }

        function info() {
            paVolControl.arguments = ["info"]
            paVolControl.start()
        }

        function parseOutput(str){
            var strList = str.split(" ")
            if(strList[0] === "info"){
                var volumeInt = parseInt(strList[2])
                var isMuteStr = strList[3]

                if(isMuteStr === "yes") {
                    isMute = true
                } else if(isMuteStr === "no") {
                    isMute = false
                }

                console.debug(volumeInt + " " + isMuteStr)
            }
        }

        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim()
            console.log("output from paVolControl:",data)
            parseOutput(data)
        }
    }

    Timer {
        id: check_nucleus_faults_timer
        interval:24*3600*1000
        running: true
        repeat: true
        onTriggered: {
            // Retrieve Nucleus error logs
            console.log("Retrieving Nucleus fault log")
            errorManager.retrieveFaultLogs()
        }
    }

    states: [
        State {
            name: "HomeState"
            PropertyChanges {target: home_screen_loader ; opacity: 1 }
            PropertyChanges {target: screen_lock_button ; opacity: 1 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader; opacity: 0 }
            PropertyChanges {target: diagnosticsMain ; opacity: 0 ; z: 0}
            PropertyChanges {target: mainProgItem ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false; isNewWorkoutSelected: false }
            // set timeout for screen-saver or dim mode or sleep mode
            PropertyChanges {target: idle_timer ;
                interval: {
                    if(options.screenSaveTimeout)
                        return options.screenSaveTimeout*1000
                    else if(options.sleepModeTimeout)
                        return options.sleepModeTimeout*1000
                    else if(options.erpEnabled && options.erpTimeout)
                        return options.erpTimeout*1000
                    else
                        return 0
                }
            }
            PropertyChanges { target: dim_timer; running: options.dimDisplayTimeout > 0 && !options.screenSaveTimeout }
            PropertyChanges {target: workout_summary ; dialogVisible: false }
            PropertyChanges {target: main; showMediaBrowser: false}
            PropertyChanges {target: gem; connectionAllowed: true}
        },
        State {
            name: "LoginState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader; opacity: 0 }
            PropertyChanges {target: diagnosticsMain ; opacity: 0 ; z: 0}
            PropertyChanges {target: mainProgItem ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: 60000}
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; showMediaBrowser: false}
            PropertyChanges {target: gem; connectionAllowed: false}
        },
        State {
            name: "RegisterState"
            PropertyChanges { target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader; opacity: 0 }
            PropertyChanges {target: diagnosticsMain ; opacity: 0 ; z: 0}
            PropertyChanges {target: mainProgItem ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: 180000}
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; showMediaBrowser: false}
            PropertyChanges {target: gem; connectionAllowed: false}
        },
        State {
            name: "LanguageMenuState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 1 }
            PropertyChanges {target: workout_menu_loader; opacity: 0 }
            PropertyChanges {target: diagnosticsMain ; opacity: 0 ; z: 0}
            PropertyChanges {target: mainProgItem ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: options.workoutMenuTimeout * 1000}
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; showMediaBrowser: false}
            PropertyChanges {target: gem; connectionAllowed: false}
        },
        State {
            name: "MenuState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 1 }
            PropertyChanges {target: diagnosticsMain ; opacity: 0 ; z: 0}
            PropertyChanges {target: mainProgItem ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: options.workoutMenuTimeout * 1000}
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; showMediaBrowser: false}
            PropertyChanges {target: gem; connectionAllowed: true}
        },
        State {
            name: "LoadState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: diagnosticsMain ; opacity: 0 ; z: 0}
            PropertyChanges {target: mainProgItem ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: loadingProgramNotice ; opacity: 1 }
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: main; showMediaBrowser: false}
            PropertyChanges {target: gem; connectionAllowed: true}
        },
        State {
            name: "SetupState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: diagnosticsMain ; opacity: 0 ; z: 0}
            PropertyChanges {target: mainProgItem ; opacity: 1 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: options.workoutSetupTimeout * 1000}
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; showMediaBrowser: false}
            PropertyChanges {target: gem; connectionAllowed: true}
        },
        State {
            name: "RunState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: diagnosticsMain ; opacity: 0 ; z: 0}
            PropertyChanges {target: mainProgItem ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 1 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            // turn off idle timer - run screen will have its own
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: 0}
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; showMediaBrowser: false}
            PropertyChanges {target: gem; connectionAllowed: true}
        },
        State {
            name: "WorkoutSummaryState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader; opacity: 0 }
            PropertyChanges {target: diagnosticsMain ; opacity: 0 ; z: 0}
            PropertyChanges {target: mainProgItem ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 1 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: options.workoutSummaryTimeout * 1000 }
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; showMediaBrowser: false}
            PropertyChanges {target: gem; connectionAllowed: false}
        },
        State {
            name: "DiagnosticState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: diagnosticsMain ; opacity: 1 ; z: 50}
            PropertyChanges {target: mainProgItem ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0  }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: 300000}
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; showMediaBrowser: false}
            PropertyChanges {target: gem; connectionAllowed: true}
        },
        State {
            name: "SetupWizardState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: diagnosticsMain ; opacity: 0 ; z: 0}
            PropertyChanges {target: mainProgItem ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0  }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 1 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: 0}
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; showMediaBrowser: false}
            PropertyChanges {target: gem; connectionAllowed: false}
        },
        State {
            name: "ScreenSaverState"
             // set timeout until moving from screensaver to dim-mode, sleep-mode, or power-down
            PropertyChanges {target: idle_timer ; interval: options.sleepModeTimeout ? options.sleepModeTimeout * 1000 : options.erpTimeout * 1000 }
            PropertyChanges { target: dim_timer; running: options.dimDisplayTimeout > 0 }
            PropertyChanges {target: main; showMediaBrowser: false}
            PropertyChanges {target: gem; connectionAllowed: true}
        },
        State {
            name: "SleepModeState"
             // set timeout until moving from screensaver to power-down/sleep mode
            PropertyChanges {target: main; lcdBrightness: 0; restoreEntryValues: true ; explicit: true }
            PropertyChanges {target: idle_timer ; interval: options.erpTimeout * 1000 }
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; showMediaBrowser: false}
            PropertyChanges {target: gem; connectionAllowed: false}
        }

    ] // end states
    transitions: [
        Transition {
                NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
        },

        Transition {
            from: "HomeState"
            to: "MenuState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    workout_menu.reset()
                }
            }
        },
        Transition {
            from: "MenuState"
            to: "HomeState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    application.selectLanguage(options.defaultLanguage) // reset language after workout
                    home_screen.updateLanguageIcon(options.defaultLanguage)
                    clearQLists()
                    main.quickStartLoading = false
                }
            }
        },
        Transition {
            from: "SetupState"
            to: "HomeState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    application.selectLanguage(options.defaultLanguage) // reset language after workout
                    home_screen.updateLanguageIcon(options.defaultLanguage)
                    clearQLists()
                    main.quickStartLoading = false
                }
            }
        },
        Transition {
            from: "DiagnosticState"
            to: "HomeState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    application.selectLanguage(options.defaultLanguage) // reset language after workout
                    home_screen.updateLanguageIcon(options.defaultLanguage)
                    clearQLists()
                    main.quickStartLoading = false
                    syncDisk.start()
                    if(machinecontrol.treadmillSpeedActual > 0 && main.state != "RunState"){
                        machinecontrol.treadmillSpeedTarget = 0
                    }
                }
            }
        },
        Transition {
            from: "HomeState"
            to: "DiagnosticState"
            ScriptAction {
                script: {
                    videoRunner.pause()
                    softwareUpdatePrompt.hide()
                }
            }
        },
        Transition {
            from: "HomeState"
            to: "LoginState"
//            NumberAnimation { target: signin_window; properties: "opacity"; easing.type: Easing.Linear; duration: 100 }
            NumberAnimation { target: home_screen; properties: "opacity"; easing.type: Easing.Linear; duration: 100 }
        },
        Transition {
            from: "WorkoutSummaryState"
            to: "HomeState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    workout_summary.saveHasHappened = false
                    application.selectLanguage(options.defaultLanguage) // reset language after workout
                    home_screen.updateLanguageIcon(options.defaultLanguage)
                    clearQLists()
                    main.quickStartLoading = false
                    gem.appleWatchConnected = false
                    if(machinecontrol.treadmillSpeedActual > 0){
                        machinecontrol.treadmillSpeedTarget = 0
                    }
                }
            }
        },
        Transition {
            to: "RunState"
            ScriptAction {
                script: {
                    videoRunner.pause()
                }
            }
        }
    ]
}
